#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2011 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/billing/BillingDemo.pm, hpc.cloud.billing, hpcc_rbei, rbeis002a 1.14 12/15/11 04:19:40";

package BillingDemo;
use strict;
use warnings;
require "Billing.pm";

sub new {
    my $type = shift;
    my $this = Billing->new();

    $this->{'Version'} = '0.1';
    $this->{'Description'} = 'HPC Cloud Billing system demo';

    bless $this, $type;
    return $this;
}

use DBI;

use Utils::Logger;

%::CONFIG = ();

&loadConfigFile;
my $DBH = &getDBConn;

sub loadConfigFile {
	my $cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root \"/opt/ibm/hpccloud/cloudbin/utils/getConffromcfg\"";
    my @out;
	eval {
	   @out = `$cmd 2>&1`;
	};
	if($@) {
	   print("@out\n");
	   return ;
	}
	my $rv = $? >> 8;
	if ($rv) {
	   print("@out\n");
	   return ;
	}
	if (scalar(@out) == 0){
	   print ("Failed to get configuration from hpccloud.cfg.\n");
	   return ;
	}
	foreach (@out){
	   if (/(\S+)\s*=\s*(\S+)/) {
		   $::CONFIG{$1} = $2;
		}
	}
}

sub getDBConn {
    #my $dsn = 'dbi:mysql:test:localhost:3306';
    my $dsn = $::CONFIG{'DATA_SOURCE'} ? $::CONFIG{'DATA_SOURCE'} : 'dbi:mysql:database=HCDB;host=127.0.0.1';

    # set the user and password
    my $user = $::CONFIG{'DB_USER'} ? $::CONFIG{'DB_USER'} : 'root';
    my $pass = $::CONFIG{'DB_PASSWORD'} ? $::CONFIG{'DB_PASSWORD'} : '';

    my $dbh = DBI->connect($dsn, $user, $pass, {AutoCommit => 0, RaiseError => 0, PrintError => 0});
    # my $dbh = DBI->connect($dsn, $user, $pass, {AutoCommit => 0, RaiseError => 0, PrintError => 1});

    unless ($dbh) {
        #log the error $DBI::errstr
	    Logger->get()->error('Billing', 'NA', "Failed to connect to DB. $DBI::errstr");
        #exit(1);
    }

    return $dbh;
}


sub isReady {
    if ($DBH) {
        return 1;
    }
    return 0;
}


sub getAllAccountInfoFromDB {
    
    my $st = $DBH->prepare("select * from account");
    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    return $st->fetchall_hashref('userName');
}

sub getAllMachineInfoFromDB {

    my $st = $DBH->prepare("select * from machine");
    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    return $st->fetchall_hashref('machineName');
}

sub getProvisionInfoFromDB {

    my ($class, $prov) = @_;
    my $st = $DBH->prepare("select * from provision where provisionId=\"$prov\"");
    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    if (my $result = $st->fetchall_hashref('provisionId')) {
        return $result->{$prov};
    } else {
        return undef;
    }
}


sub getUserProvisionInfoFromDB {

    my ($class, $user) = @_;

    my $st = $DBH->prepare("select * from provision where userName='$user'");
    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    return $st->fetchall_hashref('provisionId');
}

sub getUserJobInfoFromDB {

    my ($class, $user) = @_;

    my $st = $DBH->prepare("select * from job where userName='$user'");
    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    return $st->fetchall_hashref('jobId');
}


sub addUserProvisionInfoIntoDB {
    my ($user, $pid, $cost, $balance) = @_;
    my $now = time;
    my $st = $DBH->prepare("delete from provision where provisionId='$pid'");

    my $rv = $st->execute;

    $st = $DBH->prepare("insert into provision set provisionId='$pid', userName='$user', cost=$cost, requestTime=$now");

    $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    unless (updateAccountInfoIntoDB($user, $balance-$cost)) {
        $DBH->rollback();
    }
	$DBH->commit;
    return 1;
}

sub addUserJobInfoIntoDB {
    my ($user, $jid, $cost, $balance) = @_;
    my $now = time;
    my $st = $DBH->prepare("insert into job set jobId='$jid', userName='$user', cost=$cost, requestTime=$now");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    unless (updateAccountInfoIntoDB($user, $balance-$cost)) {
        $DBH->rollback();
    }
	$DBH->commit;
    return 1;
}

sub updateAccountInfoIntoDB {
    my ($user, $balance) = @_;
    my $st = $DBH->prepare("update account set balance=$balance where userName='$user'");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }
    $DBH->commit;

    return 1;
}
sub removeAccountInfoFromDB {
    my ($user) = @_;
    my $st = $DBH->prepare("delete from account where userName='$user'");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }
    $DBH->commit;

    return 1;
}

sub addAccountInfoIntoDB {
    my ($user, $init) = @_;
    my $st = $DBH->prepare("insert into account set userName='$user', init=$init, balance=$init");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    $DBH->commit;
    return 1;
}

sub addMachineInfoIntoDB {
    my ($machine, $price) = @_;
    my $st = $DBH->prepare("insert into machine set machineName='$machine', price=$price");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    $DBH->commit;
    return 1;
}
sub modMachineInfoIntoDB {
    my ($machine, $price) = @_;
    my $st = $DBH->prepare("update machine set price=$price where machineName='$machine'");

    my $rv = $st->execute;

    if (!$rv) {
        #log the error $DBI::errstr
        return $rv;
    }

    $DBH->commit;
    return 1;
}

sub getUserAccount {
    my ($class, $user, $create) = @_;

    my $all_account = getAllAccountInfoFromDB();
    unless (defined($all_account->{$user})) {
        unless (defined($create)) {
            return undef;
        }

        my $default = 5000000;
        addAccountInfoIntoDB($user, $default);

        $all_account = getAllAccountInfoFromDB();
    }
    my $u = $all_account->{$user};
    return $u->{'balance'};
}
sub setUserAccount {
    my ($class, $user, $price) = @_;
	if(!defined($user) or !defined($price)){
		return 0;
	}
    my $all_account = getAllAccountInfoFromDB();
	my $rtv = 1;
    unless (defined($all_account->{$user})) {
		if($price >= 0){
        	$rtv = addAccountInfoIntoDB($user, $price);
		}
	}else{
		if($price >= 0){
        	$rtv = updateAccountInfoIntoDB($user, $price);
		}else{
			$rtv = removeAccountInfoFromDB($user);
		}
    }
	return $rtv;
}

sub getMachinePrice{
    my ($class, $machine, $create) = @_;
    my $all_machine = getAllMachineInfoFromDB();
    unless (defined($all_machine->{$machine})) {
        unless (defined($create)) {
            return undef;
        }

        my $default = 400;
		if($machine eq "IBM_KVM"){
			$default = 50;
		}
        addMachineInfoIntoDB($machine, $default);

        $all_machine = getAllMachineInfoFromDB();
    }

    my $m = $all_machine->{$machine};
    return $m->{'price'};
}
sub setMachinePrice{
    my ($class, $machine, $price) = @_;
    my $all_machine = getAllMachineInfoFromDB();
	my $rtv;
    unless (defined($all_machine->{$machine})) {
        $rtv = addMachineInfoIntoDB($machine, $price);
	}else{
        $rtv = modMachineInfoIntoDB($machine, $price);
    }
	return $rtv;
}


sub evaluteJobRequest {
     return 200;
}

sub evaluteProvisionRequest {
     my ($p) = @_;
     if ($p) {
        my $duration  = $p->{'duration'};
        my @machines;
        if($p->{'select_method'} == 0){#machList mode
             @machines  =@{$p->{'machList'}};
	 	}
        else{#mach number mode
             my $machineType = $p->{'requestMachineType'};
             my $machineNumber = $p->{'requestMachineNumber'};
             if ($machineNumber) {
             	for (my $i =0; $i < $machineNumber; $i++) {
                	push(@machines, $machineType);
            	}
             }
        }
        my $sum = 0;
        foreach my $mach(@machines) {
			if(defined($p->{'hypv_flag'}) and ($p->{'hypv_flag'} == -1)){#vm
	            $sum += BillingDemo->getMachinePrice("IBM_KVM",1)*$p->{'request_machine_cpu'};
	        }else{
				$sum += BillingDemo->getMachinePrice($mach, 1);
			}
		}
        $sum *= $duration;
        return $sum;
     }
     return undef;
}


sub testProvisionRequest {
    my ($self, $p, $flag) = @_;
    my $balance = BillingDemo->getUserAccount($p->{'provisionOwner'}, 1);
    my $cost = evaluteProvisionRequest($p);
    my $changed = 0;
	if(defined($flag) and ($flag eq 'Modify')){
		my $prov_info = BillingDemo->getProvisionInfoFromDB($p->{'provisionId'});
		if($prov_info){
			$balance +=$prov_info->{'cost'};			
		}
		if (exists($p->{'post_duration'}) and exists($p->{'post_requestMachineNumber'})){
			my $post_p;
			$post_p->{'duration'} = $p->{'post_duration'};
			$post_p->{'requestMachineNumber'} = $p->{'post_requestMachineNumber'};
			$post_p->{'select_method'} = $p->{'select_method'};
			$post_p->{'requestMachineType'} = $p->{'requestMachineType'};
			$post_p->{'hypv_flag'} = $p->{'hypv_flag'};
			if(exists($p->{'request_machine_cpu'})){
				$post_p->{'request_machine_cpu'} =  $p->{'request_machine_cpu'};
			}
			my $post_cost = evaluteProvisionRequest($post_p);
			$cost += $post_cost;
		}			
		if(defined($prov_info)){
			$changed = $cost - $prov_info->{'cost'};
		}else{
			$changed = 0;
		}
	}
    my $result = ($balance >= $cost);
	Logger->get()->debug('Billing', 'testProvisionRequest', "Result: $result, Cost: $cost, Changed: $changed.");
    return ($result, $cost, $changed);
}

sub testJobRequest {
    my ($self, $j) = @_;
    my $balance = BillingDemo->getUserAccount($j->{'jobOwner'}, 1);
    my $cost = evaluteJobRequest($j);

    my $result = ($balance >= $cost);
    return ($result, $cost);
}

sub applyProvisionRequest {
	my ($self, $p, $flag, $changed_info) = @_;
#to fit external interface and make the min code change in current file, we translate the input $p to internal structure
	my $request;
	$request->{'provisionOwner'} = $p->{'request.owner'};
	$request->{'provisionId'} = $p->{'request.name'};
	
	my $balance = BillingDemo->getUserAccount($request->{'provisionOwner'}, 1);
	my $cost = 0;
	if($flag eq 'Create'){
		foreach (@{$p->{'machList'}}){
			push(@{$request->{'machList'}}, $_->{'machine.name'});
		}
		$request->{'duration'} = ($p->{'request.expiry_time'} - $p->{'request.start_time'})/60;#duration is per minute;
		$request->{'requestMachineType'} = $p->{'request.machine_type'};
		$request->{'requestMachineNumber'} = $p->{'request.machine_num'};
		$request->{'hypv_flag'} = $p->{'request.hypv_flag'};
		$request->{'request_machine_cpu'} = $p->{'request.machine_cpu'};
		$request->{'select_method'} = $p->{'request.select_method'};
		$cost = evaluteProvisionRequest($request, $flag);
	}
	else{
		if(!defined($changed_info)){#modify or cancel but no changed_info, we do not update account
			return (1,0);
		}
		my $prov_info = BillingDemo->getProvisionInfoFromDB($request->{'provisionId'});
		if(defined($prov_info)){
			$balance +=$prov_info->{'cost'};
		}
		$cost = $changed_info->{'cost'};
	}
	my $result = ($balance >= $cost);
#Permit moify or cancel the request in any times so, billing also support
#If modify or cacnel, the billing shall cacualte real cost and return un used back
	if (($result > 0) &&
			(($p->{'request.status'} ne 'Failed')&&
			 ($p->{'request.status'} ne 'FAIL')&&
			 ($p->{'request.status'} ne 'Expired'))) {
		addUserProvisionInfoIntoDB($request->{'provisionOwner'}, $request->{'provisionId'}, $cost, $balance);
	}
	return ($result, $cost);
}

sub applyJobRequest {
    my ($self, $j) = @_;
    my $balance = BillingDemo->getUserAccount($j->{'jobOwner'}, 1);
    my $cost = evaluteJobRequest($j);

    my $result = ($balance > $cost);
    if ($result > 0) {
        addUserJobInfoIntoDB($j->{'jobOwner'}, $j->{'jobId'}, $cost, $balance);
    }

    return ($result, $cost);
}

1;
