#!/usr/bin/perl -w
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/common/CloudUtils.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.120 1/31/12 06:18:40";

package CloudUtils;

use strict;
use warnings;
use DBI;

use DB::UserDAO;
use DB::ImageDAO;
use DB::ConfigDAO;
use DB::LogDAO;
use DB::EventDAO;
use DB::MachineDAO;
use DB::ProvisionDAO;
use DB::GeneralDAO;
use Utils::Logger;
use EnergyUtils;
use GeneralDefine;
use Utils::xCatUtils;
use BillingDemo;

&loadConfigFile;

$::DBH = &getDBConn;

$::LDAPHost = $::CONFIG{'LDAP_HOST'} ? $::CONFIG{'LDAP_HOST'} : '127.0.0.1';
$::LDAPADMPASSWD = $::CONFIG{'LDAP_ADMIN_PASSWORD'};

sub loadConfigFile {
	my $cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root \"/opt/ibm/hpccloud/cloudbin/utils/getConffromcfg\"";
	my @out;
	eval {
	   @out = `$cmd 2>&1`;
	};
	if($@) {
		print("@out\n");
		return ;
	}
	my $rv = $? >> 8;
	if ($rv) {
		print("@out\n");
		return ;
	}
	if (scalar(@out) == 0){
		print ("Failed to get configuration from hpccloud.cfg.\n");
		return ;
	}
	foreach (@out){
		if (/(\S+)\s*=\s*(\S+)/) {
		   $::CONFIG{$1} = $2;
		}
	}
}

sub getDBConn {
	#my $dsn = 'dbi:mysql:test:localhost:3306';
	my $dsn = $::CONFIG{'DATA_SOURCE'} ? $::CONFIG{'DATA_SOURCE'} : 'dbi:mysql:database=HCDB;host=127.0.0.1';

	# set the user and password
	my $user = $::CONFIG{'DB_USER'} ? $::CONFIG{'DB_USER'} : 'root';
	my $pass = $::CONFIG{'DB_PASSWORD'} ? $::CONFIG{'DB_PASSWORD'} : '';

	my $dbh = DBI->connect($dsn, $user, $pass, {AutoCommit => 0, RaiseError => 1, PrintError => 0});
	# my $dbh = DBI->connect($dsn, $user, $pass, {AutoCommit => 0, RaiseError => 0, PrintError => 1});

	unless ($dbh) {
		#log the error $DBI::errstr
		Logger->get()->error('DB', 'NA', "Failed to connect to DB. $DBI::errstr");
		#exit(1);
	}

	return $dbh;
}

sub isReady {
	if ($::DBH) {
		return 1;
	}
	return 0;
}

# the args $role & $comments are no used.
sub addMach {
	my ($class, $name, $role, $mode, $group, $arch, $type, $status, $pstatus, $os, $ip, $comments, $hypv_flag) = @_;

	my $md = MachineDAO->new();

	my $args = {
		'provision_mode' => $mode,
		'arch' => $arch,
		'type' => $type,
		'status' => $status,
		'power_state' => $pstatus,
		'default_os' => $os,
		'ip' => $ip,
		'hypv_flag' => $hypv_flag,
	};

	my ($rc, $param) = $md->insertMachine($name, $args);
	if ($rc != 0) {
		return 0;
	}

	$md->commit();

	return 1;
}

sub getAllMachineGroups {
	my $md = MachineDAO->new();
	my ($rc, $param) = $md->queryAllMachineGroup();
	if ($rc == 0) {
		return (1, $param);
	} else {
		Logger->get()->warning('MachineGroup', 'all', "Failed to query all machine.");
		return (0, $param);
	}
}

sub getMachineGroup {
	my ($class, $mg) = @_;
	my $md = MachineDAO->new();
	my ($rc, $param) = $md->queryMachineGroup({'name'=>$mg});
	if ($rc == 0) {
		return (1, $param->{$mg});
	} else {
		Logger->get()->warning('MachineGroup', "$mg", "Failed to query machine group $mg ");
		return (0, $param);
	}
}

sub getMachine {
	my ($class, $m) = @_;
	my $md = MachineDAO->new();
	my ($rc, $param) = $md->queryMachine({'name'=>$m});
	if ($rc == 0) {
		return (1, $param->{$m});
	} else {
		Logger->get()->warning('Machine', "$m", "Failed to query machine $m ");
		return (0, $param);
	}
}

sub getAllMach {
	my ($class) = @_;
	my $md = MachineDAO->new();
	my ($rc, $param) = $md->queryAllMachine();
	if ($rc == 0) {
		return (1, $param);
	} else {
		return (0, $param);
	}
}

sub getMachTypeByName {
	my ($class, $name) = @_;

	if (!$name) {
		return 0;
	}

	my $md = MachineDAO->new();
	my ($rc, $param) = $md->queryMachine({'name'=>$name});
	if ($rc == 0) {
		return $param->{$name}->{'machine.type'};
	} else {
		return 0;
	}
}

sub getProvIdByMach {
	my ($class, $mach_name) = @_;

	if (!$mach_name) {
		return 0;
	}

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryRequestMachine({'machine_name' => $mach_name});
	if ($rc != 0) {
		return 0;
	}

	my $req_machs = $param;

	foreach my $id (%$req_machs) {
		my $args = {
			'name' => $req_machs->{$id}->{'request_machine.request_name'},
		};

		($rc, $param) = $pd->queryRequest($args);
		if ($rc == 0) {
			return $req_machs->{$id}->{'request_machine.request_name'};
		}
	}

	return 0;
}

sub getAllPMach {
	my $result = {};

	my $md = MachineDAO->new();

	my ($rc, $param) = $md->queryAllPMach();

	if ($rc != 0 && $rc != 100) {
		return;
	} elsif ($rc == 100) {
		return $result;
	}

	return $param;
}

sub getAllPMachName {
	my $result = [];

	my $md = MachineDAO->new();

	my ($rc, $param) = $md->queryAllPMachName();

	if ($rc != 0 && $rc != 100) {
		return;
	} elsif ($rc == 100) {
		return $result;
	}

	foreach my $tmp_name (keys %$param) {
		push(@$result, $tmp_name);
	}

	return $result;
}

sub getAllPhyMachName {
	my $result = [];

	my $md = MachineDAO->new();

	my ($rc, $param) = $md->queryAllPhyMachName();

	if ($rc != 0 && $rc != 100) {
		return;
	} elsif ($rc == 100) {
		return $result;
	}

	foreach my $tmp_name (keys %$param) {
		push(@$result, $tmp_name);
	}

	return $result;
}
sub getAllHypv {
	my ($class) = @_;

	my @all_machine;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryAllRequestName();
	if ($rc != 0){
		return 0;
	}
	my $reqs = $param;
	foreach my $req (@$reqs) {
		my $args = {
			'name' => $req,
			'hypv_flag' => 1,
		};

		my ($rc, $param) = $pd->queryRequest($args);
		if ($rc == 0) {
			my $hypv_reqs = $param;

			foreach my $mach (@{$hypv_reqs->{$req}->{'machList'}}) {
				push(@all_machine, $mach->{'machine.name'});
			}
		}
	}

	my %saw;
	@saw{@all_machine} = ();
	return [sort(keys(%saw))];
}

sub getAllHypvWithStatus {
	my ($class, $status) = @_;
	my @all_machine;
	my $pd = ProvisionDAO->new();

	my $args = {
		'hypv_flag' => 1,
	};
	my ($rc, $param) = $pd->queryRequest($args);
	if ($rc == 0) {
		while(my ($req, $hypv_reqs) = each(%$param)){
			if($hypv_reqs->{'request.status'} eq $status){
				foreach my $mach (@{$hypv_reqs->{'machList'}}) {
					push(@all_machine, $mach->{'machine.name'});
				}
			}
		}
	}

	my %saw;
	@saw{@all_machine} = ();
	return [sort(keys(%saw))];
}

sub updateMachByName {
	my ($class, $mach, $attr) = @_;

	my $md = MachineDAO->new();

	my ($rc, $param) = $md->modifyMachine({'name' => $mach}, $attr);
	if ($rc != 0) {
		return 0;
	}

	$md->commit();

	return 1;
}

sub approveImage {
	my ($class, $user, $image, $approve) = @_;

	if (CloudUtils->checkCapability($user, "approve-image")== 0) {
		Logger->get()->error('Image', "$image", "User $user No permission to approve-image.");
		return (0, "No permission to do this operation.");
	}

	my $id = ImageDAO->new();
	my ($rc, $param) = $id->queryImage({'name'=>$image});
	if ($rc != 0) {
		Logger->get()->error('Image', "$image", "Cannot find image $image from database");
		return (0, "Cannot find the specified image from database, Reason: $param");
	}

	my $hdl = $param->{$image};

	if ($hdl->{'image.status'} eq 'INIT') {
		if ($approve =~ /^yes$/i) {
			($rc, $param) = $id->modifyImage({'name' => $image}, {'status' => 'AVAILABLE', 'approved_by' => $user});
			if ($rc == 0) {
				$id->commit();
				return 1;
			}
			$id->rollback();
			return (0, $param);
		}else{
			Logger->get()->error('Image', "$image", "Image $image is not in correct status.");
			return (0, "Status of $image has been INIT.");
		}
	}
	elsif ($hdl->{'image.status'} eq 'AVAILABLE') {
		if ($approve =~ /^no$/i) {
			($rc, $param) = $id->modifyImage({'name' => $image}, {'status' => 'INIT', 'approved_by' => $user});
			if ($rc == 0) {
				$id->commit();
				return 1;
			}
			$id->rollback();
			return (0, $param);
		}else{
			Logger->get()->error('Image', "$image", "Image $image is not in correct status.");
			return (0, "Status of $image has been AVAILABLE.");
		}
	}

}

sub changeImageGroups {
	my ($class, $image, @groups) = @_;

	my @auth_groups = ();
	foreach (@groups) {
		push(@auth_groups , {'user_group.name' => $_});
	}

	my $id = ImageDAO->new();
	my ($rc, $param) = $id->modifyImageAuthroizedUserGroups($image, {'authroized-user-group-list' => \@auth_groups});

	if ($rc == 0) {
		$id->commit();
		return 1;
	}

	$id->rollback();
	return (0, $param);
}

=comment
Note: Set the image to state REMOVED instead of delete it from database.
=cut
sub deleteImage {
	my ($class, $image, $force_flag) = @_;
	my $user = getpwuid($<);
	if (CloudUtils->checkCapability($user, "delete-image")== 0) {
		Logger->get()->error('Image', "$image", "User $user No permission to delete image.");
		return (0, "No permission to do this operation.");
	}
	my $id = ImageDAO->new();
	my $rc;
	my $param;
	if($force_flag){
		($rc, $param) = $id->deleteImageByName($image);
	}else{
		($rc, $param) = $id->modifyImage({'name' => $image}, {'status' => 'REMOVED'});
	}
	if ($rc == 0) {
		$id->commit();
		return 1;
	}

	$id->rollback();
	return 0;
}

sub captureImage {
	my ($class, $mach, $label, $arch) = @_;
	my $errmsg;
	my $user = getpwuid($<);
	if (CloudUtils->checkCapability($user, "create-image")== 0) {
		Logger->get()->error('Image', "$label", "User $user No permission to create image.");
		$errmsg = "No permission to do this operation.";
		goto cap_FAIL;
	}
	my $cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  '/opt/xcat/bin/imgcapture " . $mach ." -p " . lc($label) . "'";
	my $rc = system($cmd);
	if ($rc ne 0) {
		$errmsg = "Failed to capture image from $mach.";
		goto cap_FAIL;
	}
	$cmd = sprintf("/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root '/opt/xcat/sbin/packimage -o rhels6 -p %s -a %s'", $label, $arch);
	$rc = system($cmd);
	if ($rc ne 0) {
		$errmsg = "Failed to pack image.";
		goto cap_FAIL;
	}
	return (1);
cap_FAIL:
	return (0, $errmsg);
}

sub getAllImage {
	my @images = ();
	my $id = ImageDAO->new();
	my ($rc, $param) = $id->queryAllImage();
	if ($rc == 0) {
		@images = keys(%$param);
		return \@images;
	}
	return undef;
}

sub checkCapability{
	my ($class, $user, $capFind) = @_;
	my $permssion = 0;
	my @caps = CloudUtils->getUserCapability($user);
	if(defined($caps[0])){
		foreach my $i (@caps) {
			if ($i eq $capFind) {
				$permssion= 1;
				last;
			}
		}
	}

	return ($permssion);
}

sub newImage {
	my ($class, $name, $args) = @_;
	my $user = getpwuid($<);
	#$who only available while capture image, so we re-get the operator name 
	if (!CloudUtils->checkCapability($user, "create-image")) {
		Logger->get()->error('Image', "$name", "User $user No permission to create image.");
		return (0, "No permission to do this operation.");
	}

	my @groups =();
	if(exists($args->{'grouplist'})){
		@groups = split(",", $args->{'grouplist'});
	}

	if(!exists($args->{'status'})) {
		$args->{'status'} = 'INIT';
	}

	my $attrs = {
		'arch' => $args->{'arch'},
		'base' => $args->{'os'},
		'profile' => $args->{'profile'},
		'description' => $args->{'desc'},
		'status' => $args->{'status'},
		'captured_on' => $args->{'mach'},
		'captured_time' => $args->{'captured_time'},
		'captured_by' => $args->{'captured_by'},

		'pkg_location' => $args->{'pkglocation'},
		'pkglist_path' => $args->{'pkglist'},
		'scheduler' => $args->{'scheduler'},
		'parallel_env' => $args->{'parallelenv'},
		'pre_scripts' => $args->{'prescripts'},
		'post_scripts' => $args->{'postscripts'},
		'interconnect' => $args->{'interconnect'},
		'shared_fs' => $args->{'sharedfs'},
		'install_to_disk' => $args->{'install_to_disk'},
	};
	if(scalar(@groups) ne 0){
		print "@groups";
		$attrs->{'authroized-user-group-list'} = \@groups;
	}
	my $id = ImageDAO->new();
	my ($rc, $param) = $id->insertImage($name, $attrs);
	if ($rc != 0) {
		return (0, $param);
	}

	$id->commit();

	return 1;
}

sub modImageStatus{
	my ($class, $name,  $status) = @_;
	if (!$name || !$status) {
		return (0, "name or status is not specified.");
	}

	my $attrs = {
		'status' => $status,
	};
	my $id = ImageDAO->new();
	my $query->{'name'}=$name;
	my ($rc, $param) = $id->modifyImage($query, $attrs);
	if ($rc == 0) {
		$id->commit();
		return 1;
	}
	$id->rollback();
	return (0, $param);
}

sub getAllImageInfo {
	my $ud = ImageDAO->new();

	my ($rc, $param) = $ud->queryAllImage();
	if ($rc == 0) {
		return $param;
	}

	return undef;
}

sub getImageInfo {
	my ($class, $attrs) = @_;

	my $ud = ImageDAO->new();

	my ($rv, $param) = $ud->queryImage($attrs);

	return ($rv, $param);
}

sub newProvReq {
	my ($class, $name, $attrs, $mach_list, $user_list, $group_list) = @_;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->insertRequest($name, $attrs);
	if ($rc != 0) {
		return 0;
	}

	foreach my $mach (@$mach_list) {
		($rc, $param) = $pd->insertRequestMachine($name, {'machine_name' => $mach,'status' => $attrs->{'status'}});
		if ($rc != 0) {
			$pd->rollback();
			return 0;
		}
	}

	foreach my $user (@$user_list) {
		($rc, $param) = $pd->insertRequestUser($name, {'user_name' => $user, 'is_user_group' => 0});
		if ($rc != 0) {
			$pd->rollback();
			return 0;
		}
	}

	foreach my $group (@$group_list) {
		($rc, $param) = $pd->insertRequestUser($name, {'user_name' => $group, 'is_user_group' => 1});
		if ($rc != 0) {
			$pd->rollback();
			return 0;
		}
	}

	$pd->commit();

	return 1;
}
=description
Description:
	Used to update thc_request, thcr_request_machine and thcr_request_user three tables. It will update only existed changed information. 
	So you can use it just change one of three tables;
input: 
	$name	request name
	$info	what changed hash reference
	$special_flag	since thcr_request_machine request name need use a or d as post fix while modificaiton not completed, we need the flag to 
		let the program know it.
return:
	$rtv resutl, 0 failed, 1 ok
	$errstr maybe empty while success
=cut
sub modifyProvReq {
	my ($class, $name, $info, $special_flag) = @_;
	my $pd = GeneralDAO->new();
	if(exists($info->{'request'})){
		my ($rc, $param) = $pd->modify("request",$info->{'request'},["name,$name"]);
		if ($rc != $GeneralDefine::SUCCESS) {
			return (0, $param);
		}		
	}
	if (exists($info->{'del_machines'})){
		my @del_machs = @{$info->{'del_machines'}};
		my $new_name = $name;
		if(defined($special_flag) and defined($special_flag->{'del_machines'})){
			$new_name = $new_name.$special_flag->{'del_machines'};
			if($special_flag->{'del_machines'} eq 'd'){#delete but with a post fix 'd' means change the name only, since we do not complete it
				foreach my $mach (@del_machs) {
					my ($rc, $param) = $pd->modify("request_machine", {"request_name"=>$new_name}, ["request_name,$name", "machine_name,$mach"]);
					if ($rc != $GeneralDefine::SUCCESS) {
						$pd->rollback();
						return (0, $param);
					}
				}
			}else{#add machine but failed, now restore, we need delete it, the $special_flag->{'del_machines'} is a
				foreach my $mach (@del_machs) {
					my ($rc, $param) = $pd->delete("request_machine", ["request_name,$new_name", "machine_name,$mach"]);
					if ($rc != $GeneralDefine::SUCCESS) {
						$pd->rollback();
						return (0, $param);
					}
				}
			}

		}else{
			foreach my $mach (@del_machs) {#real want delete machine from request_machine
				my ($rc, $param) = $pd->delete("request_machine", ["request_name,$new_name", "machine_name,$mach"]);
				if ($rc != $GeneralDefine::SUCCESS) {
					$pd->rollback();
					return (0, $param);
				}
			}
		}
	}
	if (exists($info->{'add_machines'})){
		my @add_machs = @{$info->{'add_machines'}};
		my $new_name = $name;
		if(defined($special_flag) and defined($special_flag->{'add_machines'})){
			$new_name = $new_name.$special_flag->{'add_machines'};
			if($special_flag->{'add_machines'} eq 'a'){
				foreach my $mach (@add_machs) {#add machine and the request name add 'a' since we do not complete it
					my ($rc, $param) = $pd->insert("request_machine", {"request_name"=>$new_name, "machine_name" => $mach});
					if ($rc != $GeneralDefine::SUCCESS) {
						$pd->rollback();
						return (0, $param);
					}
				}
			}else{
				foreach my $mach (@add_machs) {#delete machine failed and now we restore, so we just change the request name from xxxd to xxx
					my ($rc, $param) = $pd->modify("request_machine", {"request_name"=>$name}, ["request_name,$new_name", "machine_name,$mach"]);
					if ($rc != $GeneralDefine::SUCCESS) {
						$pd->rollback();
						return (0, $param);
					}
				}
			}
		}else{#real want delete machine from request_machine
			foreach my $mach (@add_machs) {
				my ($rc, $param) = $pd->insert("request_machine", {"request_name"=>$new_name, "machine_name" => $mach});
				if ($rc != $GeneralDefine::SUCCESS) {
					$pd->rollback();
					return (0, $param);
				}
			}
		}
	}
	if (exists($info->{'incUsers'})){
		my ($rc, $param) = $pd->delete("request_user", ["request_name,$name", "is_user_group,0"]);
		if ($rc != $GeneralDefine::SUCCESS) {
			$pd->rollback();
			return (0, $param);
		}
		my @users=@{$info->{'incUsers'}};
		foreach my $user (@users) {
			($rc, $param) = $pd->insert("request_user", {"request_name"=>$name, "user_name" => $user, 'is_user_group' => 0});
			if ($rc != $GeneralDefine::SUCCESS) {
				$pd->rollback();
				return (0, $param);
			}
		}
	}
	if (exists($info->{'incGrps'})){
		my ($rc, $param) = $pd->delete("request_user", ["request_name,$name", "is_user_group,1"]);
		if ($rc != $GeneralDefine::SUCCESS) {
			$pd->rollback();
			return (0, $param);
		}
		my @groups = @{$info->{'incGrps'}};
		foreach my $group (@groups) {
#			my($rc, $param) = $pd->insertRequestUser($name, {'user_name' => $group, 'is_user_group' => 1});
			my ($rc, $param) = $pd->insert("request_user", {"request_name"=>$name, "user_name" => $group, 'is_user_group' => 1});
			if ($rc != $GeneralDefine::SUCCESS) {
				$pd->rollback();
				return (0, $param);
			}
		}
	}	
	$pd->commit();
	return 1;
}

# This API is invoked when a NON-submitted request is canceled.
sub removeProvReq {
	my ($class, $name) = @_;

	my $pd = ProvisionDAO->new();

	$pd->deleteRequest({"name" => $name});
	$pd->deleteRequestMachine({"request_name" => $name});
	$pd->deleteRequestUser({"request_name" => $name});

	$pd->commit();

	return 1;
}

# This API is invoked after a submitted Request is terminated
# or the Request is modified.
sub cleanProvReq {
	my ($class, $request_id, $machs, $users, $groups, $status_p) = @_;

	if (!defined($machs) || !defined($users) || !defined($groups)) {
		return ;
	}

	my $pd = ProvisionDAO->new();

	foreach my $mach (@$machs) {
		# clean up the request machine record.
		$pd->deleteRequestMachine({'request_name' => $request_id, 'machine_name' => $mach});

		# clean up the energy usage.
		EnergyUtils->rmEnergyUsageByMachProv($mach, $request_id);
	}

	foreach my $user (@$users) {
		$pd->deleteRequestUser({'request_name' => $request_id, 'user_name' => $user, 'is_user_group' => 0});
	}

	foreach my $group (@$groups) {
		$pd->deleteRequestUser({'request_name' => $request_id, 'user_name' => $group, 'is_user_group' => 1});
	}

	# Do not clean the energy schemes if terninating a Reserved Request.  
	if (!defined($status_p) || $status_p ne "Reserved") {
		EnergyUtils->rmESchemeMachineByM($machs);
	}

	$pd->commit();

	return ;
}

sub updateProvDynAttrById {
	my ($class, $name, $attrs) = @_;
	my $query->{'name'} = $name;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->modifyRequest($query, $attrs);
	if ($rc != 0) {
		$pd->rollback();
		return(0);
	}

	$pd->commit();

	return(1);
}
=desc
	New consider requestName+machineName modification in future
=cut
sub updateProvMachineById {
	my ($class, $name, $attrs) = @_;
	if(!defined($name)){
		return (0);
	}
	my $query->{'request_name'} = $name;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->modifyRequestMachine($query, $attrs);
	if ($rc != 0) {
		return(0);
	}

	$pd->commit();

	return(1);
}

sub deleteUnRequestedMachine {
	my ($class, $mach_info) = @_;

	if(!defined($mach_info)){
		return 0;
	}

	my $mach_name = $mach_info->{'machine.name'};

	my $pd = ProvisionDAO->new();
	my ($rv, $param) = $pd->queryRequestMachine({'machine_name' => $mach_name});
	if ($rv == 0) {
		return (1, "Machine $mach_name used by request $param->{'request.name'}.");
	}

	my $md = MachineDAO->new();
	$rv = $md->deleteMachine({'name' => $mach_name});
	if ($rv != 0) {
		return (1, "Delete machine $mach_name from db failed.");
	}

	$md->commit();

	return 0;
}

sub addRoleCapability {
	my ($class, $role, $cap) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryRole({'name'=>$role});
	if ($rc != 0) {
		return 0;
	}

	my $hdl = $param->{$role}->{'cap-list'}->{'capability'};
	return 0 unless (defined($hdl)) ;

	my @caps = ();
	foreach my $i (@$hdl) {
		my @temp = values(%$i);
		if ($temp[0] eq $cap) {
			return 1;
		} else {
			push(@caps, $temp[0]);
		}
	}

	push(@caps, $cap);
	($rc, $param) = $ud->modifyRoleCapabilities($role, @caps);
	if ($rc == 0) {
		$ud->commit();
		return 1;
	}

	$ud->rollback();
	return 0;
}

sub removeRoleCapability {
	my ($class, $role, $cap) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryRole({'name'=>$role});
	if ($rc != 0) {
		return 0;
	}

	my $hdl = $param->{$role}->{'cap-list'}->{'capability'};
	return 0 unless (defined($hdl)) ;

	my @caps = ();
	my $found = 0;
	foreach my $i (@$hdl) {
		my @temp = values(%$i);
		if ($temp[0] eq $cap) {
			$found = 1;
		} else {
			push(@caps, $temp[0]);
		}
	}

	return(1) if ($found == 0);

	($rc, $param) = $ud->modifyRoleCapabilities($role, @caps);
	if ($rc == 0) {
		$ud->commit();
		return 1;
	}

	$ud->rollback();
	return 0;
}

sub getRole {
	my ($class, $role) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryRole({'name'=>$role});
	if ($rc != 0) {
		return undef;
	}

	return $param;
}

sub changeUserRole {
	my ($class, $user, $role) = @_;
	if (!defined($user) || !defined($role)) {
		return $GeneralDefine::FAILED;
	}

	my $rc;
	my $param;
	my $ud = UserDAO->new();
	if ($role eq 'NA') {
		($rc, $param) = $ud->removeUserRole($user);
	} else {
		($rc, $param) = $ud->modifyUser({'name'=>$user}, {'role'=>$role});
	}
	if ($rc == 0) {
		$ud->commit();
		return $GeneralDefine::FAILED;
	}

	$ud->rollback();
	return $GeneralDefine::SUCCESS;
}

sub getUserName {
	my ($class, $uname) = @_;
	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryUser({'name'=>$uname});
	if ($rc == 1) {
		return undef;
	}
	return $uname;
}

sub getUserCapability {
	my ($class, $user) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryUser({'name'=>$user});
	if ($rc != 0) {
		return undef;
	}

	my $hdl = $param->{$user}->{'cap-list'}->{'capability'};
	my @caps = ();
	foreach my $i (@$hdl) {
		push(@caps, values(%$i));
	}

	return (@caps);
}

sub addUser {
	my ($class, $name, $attr) = @_;

	if (!$name) { return 0; }

	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "add-user")== 0) {
		Logger->get()->error('User', "$name", "User $current_user No permission to add user.");
		return (0, "No permission to do this operation.");
	}

   	my $ud = UserDAO->new();
   	my ($rc, $param) = $ud->insertUser($name, $attr);
   	if ($rc == 0) {
	   	$ud->commit();
	   	return 1;
   	}

   	$ud->rollback();
   	return (0, $param);
}

sub deleteUser {
	my ($class, $name) = @_;

	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "delete-user")== 0) {
		Logger->get()->error('User', "$name", "User $current_user No permission to delete user.");
		return (0, "No permission to do this operation.");
	}

	# The user is not allowed to be deleted if he owns some requests.
	my $myprovs = CloudUtils->getActiveProvOwnedByUser($name);
	if ($myprovs && scalar(%$myprovs) > 0) {
		return (0, "The user $name owns at least one request.");
	}

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->deleteUserByName($name);
	if ($rc == 0) {
		$ud->commit();
		BillingDemo->setUserAccount($name, -1);#-1 means delete the user
		return 1;
	}

	$ud->rollback();
	return 0;
}

sub addGroup {
	my ($class, $name, $owner, $userlist, $comments) = @_;

	my $permssion = 0;
	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "add-group")== 0) {
		Logger->get()->error('Group', "$name", "User $current_user No permission to add group.");
		return (0, "No permission to do this operation.");
	}
	if (!$name || !$owner) { return 0; }

	my $users = [];
	foreach my $i (@$userlist) {
		push(@$users, {'user.name'=>$i});
	}

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->insertGroup($name, {'owner'=>$owner, 'user-list'=>$users, 'description'=>$comments});
	if ($rc == 0) {
		$ud->commit();
		return 1;
	}

	$ud->rollback();
	return 0;
}

sub updateGroup {
	my ($class, $name, $userlist, $desc) = @_;

	if (!$name) { return (0, "Invalid group name"); }

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryGroup({'name'=>$name});
	if($rc != 0){
		return (0, "Nonexistent Group");
	}
	my $current_user = getpwuid($<);
	if(($param->{$name}->{"user_group.owner"} ne $current_user)&&(CloudUtils->checkCapability($current_user, "modify-group")== 0)){
		Logger->get()->error('Group', "$name", "User $current_user No permission to modify group.");
		return (0, "No permission to do this operation.");
	}
	($rc, $param) = $ud->modifyGroupMembers($name, @$userlist);
	if ($rc == 0) {
		if($desc){
			($rc, $param) = $ud->modifyGroup({'name'=>$name}, {'description'=>$desc});
			if($rc == 0){
				$ud->commit();
				return (1);
			}
		}else{
			$ud->commit();
			return (1);
		}
	}

	$ud->rollback();
	return (0, $param);

}

sub deleteGroup {
	my ($class, $name) = @_;

	my $permssion = 0;
	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "delete-group")== 0) {
		Logger->get()->error('Group', "$name", "User $current_user No permission to delete group.");
		return (0, "No permission to do this operation.");
	}

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->deleteGroupByName($name);
	if ($rc == 0) {
		$ud->commit();
		return 1;
	}

	$ud->rollback();
	return 0;
}

sub getProvInfoWOCurrEnergy {
	my ($class) = @_;

	my $pd = ProvisionDAO->new();

	my $all_prov_info = ();

	my ($rc, $param) = $pd->queryAllRequestName();
	if ($rc == 0) {
		foreach my $req_name (@$param) {
			my $prov = CloudUtils->getProvInfoById($req_name);
			$all_prov_info->{$req_name} = $prov;
		}
	}

	return $all_prov_info; 
}

=comment
Description:
	Get the request information according different query type
input $brief
   1: Get the request names only.
   0: Get the detailed information of the requests.
   $fuzzy:
		our $QUERY_NAME_ONLY = 0; #: get machines of cloudadm_10.27.2011_01:53:45 only
		our $QUERY_NAME_RELATED_ALL = 1; #: get machines of request cloudadm_10.27.2011_01:53:45a or cloudadm_10.27.2011_01:53:45d also
		our $QUERY_NAME_ADD = 2; # get machines of request cloudadm_10.27.2011_01:53:45a also but not ***d
		our $QUERY_NAME_DEL = 3; # get machines of request cloudadm_10.27.2011_01:53:45d also but not ***a
	$flag:
		0: Arrayref for machList
		1: Hashref for machList
=cut
sub getProvInfo {
	my ($class, $brief, $fuzzy, $flag) = @_;

	my $pd = ProvisionDAO->new();
	my $all_prov_info = ();
	if(!defined($flag)){
		$flag = 0;
	}

	my ($rc, $param) = $pd->queryAllRequestName();

	if ($rc == 0) {
		foreach my $req_name (@$param) {
			if ($brief) {
				# Return the request names only.	
				$all_prov_info->{$req_name} = $req_name;
			} else {
				# Else, return the detailed information of the requests.
				my $prov = CloudUtils->getProvAndEnergyInfoById($req_name, $flag, $fuzzy);
				$all_prov_info->{$req_name} = $prov;
			}
		}
	}

	return $all_prov_info; 
}

sub getProvInfoById {
	my ($class, $pid, $flag, $fuzzy) = @_;
	my $prov_info->{'name'} = $pid;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryRequest($prov_info, $flag, $fuzzy);
	if ($rc != 0) {
		#log the error $DBI::errstr
		return undef;
	}

	if(!exists($param->{$pid}->{'machList'})&&
			($param->{$pid}->{'request.status'} eq 'Expired' || $param->{$pid}->{'request.status'} eq 'Canceled' || $param->{$pid}->{'request.status'} eq 'Failed' )){
		my $tmpParam;
		($rc, $tmpParam) = $pd->queryRequestfromhistory($prov_info,$flag);
		if(($rc==0)&&(exists($tmpParam->{$pid}->{'machList'}))){
			$param = $tmpParam;
		} 
	}

	return $param->{$pid};
}

# Get requests owned by the specified user.
# Requestes in status Expired/Canceled/Failed will be excluded.
sub getActiveProvOwnedByUser {
	my ($class, $uid) = @_;
	if (!$uid) { return undef; }

	my $allprov = CloudUtils->getProvInfo();
	if (!$allprov) { return undef; }

	foreach my $provId (keys %$allprov) {
		my $prov = $allprov->{$provId};
		if ( ($prov->{'request.status'} eq "Expired") ||
			 ($prov->{'request.status'} eq "Canceled") ||
			 ($prov->{'request.status'} eq "Failed") ||
			 ($prov->{'request.owner'} ne $uid)) {
			delete $allprov->{$provId};
		}
	}
	return $allprov;
}

# Get requests that the specified user is authorized to view.
sub getProvByUser {
	my ($class, $uid, $fuzzy, $flag) = @_;

	my $allprov = CloudUtils->getProvInfo(0, $fuzzy, $flag);
	my $user = CloudUtils->getUserByName($uid);
	if (!$allprov || !$user) { return undef; }

	foreach my $provId (keys %$allprov) {
		unless (CloudUtils->checkProvAuth($allprov->{$provId}, $user->{$uid})) {
			delete $allprov->{$provId};
		}
	}
	return $allprov;
}

sub checkProvAuth {
	my ($class, $prov, $user) = @_;
	if ($prov->{'request.owner'} eq $user->{'user.name'}) {
		return 1;
	}
	my $groupAllowed = 2;  # 0: Forbid, 1: Allow, 2: Unset
	my $userAllowed = 2;
	if ($prov->{'incGrps'}) {
		foreach my $grp (@{$user->{'group-list'}->{'group'}}) {
			if (grep /^$grp->{'group.name'}$/, @{$prov->{'incGrps'}}) {
				$groupAllowed = 1;
				last;
			}
		}
	} elsif ($prov->{'excGrps'}) {
		foreach my $grp (@{$user->{'group-list'}->{'group'}}) {
			if (grep /^$grp->{'group.name'}$/, @{$prov->{'excGrps'}}) {
				$groupAllowed = 0;
				last;
			}
		}
	} else {
		$groupAllowed = 2;
	}
	if ($prov->{'incUsers'}) {
		if (grep /^$user->{'user.name'}$/, @{$prov->{'incUsers'}}) {
			$userAllowed = 1;
		}
	} elsif ($prov->{'excUsers'}) {
		if (grep /^$user->{'user.name'}$/, @{$prov->{'excUsers'}}) {
			$userAllowed = 0;
		}
	} else {
		$userAllowed = 2;
	}

	if ($userAllowed != 2) {
		return $userAllowed;
	} elsif ($groupAllowed != 2) {
		return $groupAllowed;
	} else {
		return 0;
	}
}

sub isCloudUser {
	my ($class, $uid) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryUser({'name'=>$uid});
	if ($rc == 0) {
		return 1;
	} else {
		return 0;
	}
}

sub isAdmin {
	my ($class, $uid) = @_;

	my $param = CloudUtils->getConfig();
	if ($param) {
		if (defined($param->{'adminList'})) {
			my @admin = split(/,/, $param->{'adminList'});
			return grep(/^$uid$/, @admin);
		}
	}

	return 0;
}

sub getAllUser {
	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryAllUser();
	if ($rc == 0) {
		my @users = keys(%$param);
		foreach my $username (@users){
			my $user_balance = BillingDemo->getUserAccount($username, 1);
			$param->{$username}->{"balance"} = $user_balance;
			my ($rtv, $userInfo) = CloudUtils->querySysUser($username);
			if($rtv == 1){
				my $shell = $userInfo->{$username}->{"loginShell"};
				$shell->[0] =~ /.+\/(\S+)/;
				if(defined($1)){
					$param->{$username}->{"shell"} = $1;
				}
			}
		}
		return $param;
	}
	return undef;
}

sub getAllUserName {
	my $users = &getAllUser;
	if (defined($users)) {
		return [keys(%$users)];
	}
	return undef;
}

sub getAllGroupName {
	my $groups = &getAllGroup;
	if (defined($groups)) {
		return [keys(%$groups)];
	}
	return undef;
}


sub excludeUser {
	my ($class, $excUsers) = @_;
	my $alluser = CloudUtils->getAllUserName;

	my @arr;

	foreach my $name (@$alluser) {
		if (!grep /^$name$/, @$excUsers) {
			push @arr, $name;
		}
	}
	return \@arr;
}

sub excludeGroup {
	my ($class, $excGrps) = @_;
	my $allgrp = CloudUtils->getAllGroupName;

	my @arr;

	foreach my $name (@$allgrp) {
		if (!grep /^$name$/, @$excGrps) {
			push @arr, $name;
		}
	}
	return \@arr;
}

sub getUserByName {
	my ($class, $uname) = @_;

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryUser({'name'=>$uname});
	if ($rc == 0) {
		my $user_balance = BillingDemo->getUserAccount($uname, 1);
		$param->{$uname}->{"balance"} = $user_balance;
		my ($rtv, $userInfo) = CloudUtils->querySysUser($uname);
		if($rtv == 1){
			my $shell = $userInfo->{$uname}->{"loginShell"};
			$shell->[0] =~ /.+\/(\S+)/;
			if(defined($1)){
			$param->{$uname}->{"shell"} = $1;
			}
		}
		return $param;
	}

	return undef;
}

sub getAllGroup {
	my ($class, $owner) = @_;
	my $st;
	my $rv;
	my $condition;

	my $args = {};

	if (defined($owner)) {
		$args->{'owner'} = $owner;
	}

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryAllGroup($args);
	if ($rc == 0) {
		return $param;
	}

	return undef;
}

sub getConfigDetail {
	my $result = {};
	my $confd = ConfigDAO->new();
	my ($rc, $param) = $confd->query();
	if ($rc == 0) {
		return $param;
	}

	return undef;
}

sub getConfig {
	my $result = {};
	my $confd = ConfigDAO->new();
	my ($rc, $param) = $confd->query();
	if ($rc == 0) {
		foreach my $name (keys %$param) {
			$result->{$name} = $param->{$name}->{'sysconfig.value'}
		}
		return $result;
	}

	return undef;
}

sub putConfig {
	my ($class, $config) = @_;

	my $confd = ConfigDAO->new();
	my ($rc, $param) = $confd->query();
	while (my ($key, $value) = each (%$config)) {
		if(exists($param->{$key})){
			$rc  = $confd->modify($key, $value);
			if($rc != 0) {
				goto FAIL;
			}
		}
		else{
			$rc = $confd->insert($key, $value);
			if ($rc != 0) {#try again to avoid concurrent issue
				$rc = $confd->modify($key, $value);
				if ($rc != 0) {
					goto FAIL;
				}
			}
		}
	}

	$confd->commit();
	return 1;
FAIL:
	$confd->rollback();
	return 0;
}

# add a record into THC_ProvisionMachineLog
sub addProvMachLog {
	my ($class, $machname, $pid, $startTime, $stopTime, $energy, $arch, $os, $ip) = @_;

	my $args = {
		'machine_name' => $machname,
		'prov_start_time' => $startTime,
		'prov_stop_time' => $stopTime,
		'energy' => $energy,
		'arch' => $arch,
		'os' => $os,
		'ip' => $ip,
	};

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->insertProvisionMachineHistory($pid, $args);
	if ($rc != 0) {
		$pd->rollback();
		return 0;
	}

	$pd->commit();

	return 1;
}

sub getProvMachLogByPM {
	my ($class, $pid, $machName) = @_;

	if (!$pid) {
		return 0;
	}

	my $args = {
		'request_name' => $pid,
		'machine_name' => $machName,
	};
	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryProvisionMachineHistory($args);
	if ($rc != 0) {
		return undef;
	}

	return $param;
}

sub updateProvMachLogByPM {
	my ($class, $pid, $machName, $attr) = @_;

	if (!$pid || !$machName) {
		return 0;
	}

	my $ref = CloudUtils->getProvMachLogByPM($pid, $machName);
	if (!defined($ref)) {
		# no matched record
		return 0;
	}

	my $record = ();
	foreach my $id (keys %$ref) {
		$record = $ref->{$id};
		last;
	}

	my $pd = ProvisionDAO->new();

	my $query = {
		'request_name' => $pid,
		'machine_name' => $machName,
	};

	my $args = {};
	if (exists($attr->{'machineName'})) {$args->{'machine_name'} = $attr->{'machineName'};}
	if (exists($attr->{'provisionStart'})) {$args->{'prov_start_time'} = $attr->{'provisionStart'};}
	if (exists($attr->{'provisionStop'})) {$args->{'prov_stop_time'} = $attr->{'provisionStop'};}
	if (exists($attr->{'energy'})) {$args->{'energy'} = $attr->{'energy'} + $record->{'provision_machine_history.energy'};}
	if (exists($attr->{'arch'})) {$args->{'arch'} = $attr->{'arch'};}
	if (exists($attr->{'os'})) {$args->{'os'} = $attr->{'os'};}
	if (exists($attr->{'ip'})) {$args->{'ip'} = $attr->{'ip'};}
	
	if (!scalar(%$args)) {
		# nothing to update
		return 0;
	}

	my ($rc, $param) = $pd->modifyProvisionMachineHistory($query, $args);
	if ($rc != 0) {
		$pd->rollback();
		return 0;
	}

	$pd->commit();

	return 1;
}

sub addSysCloudUser {
	my ($class, $userattr) = @_;
	if (!exists($userattr->{"userid"}) || !exists($userattr->{"password"})) { return 0; }
	my $uid = $userattr->{"userid"};
	my $passwd = $userattr->{"password"};
	use Net::LDAP;
	use LDAPUtils;
	my ($rc, $error_str);
	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "add-user")== 0) {
		Logger->get()->error('User', "$uid", "User $current_user No permission to add user.");
		return (0, "No permission to do this operation.");
	}
	my $param=CloudUtils->getAllUser();
	if(exists $param->{$uid}){
		return (0, "Duplicated user exist in Cloud DB");
	}
	my $shell = $userattr->{"shell"};
	if(!defined($shell)){
		$shell = "sh";
	}
	my $ldapc = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	$ldapc->bind();

	my $home = "/hpcc/home/$uid/";
	($rc, $error_str) = $ldapc->addUser($uid, {'gidNumber'=>[10001], 'homeDirectory'=>[$home], 'loginShell'=>["/bin/$shell"], 'host'=>['*']});
	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to $error_str.");
		$ldapc->unbind();
		return (0, "$error_str");
	}
	($rc, $error_str) = $ldapc->modUserPassword($uid, $passwd);
	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to $error_str.");
		$ldapc->delUser($uid);
		$ldapc->unbind();
		return (0, "$error_str");
	}

	# create user home dir;
	my $cmd = sprintf("/opt/ibm/hpccloud/cloudbin/utils/inithome %s", $uid);
	$rc = system("/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root '$cmd'");
	if ($rc != 0) {
		# failed to initialize the user home dir;
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to inithome failed.");
		$ldapc->delUser($uid);
		$ldapc->unbind();
		return (0, "inithome failed");
	}
	my $initBalance = $userattr->{"balance"};
	my @not_hpcdb_column=("userid", "password", "balance", "shell");
	foreach (@not_hpcdb_column){
		delete($userattr->{$_});
	}
	($rc, $error_str) = CloudUtils->addUser($uid, $userattr);
	if (!$rc) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to $error_str.");
		$ldapc->delUser($uid);
		$ldapc->unbind();
		return ($rc, $error_str);
	}
	$ldapc->unbind();
	if(defined($initBalance)){
		BillingDemo->setUserAccount($uid, $initBalance*100);
	}
	return 1;
}

sub changePassword {
	my ($class, $uid, $old_passwd, $passwd) = @_;
	if (!$uid || !$passwd) { return 0; }

	use Net::LDAP;
	use LDAPUtils;

	my $rc;
	my $current_user = getpwuid($<);
	if($current_user ne $uid){
		return (0, "Can only change self password");
	}
	my ($ldapc) = LDAPUtils->new( {'host'=>$::LDAPHost} );

	$ldapc->connect();
	($rc) = $ldapc->bindAs($uid, $old_passwd);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "LDAP failed to bind to user $uid.");
		return (0, "LDAP failed to bind to user $uid.");
	}

	($rc) = $ldapc->modUserPassword($uid, $passwd);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to LDAP error.");
		$ldapc->unbind();
		return (0, "Failed to add user $uid, due to LDAP error.");
	}

	$ldapc->unbind();
	return (1);
}

sub delSysUser {
	my ($class, $uid) = @_;

	if (!$uid) { return 0; }

	use Net::LDAP;
	use LDAPUtils;

	my ($ldapc) = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	$ldapc->bind();

	my ($rc) = $ldapc->delUser($uid);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to LDAP error.");
		$ldapc->unbind();
		return 0;
	}

	$ldapc->unbind();
	return 1;
}

sub querySysUser {
	my ($class, $uid) = @_;

	if (!$uid) { return 0; }

	use Net::LDAP;
	use LDAPUtils;

	my ($ldapc) = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	$ldapc->bind();

	my ($rc, $userInfo) = $ldapc->getUserInfo($uid);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to get user $uid information due to LDAP error.");
		$ldapc->unbind();
		return 0;
	}

	$ldapc->unbind();
	return (1, $userInfo);
}

sub modSysUsershell {
	my ($class, $uid, $shellRef) = @_;

	if (!$uid) { return 0; }
	if (!defined($shellRef) or scalar(@$shellRef) == 0) { return 0; }
	use Net::LDAP;
	use LDAPUtils;

	my ($ldapc) = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	$ldapc->bind();
	my %changedShell=("loginShell"=>$shellRef);
	my ($rc, $userInfo) = $ldapc->modUser($uid, \%changedShell);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to modify user $uid shell due to LDAP error.");
		$ldapc->unbind();
		return 0;
	}

	$ldapc->unbind();
	return 1;
}

sub status2Color {
	my %color = ("None" => "lightsteelblue", "Init" => "#fffc00", "Reserved" => "gold", "Preparing" => "goldenrod",
				 "Setup" => "royalblue", "Verifying" => "darkgreen", "Waiting" => "darkorchid", "Ready" => "lime", "Expired" => "gray",
				 "Failed" => "#ff0000", "Canceled" => "gray");

	return \%color;
}

# for accounting purpose;
# display provision accounting information;

sub trackEvent {
	my ($class, $user, $opt, $type, $target, $desc) = @_;
	my $time = time();
	
	unless (defined($desc)) {
		$desc = '';
	}
	
	eval {
		my $st = $::DBH->prepare("insert into THC_EventLog (user, operation, eventType, eventTarget, eventTime, eventDesc) VALUES (?, ?, ?, ?, ?, ?)");
		$st->bind_param(1, $user);
		$st->bind_param(2, $opt);
		$st->bind_param(3, $type);
		$st->bind_param(4, $target);
		$st->bind_param(5, $time);
		$st->bind_param(6, $desc);
		
		my $rv = $st->execute();
		$::DBH->commit();
	}; if ($@) {
		# log "Transaction aborted: $@";
		$::DBH->rollback();
		return 0;
	}
	
	return 1;
}

sub trackSubmitProvsionEvent {
	my ($class, $pid) = @_;
	my $user = getpwuid($>);
	Logger->get()->info('User', "$user", "Submit provision: $pid");
	Logger->get()->info('Provision', "$pid", "$user submit provision $pid");
	return 1;
}

sub trackCancelProvsionEvent {
	my ($class, $pid) = @_;
	my $user = getpwuid($>);
	Logger->get()->info('User', "$user", "Delete provision: $pid");
	Logger->get()->info('Provision', "$pid", "$user delete provision $pid");
	return 1;
}

sub trackChangeProvsionEvent {
	my ($class, $pid, $desc) = @_;
	my $user = getpwuid($>);
	Logger->get()->info('User', "$user", "Modify provision: $pid");
	Logger->get()->info('Provision', "$pid", "$user modify provision: $pid :: $desc");
	return 1;
}

sub getEventTracks {
	my ($class, $target) = @_;
	my $ldb = LogDAO->new();

	my $query = {};
	$query->{'level'} = 3;
	$query->{'object'} = $target;

	my ($rc, $param) = $ldb->queryLog($query);
	if ($rc == 0) {
		return $param;
	}

	return;
}

sub getLogs {
	my ($class, $query) = @_;
	my $ld = LogDAO->new();
	my ($rc, $param) = $ld->queryLog($query);
	if ($rc == 0) {
		return (1, $param);
	}
	return (0, $param);
}

sub getEvents {
	my ($class, $query) = @_;
	my $ed = EventDAO->new();
	my ($rc, $param) = $ed->queryEvent($query);
	if ($rc == 0) {
		return (1, $param);
	}
	return (0, $param);
}

sub getProvisionEvents {
	my ($class, $pid) = @_;
	return $class->getEventTracks($pid);
}

sub formatTime {
	my ($class, $secs, $fmt) = @_;

	use POSIX qw/strftime/;

	unless (defined($fmt)) {
		return strftime('%D %T',localtime($secs));
	}

	return '';
}

sub updatevNodeHypv {
	my ($class, $reqid, $vhref) = @_;

	my $pd = ProvisionDAO->new();

	foreach my $vnode (keys(%$vhref)) {
		my $query = {
			'request_name' => $reqid,
			'machine_name' => $vnode,
		};

		my $args = {
			'hypervisor' => $vhref->{$vnode},
		};

		my ($rc, $param) = $pd->modifyRequestMachine($query, $args);
		if ($rc != 0) {
			$pd->rollback();
			return 0;
		}
	}

	$pd->commit();

	return 1;
}

sub getvNodeHypv {
	my ($class, $reqid, $vnodes) = @_;
  
	my $vhref;
 
	my $pd = ProvisionDAO->new();
	
	my ($rc, $param) = $pd->queryRequest({'name' => $reqid});
	if ($rc != 0) {
		return $vhref;
	}

	my $attr = $param->{$reqid};

	foreach my $vnode (@$vnodes) {
		my $args = {
			'request_name' => $reqid,
			'machine_name' => $vnode,
		};

		($rc, $param) = $pd->queryRequestMachine($args);
		if ($rc == 0) {
			foreach my $id (keys(%$param)) {
				$vhref->{$vnode} = $param->{$id}->{'request_machine.hypervisor'};
			}
		}else{
			$args->{'request_name'} = "$reqid"."a";
			($rc, $param) = $pd->queryRequestMachine($args);
			if ($rc == 0) {
				foreach my $id (keys(%$param)) {
					$vhref->{$vnode} = $param->{$id}->{'request_machine.hypervisor'};
				}
			}else{
				$args->{'request_name'} = "$reqid"."d";
				($rc, $param) = $pd->queryRequestMachine($args);
				if ($rc == 0) {
					foreach my $id (keys(%$param)) {
						$vhref->{$vnode} = $param->{$id}->{'request_machine.hypervisor'};
					}
				}
			}
		}
	}
 
	return $vhref;
}
sub getHypvVnode {
	my ($class, $hypv) = @_;
  
	my $vhref;
 
	my @mach_array;

	my $pd = ProvisionDAO->new();
	
	my $args = {
		'hypervisor' => $hypv,
	};

	my ($rc, $param) = $pd->queryRequestMachine($args);
	if ($rc == 0) {
		foreach my $id (keys(%$param)) {
			if(defined($param->{$id}->{'request_machine.request_name'}) && 
					$param->{$id}->{'request_machine.request_name'} !~ /d$/) {
				push(@mach_array, $param->{$id}->{'request_machine.machine_name'});
			}
		}
	}
 
	return \@mach_array;
}


sub getAllvNode {
	my $kvmtype = "IBM_KVM";

	my $sysconf = CloudUtils->getConfig();
	my @types = split(",", $sysconf->{'machTypeName'});
	foreach (@types) {
		if (/kvm/i) {
			$kvmtype = $_;
		}
	}

	my $pd = MachineDAO->new();

	my ($rc, $param) = $pd->queryAllMachineName();
	if ($rc != 0) {
		return $rc;
	}

	my $mach_names = $param;
	my @mach_array;
	my $args = {
		'type' => $kvmtype,
	};

	foreach my $name (@$mach_names) {
		$args->{'name'} = $name;
		($rc, $param) = $pd->queryMachine($args);
		if ($rc == 0) {
			push(@mach_array, $name);
		}
	}

	return \@mach_array;
}


sub addRole {
	my ($class, $role, $comments) = @_;
	my $attr = {};

	my $ud = UserDAO->new();
	my ($rc, $param) = $ud->queryRole({'name'=>$role});
	if ($rc == 0) {
		return;
	}

	if ($comments) {
		$attr->{'desc'} = $comments;
	}

	($rc, $param) = $ud->insertRole($role,$attr);
	if ($rc != 0) {
		$ud->rollback();
		return;
	}

	$ud->commit();
	return 1;
}

sub removeRole {
	my ($class, $role) = @_;

	my $ud = UserDAO->new();

	my ($rc, $param) = $ud->deleteRoleByName($role);
	if ($rc != 0) {
		$ud->rollback();
		return;
	}

	$ud->commit();
	return 1;
}

sub getAllCapability {
	my $ud = UserDAO->new();

	my ($rc, $param) = $ud->queryAllCapability();
	if ($rc == 0) {
		return $param;
	}

	return undef;
}

sub getAllRole {
	my $ud = UserDAO->new();
	my $result = [];

	my ($rc, $param) = $ud->queryAllRole();
	if ($rc != 0) {
		return;
	}

	foreach my $i (keys %$param) {
		push(@$result, $i);
	}

	return $result;
}

sub changeUserAttribute {
	my ($class, $userattr) = @_;
	if(!defined($userattr)){
		return (0, "Invalid input.");
	}	
	my $user = $userattr->{'userid'};
	if (!defined($user)) {
		return (0, "Error input user");
	}
	my $current_user = getpwuid($<);
	if (CloudUtils->checkCapability($current_user, "modify-user")== 0){
		if($current_user ne $user) {
			Logger->get()->error('User', "$user", "User $current_user No permission to modify user.");
			return (0, "No permission to do this operation."); 
		}
		if(exists($userattr->{'recharge'})){
			Logger->get()->error('User', "$user", "User $current_user No permission to recharge.");
			return (0, "No permission to do this operation."); 
		}

	}
	delete($userattr->{'userid'});
	if(scalar(keys(%$userattr)) == 0){
		return (100, "Nothing to be requested to modify.");
	}
	my $price = undef;
	if(exists($userattr->{'recharge'})){
		$price = $userattr->{'recharge'}*100;
		delete($userattr->{'recharge'});
	}
	my $balance = 0;
	if(defined($price)){
		$balance = BillingDemo->getUserAccount($user, 1);
		$balance += $price;
		my $rc = BillingDemo->setUserAccount($user, $balance);
		if($rc != 1){
			return (0, "Update user $user balance failed.");
		}
	}
	my $preshell = undef;
	my @currshell = ();
	if(exists($userattr->{'shell'})){
		push(@currshell, "/bin/$userattr->{'shell'}");
		my ($rc, $userInfo) = CloudUtils->querySysUser($user);
		$preshell= $userInfo->{$user}->{"loginShell"};
		if($preshell->[0] ne $currshell[0]){
			$rc = CloudUtils->modSysUsershell($user, \@currshell);
			if($rc == 0){
				if(defined($price)){
				#restore the balance
					BillingDemo->setUserAccount($user, $balance - $price);
				}
				return (0, "Modify user $user shell failed.");
			}
		}else{
			$preshell = undef;
		}
		delete($userattr->{'shell'});
	}
	if(scalar(keys(%$userattr)) != 0){
   		my $ud = UserDAO->new();
		my ($rc, $param) = $ud->modifyUser({'name'=>$user}, $userattr);
		if ($rc == 0) {
			$ud->commit();
			return (1);
		}
		if(defined($price)){
		#restore the balance
			BillingDemo->setUserAccount($user, $balance - $price);
		}
		if(defined($preshell)){
			CloudUtils->modSysUsershell($user, $preshell);
		}
		$ud->rollback();
		return (0, "$param");
	}
	return (1);
}

sub changeUserShell {
	my ($class, $user, $newshell) = @_;
	my $attr = {};
	my $error_str;

	if (!$user) {
		return (0, "Error input user");
	}

	my $current_user = getpwuid($<);
	if ((CloudUtils->checkCapability($current_user, "modify-user")== 0)&&($current_user ne $user)) {
	Logger->get()->error('User', "$user", "User $current_user has no permission to modify user $user.");
	return (0, "No permission to do this operation."); 
	}

	if ($newshell) {
		$attr->{'shell'} = $newshell;
	}

	use Net::LDAP;
	use LDAPUtils;

	my $ldapc = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	$ldapc->bind();

	my ($rc, $param) = $ldapc->modUserShell({'name'=>$user}, $attr);

	if ($rc != 0) {
		Logger->get()->error('User', "$user", "Failed to modify shell for user $user, due to $error_str.");
		$ldapc->unbind();
		return (0, "$error_str");
	}

	$ldapc->unbind();

	return (0, "$param");
}

sub getProvAndEnergyInfoById {
	my ($class, $pid, $flag, $fuzzy) = @_;
	my $result;
	my $current_energy = 0;
	$result=CloudUtils->getProvInfoById($pid, $flag, $fuzzy);

	if (!$result) {
		return undef;
	}

	if (!($result->{'request.energy'})) {
		$current_energy = EnergyUtils->getEnergyOfProv($pid);
		$result->{'request.energy'} = $current_energy;
	}

	return $result;
}
=description
The function can get all information of a request queried by status.
$status:
	Input like "Ready,Waiting";

$fuzzy:
	Refer the define of queryRequest
$flag:
	0 Array Ref
	1 Hash Ref
=cut
sub getProvAndEnergyInfoByStatus{
    my ($class, $status, $fuzzy, $flag) = @_;
	my @query=();
	my @queryHistory=("Failed","Expired","Canceled");
	my @queryEnergy=("Ready");
	my @qStatus=split(",", $status);
	my $pd = ProvisionDAO->new();
	my $rc;
	my $param={};
	my @qRun = CloudUtils->diff(\@qStatus, \@queryEnergy); 
#Check if the Ready status exist
	if(scalar(@qRun) != scalar(@qStatus)){
		my $eachQuery->{"status"}="Ready";
		push(@query, $eachQuery);
		($rc, $param) = $pd->queryRequest(\@query, $flag, $fuzzy);
		if($rc == $GeneralDefine::FAILED){
			return undef;
		}elsif($rc == $GeneralDefine::NORECORD){
			$param = {};
		}else{
			foreach my $pid(keys(%$param)){
        		my $current_energy = EnergyUtils->getEnergyOfProv($pid);
        		$param->{$pid}->{'request.energy'} = $current_energy;
    		}
		}
		@query = ();
	}
#Check all non-end statuts 
	@qStatus = CloudUtils->diff(\@qRun, \@queryHistory); 
	if(scalar(@qStatus) > 0){
		foreach (@qStatus){
			my $eachQuery->{"status"}=$_;
			push(@query, $eachQuery);
		}
		my $tparam;
		($rc, $tparam) = $pd->queryRequest(\@query, $flag, $fuzzy);
		if($rc == $GeneralDefine::FAILED){
			return undef;
		}elsif($rc == $GeneralDefine::SUCCESS){
			$param = {%$param,%$tparam};
		}
		@query = ();
	}
	my $paramH;
#Check end status
	@qStatus = CloudUtils->diff(\@qRun, \@qStatus);
	if(scalar(@qStatus) > 0){
		foreach (@qStatus){
			my $eachQuery->{"status"}=$_;
			push(@query, $eachQuery);
		}
		my $tparam;
		($rc, $tparam) = $pd->queryRequestfromhistory(\@query, $flag);
		if($rc == $GeneralDefine::FAILED){
			return undef;
		}elsif($rc == $GeneralDefine::SUCCESS){
			$param = {%$param,%$tparam};
		}
	}
	return $param;
}

sub isVMalive {
	my ($class, $mach) = @_;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryRequestMachine({'hypervisor' => $mach});
	if ($rc == 0) {
		return 1;
	}

	return 0;
}

sub getVMCountonMach {
	my ($class, $mach) = @_;

	my $count = 0;

	my $pd = ProvisionDAO->new();

	my ($rc, $param) = $pd->queryRequestMachine({'hypervisor' => $mach});
	if ($rc != 0) {
		return $count;
	}

	foreach my $id (keys(%$param)) {
		++$count;
	}

	return $count;
}

sub isaliveVMonit {
	my ($class, $reqid, $machlistarray) = @_;

	my $prov = CloudUtils->getProvInfoById($reqid, 1);
	if (!$prov || $prov->{'request.hypv_flag'} != 1) {
		return 0;
	}
	
	my @mach_list;
	if(defined($machlistarray)){
		@mach_list = @$machlistarray;
	}else{
		@mach_list = keys(%{$prov->{'machList'}});
	}
	foreach my $mach (@mach_list) {
		if(CloudUtils->isVMalive($mach)){
			return 1;
		}
	}

	return 0;
}

=comments
Description: it is used to generete VM name
Input:
		$prefix VM prefix
		$num	the VM number
Return:
		VM name which combined by prefix and num
=cut
sub formatVMName{
	my ($class, $prefix, $num)=@_;
	my $format=$prefix;
	$format=~ s/.*%/%/;
	$prefix=~ s/(.*)%.*/$1/;
	my $num_str="";
	my $cur_node;
	if($format ne ""){
		if($format =~ /^%\d*d$/){
			$num_str = sprintf("$format", $num);
		}
	}
	if($num_str ne ""){
		$cur_node = join("", $prefix, $num_str);
	}
	else{
		$cur_node = join("", $prefix, $num);
	}   
	return ($cur_node);
}

=comments
Description: Query all provison information which is VM
Input:
			NA
Return:
			Reference of provision inforamtion for all VM
=cut
sub getAllVMInfo{
	my ($class) = @_;
	my $prov_info->{'hypv_flag'} = -1;
	my $pd = ProvisionDAO->new();
	my ($rc, $param) = $pd->queryRequest($prov_info);
	if ($rc != 0) {
			#log the error $DBI::errstr
		return undef;
	}
	return $param;
}

=comments
Description: it is used to remove VM from thc_machine table; 
Input:
		$prefix VM prefix
		$rmvNum	the VM number
Return:
		0 means failed, 1 means ok		
=cut
sub removeVM{
	my ($class, $rmvNum, $prefix) = @_;
	my $sysconf = CloudUtils->getConfig();
	my $maxKvmVms = $sysconf->{'maxKvmVms'};
	if($rmvNum <= 0){
		return (0);
	}
	my $md = MachineDAO->new();
	while($rmvNum > 0){
		my $cur_node = CloudUtils->formatVMName($prefix, ($maxKvmVms-$rmvNum+1));
		my $arg={
			'name' => $cur_node,
		};
		my $rc  = $md->deleteMachine($arg);
		if ($rc != 0) {
			$md->rollback();
			return (0);
		}
		$rmvNum--;
	}
	$md->commit();
	return (1);
}

=comments
Description: it is used to check each input item whether it is 
			 Can be changed or not.
Input:
		$name changed sysconfig name
		$value changed syconfig value
		$related_value  related value with the changed sysconfig item
Return:
		100 means no change
		0 means failed and a extra return value is the error reason
		1 means sucess and a extra return value is the difference between changed value and previous value	
=cut
sub sysConfChangeCheck{
	my ($class, $name, $value, $related_value, $type) = @_;
	my $sysconf = CloudUtils->getConfig();
	my $cur_node;
	my $invalid_status_str="Failed,Expired,FAIL,Canceled,None";
	my $vmprv_list = CloudUtils->getAllVMInfo();
	my $new_host_num;
	while(my ($vmprv_key, $vmprv_value)=each(%$vmprv_list)){
		my $prov_status=$vmprv_value->{'request.status'};
		if($invalid_status_str =~ /$prov_status/){
			delete $vmprv_list->{$vmprv_key};
		}
	}	
	if(!defined($type)){
		$type = "Number";
	}
	if($type eq "Number"){
		unless(defined($sysconf->{$name})){
			$sysconf->{$name} = 0;
		}
		if(($sysconf->{$name} - $value)==0){
			if(($name eq "maxKvmVms") and defined($related_value) and ($related_value ne "")){
				return (1, $value);
			}
			return (100, 0);#no change
		}
		elsif(($sysconf->{$name} - $value)<0){
			if($name eq "maxKvmVms"){
				if(!defined($related_value)||($related_value eq "")){
					return (0, "The prefix of virtual machine is blank");
				}
				$new_host_num=$value;
				while($new_host_num > $sysconf->{$name}){
					$cur_node = CloudUtils->formatVMName($related_value, $new_host_num);
					my $ip = gethostbyname($cur_node);
					if (!(defined $ip)){
						return (0, "Can not find virtual machine $cur_node, prefix error or /etc/hosts not updated");
					}
					$new_host_num--;
				}
			}#end maxKvmVms checking
		}
		else{
			if($name eq "maxKvmVms"){
				if(!defined($related_value)||($related_value eq "")){
					return (0, "The prefix of virtual machine is blank");
				}
				if($value < $sysconf->{'UsedKvmVms'}){
					return (0, "UsedKvmVms greater than maxKvmVms");
				}
				$new_host_num=$sysconf->{$name};
				while($new_host_num > $value){
					$cur_node = CloudUtils->formatVMName($related_value, $new_host_num);
					while(my($vm_key, $vm_value)=each(%$vmprv_list)){
						foreach my $machInfoArray(@{$vm_value->{'machList'}}){
								if($machInfoArray->{'machine.name'} eq $cur_node){
									return (0, "$cur_node is used. Can not change maxKvmVms less than $new_host_num");
								}
						}
					}
					$new_host_num--;
				}
			}
			elsif($name eq "maxHypvNums"){
				if($sysconf->{'UsedHypvNums'} > $value){
					return (0, "UsedHypvNums greater than maxHypvNums");
				}
				elsif($value * $related_value < $sysconf->{'UsedKvmVms'}){
					return (0, "UsedKvmVms greater than maxHypvNums*maxVmsPerHypv");
				}
			}
			elsif($name eq "maxvCpusPerVm"){
				foreach my $vm_prv (keys(%$vmprv_list)){
					if($vmprv_list->{$vm_prv}->{'request.machine_cpu'} > $value){
						return (0, "Used CPU number greater than maxvCpusPerVm");
					}
				}
			}
			elsif($name eq "maxVmsPerHypv"){
=comments
		1. new changed value shall make maxVmsPerHypv*maxHypvNums >= UsedKvmVms;
		2. Make sure running vm number per hypv <= new maxVmsPerHypv;
		3. We also need consider potential running VM has enough hypv,
			so min(maxKvmVms, maxVmsPerHypv*maxHypvNums)<=maxVmsPerHypv*maxHypvNums
		   It must be true, need not checking
=cut
				if($value * $related_value < $sysconf->{'UsedKvmVms'}){
					return (0, "UsedKvmVms greater than maxHypvNums*maxVmsPerHypv");
				}
				my $hypv_list = CloudUtils->getAllHypv();
				my $maxUsedVMpHypv = 0;
				foreach my $hypv_name (@$hypv_list){
					my $vmnum = CloudUtils->getVMCountonMach($hypv_name);
					$maxUsedVMpHypv = $vmnum > $maxUsedVMpHypv ? $vmnum : $maxUsedVMpHypv;
				}	
				if ($maxUsedVMpHypv > $value){
					return (0, "Alive VM number per Hypervisor greater than maxVmsPerHypv");
				}
			}
		}
	}#end if($type eq "Number")
	elsif($type eq "String"){
		unless(defined($sysconf->{$name})){
			$sysconf->{$name} = "";
		}
		if($sysconf->{$name} eq $value){
			return (100, 0);
		}else{
			if($name eq "machTypeName"){
				my @origType = split(",", $sysconf->{$name});
				my @newType = split(",", $value);
				my @rmType= CloudUtils->diff(\@origType, \@newType);
				my @addType= CloudUtils->diff(\@newType, \@origType);
				if((scalar(@addType)==0) and (scalar(@rmType)==0)){
					return (100, 0);
				}else{
					if(scalar(@rmType) != 0){
						my($rc, $param) = CloudUtils->QueryMachByType(\@rmType);
						if($rc != $GeneralDefine::NORECORD){
							return (0, "Can not remove the type(s) by @rmType still used by machine.");
						}
					}
					my $rtvstr = sprintf("[%s]:[%s]", join(",",@addType), join(",",@rmType));
					return (1, $rtvstr);
				}
			}else{
				return (1, 0);
			}
		}
	}
	return (1, ($value - $sysconf->{$name}));
}

sub QueryMachByType{
	my ($class, $typeRef) = @_;
	my $confd = GeneralDAO->new();
	my @conditions = ();
	my @qeuryField = ("name");
	foreach (@$typeRef){
		push(@conditions, "type,$_");
	}
	my ($rc, $param) = $confd->query("machine", "name", \@qeuryField, \@conditions, "or");
	return ($rc, $param);
}
=comments
	The function used to update the thc_switchportconfig and also update the switchs table of xCat, if failed, it shall be response restore everything
	input:
		action: add, delete, update
		hash table: the key shall be the cloumn
=cut
sub updateSwitchPortConf{
	my ($class, $action, $element) = @_;
	my @conditions=("node_name,$element->{'node_name'}");
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(getpwuid($<) ne "cloudadm"){
		return ($GeneralDefine::FAILED, "Only cloudadm has the permission to update or add switch.");	
	}
	if($action eq "add" or $action eq "update"){
		($rc, $param) = $confd->query("switchportconfig", "node_name", undef, \@conditions);
		if($rc == $GeneralDefine::FAILED){
			goto FAIL;
		}
		if(($rc eq $GeneralDefine::SUCCESS) and ($action eq "update")){#already exist
			($rc, $param)=$confd->modify("switchportconfig", $element, \@conditions);
		}elsif($action eq "add"){
			($rc, $param)=$confd->insert("switchportconfig", $element);
		}else{
			Logger->get()->warning('CloudUtils', 'updateSwitchPortConf', "$action nodeName $element->{'node_name'} not be consistent with DB record!\n");
			$param = "$action nodeName $element->{'node_name'} not be consistent with DB record!";
			goto FAIL;
		}
	}else{#delete
		($rc, $param)=$confd->delete("switchportconfig", \@conditions);
	}
	if ($rc ne $GeneralDefine::SUCCESS) {
		goto FAIL;
	}

	#issue update switchs in xCat
	my $ret=xCatUtils->mapSwitchportconfig2xCat('switch', $action, $element);
	if ($ret ne $GeneralDefine::SUCCESS){
		Logger->get()->warning('CloudUtils', 'updateSwitchPortConf', "$action nodeName $element->{'node_name'} to switch table failed!");
		goto FAIL;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;
FAIL:
	$confd->rollback();
	return ($GeneralDefine::FAILED, $param);
	
}

=comments
	The function used to update the thc_switchconfig and xCat switches table and also update /etc/hosts, if any failed, it will restore back
	input:
		action: add, delete, update
		hash table: the key shall be the cloumn
=cut
sub updateSwitchConf{
	my ($class, $action, $argv) = @_;
	my $confd = GeneralDAO->new();
	my $element={
		"switch_name"=>$argv->{'switchName'},
		"ipaddress"=>$argv->{'ipAddress'},
	};
	if(getpwuid($<) ne "cloudadm"){
		return ($GeneralDefine::FAILED, "Only cloudadm has the permission to update or add switch.");	
	}
	if(exists($argv->{'switchPortNum'})){
		$element->{"port_number"}=$argv->{'switchPortNum'};
	}
	if(exists($argv->{'comments'})){
		$element->{"description"}=$argv->{'comments'};
	}
	my @conditions=("switch_name,$element->{'switch_name'}");
	my $rc;
	my $param;
	if($action eq "add" or $action eq "update"){
		($rc, $param) = $confd->query("switchconfig", "switch_name", undef,  \@conditions);
		if($rc == $GeneralDefine::FAILED){
			return ($GeneralDefine::FAILED, "$param");
		}
		if(($rc eq $GeneralDefine::SUCCESS) and ($action eq "update")){#already exist
			($rc, $param)=$confd->modify("switchconfig", $element, \@conditions);
		}elsif($action eq "add" ){
			($rc, $param)=$confd->insert("switchconfig", $element);
		}else{
			Logger->get()->warning('CloudUtils', 'updateSwitchConf', "$action switch_name $element->{'switch_name'} not be consistent with DB record!\n");
			$param = "$action switch_name $element->{'switch_name'} not be consistent with DB record!";
			goto FAIL;
		}
	}else{#delete
			($rc, $param)=$confd->delete("switchconfig", \@conditions);
	}
	if ($rc ne $GeneralDefine::SUCCESS) {
			goto FAIL;
	}
	if(defined($element->{'ipaddress'})){
		#update it into hosts file
		$rc = CloudUtils->updateNode2hosts($action, $element->{'switch_name'}, $element->{'ipaddress'}, $argv->{"switchLongName"});
		if ($rc ne $GeneralDefine::SUCCESS){
			Logger->get()->warning('CloudUtils', 'updateSwitchConf', "$action switch_name $element->{'switch_name'} to hosts failed!\n");
			$param = "$action switch_name $element->{'switch_name'} to hosts failed!";
			goto FAIL;
		}
	}
	#issue update switchs in xCat
	my $ret=xCatUtils->mapSwitchconfig2xCat('switches', $action, $argv);
	if ($ret ne $GeneralDefine::SUCCESS){
		Logger->get()->warning('CloudUtils', 'updateSwitchConf', "$action switch_name $element->{'switch_name'} to switches failed!\n");
		$param = "$action switch_name $element->{'switch_name'} to DB failed!";
		goto FAIL;
	}
	
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;
FAIL:
	$confd->rollback();
	my $cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  '/bin/cp -f /etc/hosts.bak /etc/hosts'";
	system("$cmd");
	return ($GeneralDefine::FAILED, $param);
	
}
sub getnode2switchinfo{
	my ($class, $element) = @_;
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(defined $element){
		if(ref $element eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$element)){
				push(@tmp_ele, "$elekey,$element->{$elekey}");
			}
			$element= \@tmp_ele;
		}elsif(ref $element ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	($rc, $param) = $confd->query("switchportconfig", "node_name", undef, $element);
	if ($rc eq $GeneralDefine::FAILED) {
	   	return $GeneralDefine::FAILED;	
	}
	else
	{
		if($rc eq $GeneralDefine::NORECORD){
			return ($GeneralDefine::SUCCESS);
		}else{
			return ($GeneralDefine::SUCCESS, $param);
		}
	}
}

sub getswitchinfo{
	my ($class, $element) = @_;
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;

	if(defined $element){
		if(ref $element eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$element)){
				push(@tmp_ele, "$elekey,$element->{$elekey}");
			}
			$element= \@tmp_ele;
		}elsif(ref $element ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	($rc, $param) = $confd->query("switchconfig", "switch_name", undef, $element);
	if ($rc eq $GeneralDefine::FAILED) {
		return $GeneralDefine::FAILED;	
	}
	else
	{
		if($rc eq $GeneralDefine::NORECORD){
			return ($GeneralDefine::SUCCESS);
		}else{
			my $result = xCatUtils->dumpTab2Hash('switches');
			my @switchNames = keys(%$param);
			foreach my $switchname(@switchNames){
			$param->{"$switchname"}->{'xcatInfo'}=$result->{"$switchname"};
			}
			return ($GeneralDefine::SUCCESS, $param);
		}
	}
}
sub updateNode2hosts{
	my  ($class, $action, $node, $ip, $longname) = @_;
	#remove the request host first, whatever add, delete or modify
	my $tmpStr="sed \'/\\<$node\\>/\'d /etc/hosts > /etc/hosts.tmp";
	my $cmd="/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  \"$tmpStr\"";
	my $rc;
	eval{
		$rc=system("$cmd");
	};
	if($@) {
		Logger->get()->warning('CloudUtils', 'updateNode2hosts', "catch implement $cmd crash.");
		return ($GeneralDefine::FAILED);
	}
	if($rc){
		Logger->get()->warning('CloudUtils', 'updateNode2hosts', "Implement $cmd failed!");
		return ($GeneralDefine::FAILED);
	}
	#add the request to the end of the file
	if($action ne "delete"){
		if(!defined($longname)){
			$longname="";
		}
		my $tmpStr=sprintf("echo  %s %s %s >> /etc/hosts.tmp", $ip, $longname, $node);
		$cmd="/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  \'$tmpStr\'";
		eval{
		$rc=system("$cmd");
		};
		if($@) {
			Logger->get()->warning('CloudUtils', 'updateNode2hosts', "catch implement $cmd crash.");
			return ($GeneralDefine::FAILED);
		}
		if($rc){
			Logger->get()->warning('CloudUtils', 'updateNode2hosts', "Implement $cmd failed!");
			return ($GeneralDefine::FAILED);
		}
	}
	#make a backup to restore
	$cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  '/bin/cp -f  /etc/hosts /etc/hosts.bak'";
	eval{
		$rc=system("$cmd");
	};
	if($@) {
		Logger->get()->warning('CloudUtils', 'updateNode2hosts', "catch implement $cmd crash.");
		return ($GeneralDefine::FAILED);
	}
	#enable the new one
	$cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root  '/bin/mv  /etc/hosts.tmp /etc/hosts'";
	eval{
		$rc=system("$cmd");
	};
	if($@) {
		Logger->get()->warning('CloudUtils', 'updateNode2hosts', "catch implement $cmd crash.");
		return ($GeneralDefine::FAILED);
	}
	if($rc){
		Logger->get()->warning('CloudUtils', 'updateNode2hosts', "Implement $cmd failed!");
		return ($GeneralDefine::FAILED);
	}
	return $GeneralDefine::SUCCESS;		
}

=comments
	The function used to update the input hash to related db table;
	input:
		clean_flag: 1 means clear the table first 
		table
		element: the content of input to db
		condition: since maybe modify, so we need where $condition, it is also a Array(key1,value1 key2,value2)
=cut
sub updateHash2DB{
	my ($class, $clean_flag,$table, $element, $conditions) = @_;
	if(!defined($element) || !defined"$table" || !defined($conditions)){
		Logger->get()->warning('CloudUtils', 'updateHash2DB', "Require input element, table and conditions");
		return ($GeneralDefine::FAILED, "Require input element, table and conditions");
	}
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	my $action = "insert";
	if($clean_flag eq $GeneralDefine::CLEAR){
		($rc, $param) = $confd->delete($table);
		if($rc == $GeneralDefine::FAILED){
			goto FAIL;
		}
	}else{
		if(ref $conditions eq 'ARRAY'){
			my $ref=$$conditions[0];
			my ($tmpKey, $tmpV) = split(",", $ref);
			($rc, $param) = $confd->query($table,$tmpKey, undef, $conditions);
		}elsif(ref $conditions eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$conditions)){
				push(@tmp_ele, "$elekey,$conditions->{$elekey}");
			}
			my @ref = keys(%$conditions);
			($rc, $param) = $confd->query($table,$ref[0] , undef,  \@tmp_ele);
		}else{
			$rc = $GeneralDefine::FAILED;
		}
		if($rc == $GeneralDefine::FAILED){
			goto FAIL;
		}
		if($rc eq $GeneralDefine::SUCCESS){#already exist
			$action = "modify";
		}
	}
	($rc, $param)=$confd->$action($table, $element, $conditions);
	if ($rc ne $GeneralDefine::SUCCESS) {
		goto FAIL;
	}
	$confd->commit();
	return ($GeneralDefine::SUCCESS);
FAIL:
	$confd->rollback();
	Logger->get()->warning('CloudUtils', 'updateHash2DB', "Operate $table failed. Reason:$param");
	return ($GeneralDefine::FAILED, "Operate $table failed. Reason:$param");
	
}

=comments
	This function used to query the node name connect to the switch
	$switch_name; String
	Return the result
=cut 
sub getAllMachines2Switch{
	my ($class, $switch_name) = @_;
	my $confd = GeneralDAO->new();
	my @conditions=("switch_name,$switch_name");
	my ($rc, $param) = $confd->query("switchportconfig", "node_name", undef,  \@conditions);
	if($rc == $GeneralDefine::FAILED){
		return($GeneralDefine::FAILED);
	}
	my @mach=qw//;
	if($rc == $GeneralDefine::SUCCESS)
	{	@mach=keys(%$param);}
	return (\@mach);
}

sub setup_machine {
	my ($class, $machines, $trait) = @_;

	if (scalar(@$machines) == 0) {
		return;
	}
	my $rv = 0;
	my @outs;

	my $machlist = join(',', sort(@$machines));

	if ($trait->{'use_vm'}) {
		($rv, @outs) = CmdUtils->runxcatcmd("mkvm $machlist -f");
		if ($rv != 0) {
			print("mkvm failed.\n");
			goto failed;
		}
	}

	foreach my $mach (@$machines) {
		my $mach_info = $trait->{'machines'}->{$mach};
		my $chdef = "chdef -t node -o $mach";

		if (exists($mach_info->{'os'})) {
			$chdef .= " os=$mach_info->{'os'}";
		}

		if (exists($mach_info->{'profile'})) {
			$chdef .= " profile=$mach_info->{'profile'}";
		}

		if (exists($mach_info->{'netboot'})) {
			$chdef .= " netboot=$mach_info->{'netboot'}";
		}

		($rv, @outs) = CmdUtils->runxcatcmd($chdef);
		if ($rv != 0) {
			print("chdef failed.\n");
			goto failed;
		}
		my $image = $mach_info->{'os'} . '-' . $mach_info->{'profile'};
		my $image_info;
		($rv, $image_info) = CloudUtils->getImageInfo({'name' => $image});
		if($rv ne '0'){
			$image =~ /(.+?)-(.+)/;
			$image_info={};
			$image_info->{$image}->{'image.base'} = $1;
			$image_info->{$image}->{'image.profile'} = $2;
		}

		# check if this machine is managed by a BMC or by IPMI
		# the bootseq commands are different for these mgt modules
		my $chknodemgt = "lsdef -t node -o $mach | grep mgt";
		($rv, @outs) = CmdUtils->runxcatcmd($chknodemgt);
		if ($rv == 0) {
			my $result = join('', @outs);
			my $bootseqcmd = "";
			# if this is a disk installation, set the bootseq to hard disk
			# else set it to network boot
			if ($result =~ /ipmi/) {
				$bootseqcmd = "rsetboot $mach net";	
			} elsif (($result =~ /bmc/) || ($result =~ /blade/)) {
				if ((defined($image_info->{$image}->{'image.install_to_disk'}) and ($image_info->{$image}->{'image.install_to_disk'} eq "yes"))
					   or ($image_info->{$image}->{'image.profile'} eq "kvm"))	{
					$bootseqcmd = "rbootseq $mach net,hd0";	
				}else{
					$bootseqcmd = "rbootseq $mach net";	
				}	
			}else{
				unless($result =~ /kvm/){
					print("Error: $mach is not managed by BMC or IPMI.\n");
					goto failed;
				}
			} 
			if($bootseqcmd ne "") {
				($rv, @outs) = CmdUtils->runxcatcmd($bootseqcmd, undef, 3);
				if ($rv != 0){
					print("Error: \"$bootseqcmd\" failed.\n");
					goto failed;
				}
			}
		}else{
			print("Error: \"$chknodemgt\" failed.\n");
			goto failed;
		}	
	}

	my $vlan_info;
	if (exists($trait->{'use_vlan'})) {
		if ($trait->{'modify'} && $trait->{'vlan_id'} > 0) {
			# join exist vlan
			($rv, $vlan_info) = ProvUtils->join_vlan($trait->{'request_id'}, $trait->{'vlan_id'}, $machines);
			if ($rv != 0) {
				print("ProvUtils->join_vlan failed.\n");
				goto failed;
			}
		}
		else {
			# setup new vlan
			($rv, $vlan_info) = ProvUtils->setup_vlan($trait->{'request_id'}, $machines);
			if ($rv != 0) {
				print("ProvUtils->setup_vlan failed.\n");
				goto failed;
			}
		}
	}

	get_postscripts($trait, $vlan_info);

	foreach my $mach (@$machines) {
		my $mach_info = $trait->{'machines'}->{$mach};
		my $chdef = "chdef -t node -o $mach postbootscripts=$mach_info->{'postbootscripts'}";
		$chdef .= " postscripts=$mach_info->{'postscripts'}";

		($rv, @outs) = CmdUtils->runxcatcmd($chdef);
		if ($rv != 0) {
			print("chdef failed.\n");
			goto failed;
		}
	}

	($rv, @outs) = CmdUtils->runxcatcmd("nodeset $machlist $trait->{'install'}", undef, 3);
	if ($rv != 0) {
		print("nodeset failed.\n");
		goto failed;
	}

	($rv, @outs) = CmdUtils->runxcatcmd("rpower $machlist boot");
	if ($rv != 0) {
		print("rpower failed.\n");
	}else{
# To work around the xcat dns issue, we copy /etc/hosts into /hpcc/var and then the post script can get it
		CmdUtils->runxcatcmd("cp -rf /etc/hosts /hpcc/var/hosts", "root");
		CmdUtils->runxcatcmd("chmod a+r /hpcc/var/hosts", "root");
		CloudUtils->check_xcat_cmd("rpower $machlist status", "on", 5, 12);
	}
failed:
	return $rv;
}

sub check_xcat_cmd{
	my ($class, $xcatcmd, $expected_result, $interval, $times) = @_;
	if(!defined($xcatcmd) || !defined($expected_result)){
		return $GeneralDefine::FAILED; 
	}
	if(!defined($interval)){
		$interval = 5;#5seconds
	}
	if(!defined($times)){
		$times = 2; #twice
	}
	for(;$times > 0; $times--){
		my $neednext=0;
		my ($rv, @outs) = CmdUtils->runxcatcmd("$xcatcmd");
		foreach my $eachline (@outs){
			chomp($eachline);
			if($eachline !~ /$expected_result/){
				$neednext=1;
				last;
			}
		}
		if($neednext == 0){
			last;
		}
		sleep($interval);
	}
}

sub shutdown_machine {
	my ($class, $machines, $trait) = @_;

	if (scalar(@$machines) == 0) {
		return ;
	}

	my $machlist = join(',', sort(@$machines));

	if (defined($trait->{'vlan_id'}) && $trait->{'vlan_id'} > 0) {
		if (exists($trait->{'modify'})) {
			ProvUtils->withdraw_vlan($trait->{'vlan_id'}, $machines);
		}
		else {
			ProvUtils->remove_vlan($trait->{'vlan_id'});
		}
	}
	# the sequence for commands rpower & rmvm & rmdef is unchangable.
	CmdUtils->runxcatcmd("rpower $machlist off");
	CloudUtils->check_xcat_cmd("rpower $machlist status", "off", 5, 12);
	if ($trait->{'use_vm'}) {
		CmdUtils->runxcatcmd("rmvm $machlist -f");
		CmdUtils->runxcatcmd("rmdef $machlist -f");
	}

	return ;
}

sub release_machine {
	my ($class, $machines, $trait) = @_;

	if (scalar(@$machines) == 0) {
		return ;
	}

	my $machlist = join(',', sort(@$machines));

	CmdUtils->runxcatcmd("updatenode $machlist stoploadl");

	# we need shutdown the share node first, case some cleaning procedues.
	CloudUtils->shutdown_machine($machines, $trait);

	my @share_machines;
	foreach my $mach (@$machines) {
		if ($trait->{'machines'}->{$mach}->{'mode'} eq 'share') {
			push(@share_machines, $mach);
		}
	}

	CloudUtils->setup_machine(\@share_machines, $trait);

	return ;
}

sub wait_machine {
	my ($class, $machines, $timeout, $trait) = @_;

	my $os = $trait->{'machines'}->{$machines->[0]}->{'os'};
	if (defined($os) and ($os eq 'imagex')) {
		my $machlist = join(' ', sort(@$machines));

		for(my $i = 0;  $i <= $timeout; $i += 10) {
			sleep(10);

			my ($rv, @outs) = CmdUtils->runcmd("/usr/bin/nmap -p22 $machlist", "root");
			my $result = join('', @outs);
			if($result =~ /22\/tcp open  ssh/){
				$result =~ /\((\d+) hosts* up\)/;
				if ($1 && ($1 eq scalar(@$machines))) {
					print "$result\n";
					print "up host num:[$1]\n";
					return 0;
				}
				if($1){
					print "up host num:[$1]\n";
				}
			}else{
				print "$result\n";
			}
		}
	}
	else {
		my $machlist = join(',', sort(@$machines));

		for(my $i = 0; $i <= $timeout; $i += 10) {
			sleep(10);

			my ($rv, @outs) = CmdUtils->runxcatcmd("lsdef $machlist | grep -w status", "root");
			my $result = join('', @outs);
			if ($result !~ /powering-off|booting|installing/) {
				return(0);
			}
		}
	}

	# failed
	return(1);
}

sub machine_trait {
	my ($class, $mach_info, $req_info, $restore) = @_;
	my $trait;

	if (defined($restore)) {
		$trait->{'restore'} = 1;
		$trait->{'cluster'} = 'Join';
		# set the user for default
		$trait->{'users'} = [];
	}
	else {
		# install procedure.
		if ($req_info->{'request.vlan_flag'}) {
			$trait->{'use_vlan'} = 1;
		}

		$trait->{'request_id'} = $req_info->{'request.name'};
		$trait->{'cluster'} = $req_info->{'request.cluster'};
		$trait->{'users'} = $req_info->{'incUsers'};
	}

	if (defined($req_info->{'request.modify'}) && $req_info->{'request.modify'} > 0) {
		$trait->{'modify'} = $req_info->{'request.modify'};
	}

	if (defined($req_info->{'request.vlan_id'})) {
		$trait->{'vlan_id'} = $req_info->{'request.vlan_id'};
	}

	if ($req_info->{'request.hypv_flag'} == -1) {
		$trait->{'use_vm'} = 1;
	} else {
		$trait->{'use_vm'} = 0;
	}

	my @machines = keys(%$mach_info);

	foreach my $mach (@machines) {
		my $minfo = $mach_info->{$mach};
		my $mach_trait;
		my $image = defined($restore) ? $minfo->{'machine.default_os'} : $req_info->{'request.machine_os'};
		if(defined($image)){
			my ($rt,$image_info) = CloudUtils->getImageInfo({'name' => $image, 'status' => 'AVAILABLE'});
			if($rt ne '0'){
				$image =~ /(.+?)-(.+)/;
				$mach_trait->{'os'} = $1;
				$mach_trait->{'profile'} = $2;
			}else{
				$mach_trait->{'os'} = $image_info->{"$image"}->{'image.base'};
				$mach_trait->{'profile'} = $image_info->{"$image"}->{'image.profile'};
				if(defined($image_info->{"$image"}->{'image.install_to_disk'})){
					$mach_trait->{'install_to_disk'} = $image_info->{"$image"}->{'image.install_to_disk'};
				}
			}
		}
		if ($minfo->{'machine.provision_mode'} == 2) {
			$mach_trait->{'mode'} = 'full';
		}
		elsif ($minfo->{'machine.provision_mode'} == 3) {
			$mach_trait->{'mode'} = 'share';
		}

		$mach_trait->{'netboot'} = "xnba";
		# setup diskful install for hypervisor and other images 
		# that must be installed to hard disk
		if ((exists($mach_trait->{'install_to_disk'}) and ($mach_trait->{'install_to_disk'} eq "yes")) || 
		     (defined($mach_trait->{'profile'}) && $mach_trait->{'profile'} eq "kvm") ){
			# for hypervisor, we really install with fulldisk.
			$trait->{'install'} = "install";
		}
		else {
			# else we just netboot with diskless.
			$trait->{'install'} = "netboot";
		}

		$trait->{'machines'}->{$mach} = $mach_trait;
	}

	return $trait;
}

sub get_postscripts {
	my ($trait, $vlan_info) = @_;

	my $request_id = $trait->{'request_id'};
	my $cluster = $trait->{'cluster'};
	my @machines = sort(keys(%{$trait->{'machines'}}));
	my $using_machs;
	my $scripts;
#we use sort to make sure CM is the first and do not sort the new name by vlan, just sort the original name.
	if ($trait->{'use_vlan'}) {
		my @New_machines = ();
		if ($trait->{'use_vm'}) {
			foreach my $vm (@machines){
				push(@New_machines, $vlan_info->{'machines'}->{$vm}->{'hostname'});
			}
		}
		else {
			foreach my $vm (@machines){
				push(@New_machines, $vlan_info->{'hypervisors'}->{$vm}->{'hostname'});
			}
		}
		$using_machs = join(' ', @New_machines);
	}
	else {
		$using_machs = join(' ', @machines);
	}

	# modify_addmachs will set $trait->{'cm'} for sub cluster
	# Add cm to using_machs, so that the new added machines will know whick sub cluster to join.
	if (defined($trait->{'cm'}) &&  $trait->{'cm'} ne "") {
		$using_machs = $trait->{'cm'} . ' ' . $using_machs;
	}

	foreach my $mach (@machines) {
		my $mach_trait = $trait->{'machines'}->{$mach};

		if (exists($trait->{'restore'}) && ($mach_trait->{'mode'} eq 'full')) {
			# do not need postscript field changed for full.
			next;
		}

		if ($trait->{'use_vlan'}) {
			$scripts = "configvlan,ntpsync";
		}
		else {
			$scripts = "ntpsync";
		}

		if ($mach_trait->{'profile'} eq 'kvm') {
			$scripts .= ",mkhyperv";
		}
		else{
			my $image = $mach_trait->{'os'} . '-' . $mach_trait->{'profile'};
			my ($rv, $image_info) = CloudUtils->getImageInfo({'name' => $image});
			if (($rv == 0) && defined($image_info->{$image}->{'image.post_scripts'})) {
				$scripts .= "," . $image_info->{$image}->{'image.post_scripts'};
			}

			if($scripts =~ /cloudadm_config_scheduler/){
				$scripts =~ s/cloudadm_config_scheduler/\"cloudadm_config_scheduler loadl $cluster $::CONFIG{'XCAT_MN'} $using_machs\"/;
			}
		}

		$scripts .= "," . make_user_authority($trait->{'users'});

		# diskfull provisioning
		if ($trait->{'install'} eq 'install') {
			$mach_trait->{'postscripts'} = $scripts;

			# Do not load cpufreq modules for VMs
			if ($trait->{'use_vm'} != 1) {
				$mach_trait->{'postbootscripts'} = "loadcpufreqmod";
			} else {
				$mach_trait->{'postbootscripts'} = "";
			}
		}
		else {
			# Do not load cpufreq modules for VMs
			if ($trait->{'use_vm'} != 1) {
				$scripts .= ",loadcpufreqmod";
			}

			# we use postbootscripts since it will be used after reboot
			$mach_trait->{'postbootscripts'} = $scripts;
			$mach_trait->{'postscripts'} = "";
		}
	}
}

sub make_user_authority {
	my ($users) = @_;

	my $script = "user_auth";

	# default user_list for login
	my $user_list = "";
	my $user = (getpwuid($<))[0];
	if ($user ne "cloudadm") {
		$user_list .= " " . $user;
	}

	if (defined($users) && (scalar(@$users) > 0)) {
		$user_list .= " " . join(" ", @$users);
	}

	if(!( $user_list=~ /root/)){
		$user_list .= " root";
	}
	if(!( $user_list=~ /cloudadm/)){
		$user_list .= " cloudadm";
	}

	#compose the postscript item with input parameter
	return "\"$script $user_list\""; 
}

sub union {
	my ($class, $u1, $u2) = @_;

	my %saw;

	my @u3;
	my @temp = @$u1;

	if (defined($u2) && (scalar(@$u2) != 0)) {
		push(@temp, @$u2);
	}

	for (my $i = 0; $i < scalar(@temp); ++$i) {
		if (!exists($saw{$temp[$i]})) {
			push(@u3, $temp[$i]);
			$saw{$temp[$i]} = 1;
		}
	}

	return @u3;
}

sub diff {
	my ($class, $u1, $u2) = @_;

	my %saw = map({$_ => 1} @$u2);

	my @u3;
	foreach (@$u1) {
		if (!exists($saw{$_})) {
			push(@u3, $_);
		}
	}

	return @u3;
}

sub uniq {
	my ($class, @array) = @_;

	my %saw;

	@saw{@array} = ();

	return sort(keys(%saw));
}

#added for image new function
=comments
	The function used to get pkglist information.
	Input:
		$conditions: Hash. The query condition.
		$queryFields: Queried result fields, it is an array, if no requirement, fill undef
	Return:
		Failed.
		Success. and a Hash for the query contents. 
=cut
sub getPkgListInfo{
	my($class, $conditions, $queryFields) = @_;
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(defined $conditions){
		if(ref $conditions eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$conditions)){
				push(@tmp_ele, "$elekey,$conditions->{$elekey}");
			}
			$conditions = \@tmp_ele;
		}elsif(ref $conditions ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	($rc, $param) = $confd->query("pkglist", "name", $queryFields, $conditions);
	if ($rc eq $GeneralDefine::FAILED) {
		return ($GeneralDefine::FAILED);	
	}
	else
	{
		if($rc eq $GeneralDefine::NORECORD){
			return ($GeneralDefine::SUCCESS);
		}else{
			return ($GeneralDefine::SUCCESS, $param);
		}
	}
}
=comments
	The function used to modify pkglist information.
	Input:
		$name: The modified pkgList name
		$conditions: Hash. Modified content.
	Return:
		Failed.
		Success. 
=cut
sub updatePkgListInfo{
	my ($class, $action, $pkg_name, $element) = @_;
	my @conditions=("name,$pkg_name");
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	my $user = getpwuid($<);
	($rc, $param) = $confd->query("pkglist", "name", undef,  \@conditions);
	if($rc == $GeneralDefine::FAILED){
		return $GeneralDefine::FAILED;
	}
	if($rc eq $GeneralDefine::SUCCESS){#already exist
		if(($action ne "update")and($action ne "delete")){
			Logger->get()->error('pkglist', "$pkg_name", "$action not right.");
			return ($GeneralDefine::FAILED);
		}
		if($action eq "update"){
			if (CloudUtils->checkCapability($user, "modify-pkglist")== 0) {
				Logger->get()->error('pkglist', "$pkg_name", "User $user No permission to modify pkglist.");
				return ($GeneralDefine::FAILED);
			}
		}
		if($action eq "delete"){
			if (CloudUtils->checkCapability($user, "delete-pkglist")== 0) {
				Logger->get()->error('pkglist', "$pkg_name", "User $user No permission to delete pkglist.");
				return ($GeneralDefine::FAILED);
			}
		}
		($rc, $param)=$confd->modify("pkglist", $element, \@conditions);
	}else{
		if($action ne "add"){
			Logger->get()->error('pkglist', "$pkg_name", "$action not right.");
			return ($GeneralDefine::FAILED);
		}
		if (CloudUtils->checkCapability($user, "create-pkglist")== 0) {
			Logger->get()->error('pkglist', "$pkg_name", "User $user No permission to create pkglist.");
			return ($GeneralDefine::FAILED);
		}
		($rc, $param)=$confd->insert("pkglist", $element);
	}
	
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;

}
=comments
	The function used to remove pkglist information.
	Input:
		$name: The removed pkgList name
	Return:
		Failed.
		Success. 
=cut
sub rmPkgList{
	my ($class, $pkg_name) = @_;
	my @conditions=("name,$pkg_name");
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	#ken opened a similar defect, if remove a non-exist record, we shall return failed but not success
	($rc, $param) = $confd->query("pkglist", "name", undef,  \@conditions);
	if($rc ne $GeneralDefine::SUCCESS){
		return $GeneralDefine::FAILED;
	}
	if(!defined($param)){
		return $GeneralDefine::FAILED;
	}
	my $user = getpwuid($<);
	if (CloudUtils->checkCapability($user, "delete-pkglist")== 0) {
		Logger->get()->error('pkglist', "$pkg_name", "User $user No permission to delete pkglist.");
		return ($GeneralDefine::FAILED);
	}
	($rc, $param)=$confd->delete("pkglist", \@conditions);
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;
}
=comments
	The function used to get script information.
	Input:
		$conditions: Hash. The query condition.
		$queryFields: Queried result fields, it is an array, if no requirement, fill undef
	Return:
		Failed.
		Success. and a Hash for the query contents. 
=cut
sub getScriptInfo{
	my($class, $conditions, $queryFields) = @_;
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(defined $conditions){
		if(ref $conditions eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$conditions)){
				push(@tmp_ele, "$elekey,$conditions->{$elekey}");
			}
			$conditions = \@tmp_ele;
		}elsif(ref $conditions ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	($rc, $param) = $confd->query("script", "name", $queryFields, $conditions);
	if ($rc eq $GeneralDefine::FAILED) {
		return $GeneralDefine::FAILED;	
	}
	else
	{
		if($rc eq $GeneralDefine::NORECORD){
			return ($GeneralDefine::SUCCESS);
		}else{
			return ($GeneralDefine::SUCCESS, $param);
		}
	}
}
=comments
	The function used to modify script information.
	Input:
		$name: The modified script name
		$conditions: Hash. Modified content.
	Return:
		Failed.
		Success. 
=cut
sub updateScriptInfo{
	my ($class, $action, $scrp_name, $element) = @_;
	my @conditions=("name,$scrp_name");
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	my $user = getpwuid($<);
	($rc, $param) = $confd->query("script", "name", undef,  \@conditions);
	if($rc == $GeneralDefine::FAILED){
		return $GeneralDefine::FAILED;
	}
	if($rc eq $GeneralDefine::SUCCESS){#already exist
		if(($action ne "update")and($action ne "delete")){
			Logger->get()->error('script', "$scrp_name", "$action not right.");
			return ($GeneralDefine::FAILED);
		}
		if($action eq "update"){
			if(CloudUtils->checkCapability($user, "modify-script")== 0) {
				Logger->get()->error('script', "$scrp_name", "User $user No permission to modify script.");
				return ($GeneralDefine::FAILED);
			}
		}
		if($action eq "delete"){
			if(CloudUtils->checkCapability($user, "delete-script")== 0) {
				Logger->get()->error('script', "$scrp_name", "User $user No permission to delete script.");
				return ($GeneralDefine::FAILED);
			}
		}
		($rc, $param)=$confd->modify("script", $element, \@conditions);
	}else{
		if($action ne "add"){
			Logger->get()->error('script', "$scrp_name", "$action not right.");
			return ($GeneralDefine::FAILED);
		}
		if(CloudUtils->checkCapability($user, "create-script")== 0) {
			Logger->get()->error('script', "$scrp_name", "User $user No permission to create script.");
			return ($GeneralDefine::FAILED);
		}
		($rc, $param)=$confd->insert("script", $element);
	}
	
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;

}
=comments
	The function used to remove script information.
	Input:
		$name: The removed script name
	Return:
		Failed.
		Success. 
=cut
sub rmScript{
	my ($class, $scrp_name) = @_;
	my @conditions=("name,$scrp_name");
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	my $user = getpwuid($<);
	#ken opened a similar defect, if remove a non-exist record, we shall return failed but not success
	($rc, $param) = $confd->query("script", "name", undef,  \@conditions);
	if($rc ne $GeneralDefine::SUCCESS){
		return $GeneralDefine::FAILED;
	}
	if(!defined($param)){
		return $GeneralDefine::FAILED;
	}
	if(CloudUtils->checkCapability($user, "delete-script")== 0) {
		Logger->get()->error('script', "$scrp_name", "User $user No permission to delete script.");
		return ($GeneralDefine::FAILED);
	}
	($rc, $param)=$confd->delete("script", \@conditions);
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;
}
=comments
	The function used to get infrastructure information.
	Input:
		$conditions: Hash. The query condition.
		$queryFields: Queried result fields, it is an array, if no requirement, fill undef
	Return:
		Failed.
		Success. and a Hash for the query contents. 
=cut
sub getInfrastructureScriptInfo{
	my($class, $conditions, $queryFields) = @_;
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(defined $conditions){
		if(ref $conditions eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$conditions)){
				push(@tmp_ele, "$elekey,$conditions->{$elekey}");
			}
			$conditions = \@tmp_ele;
		}elsif(ref $conditions ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	if(defined($queryFields)){
		push(@$queryFields, "id");
		@$queryFields = CloudUtils->union($queryFields);
	}
	($rc, $param) = $confd->query("infrastr_script", "id", $queryFields, $conditions);
	if ($rc eq $GeneralDefine::FAILED) {
		return $GeneralDefine::FAILED;	
	}
	else
	{
		if($rc eq $GeneralDefine::NORECORD){
			return ($GeneralDefine::SUCCESS);
		}else{
			return ($GeneralDefine::SUCCESS, $param);
		}
	}
}
=comments
	The function used to modify infrastructure information.
	Input:
		$conditions: The removed infrastructure script condition, normally shall be name+type
		$element: Hash. Modified content.
	Return:
		Failed.
		Success. 
=cut
sub updateInfrastructureScriptInfo{
	my ($class, $action, $conditions, $element) = @_;
	if(!defined($conditions) || !defined($element)){
		return( $GeneralDefine::FAILED);
	}
	my $user = getpwuid($<);
	if($user ne "cloudadm"){
		Logger->get()->error('script', "updateInfrastructureScriptInfo", "User $user No permission to update script.");
		return ($GeneralDefine::FAILED);
	}
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	if(ref $conditions eq 'HASH'){
		my @tmp_ele;
		foreach my $elekey(keys(%$conditions)){
			push(@tmp_ele, "$elekey,$conditions->{$elekey}");
		}
		$conditions = \@tmp_ele;
	}elsif(ref $conditions ne 'ARRAY'){
		return( $GeneralDefine::FAILED);
	}
	($rc, $param) = $confd->query("infrastr_script", "id", undef,  $conditions);
	if($rc == $GeneralDefine::FAILED){
		return $GeneralDefine::FAILED;
	}
	if($rc eq $GeneralDefine::SUCCESS){#already exist
		($rc, $param)=$confd->modify("infrastr_script", $element, $conditions);
	}else{
		($rc, $param)=$confd->insert("infrastr_script", $element);
	}
	
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;

}
=comments
	The function used to remove infrastructure script information.
	Input:
		$conditions: The removed infrastructure script condition, normally shall be name+type
	Return:
		Failed.
		Success. 
=cut
sub rmInfrastructureScript{
	my ($class, $conditions) = @_;
	my $user = getpwuid($<);
	if($user ne "cloudadm"){
		Logger->get()->error('script', "rmInfrastructureScript", "User $user No permission to delete script.");
		return ($GeneralDefine::FAILED);
	}
	my $confd = GeneralDAO->new();
	my $rc;
	my $param;
	#ken opened a similar defect, if remove a non-exist record, we shall return failed but not success
	if(defined($conditions)){
		if(ref $conditions eq 'HASH'){
			my @tmp_ele;
			foreach my $elekey(keys(%$conditions)){
				push(@tmp_ele, "$elekey,$conditions->{$elekey}");
			}
			$conditions = \@tmp_ele;
		}elsif(ref $conditions ne 'ARRAY'){
			return( $GeneralDefine::FAILED);
		}
	}
	($rc, $param) = $confd->query("infrastr_script", "id", undef,  $conditions);
	if($rc ne $GeneralDefine::SUCCESS){
		return $GeneralDefine::FAILED;
	}
	if(!defined($param)){
		return $GeneralDefine::FAILED;
	}
	($rc, $param)=$confd->delete("infrastr_script", $conditions);
	if ($rc ne $GeneralDefine::SUCCESS) {
		$confd->rollback();
		return $GeneralDefine::FAILED;
	}
	
	$confd->commit();
	return $GeneralDefine::SUCCESS;
}
=comments
	Description:
		It used to reset a user password by super user right
	Input:
		uid: Who will be reset
		resetpwd: New password
	Return:
		rtv: result of reset passowrd
		errstr: if failed, return the reason
=cut
sub resetPassword {
	my ($class, $uid, $resetpwd) = @_;
	if (!$uid) { return ($GeneralDefine::FAILED, "Lack of user id."); }
	if (!$resetpwd) { return ($GeneralDefine::FAILED, "Lack of new password."); }

	use Net::LDAP;
	use LDAPUtils;

	my $rc;
	#my $resetpwd = "changeme1";
	my $current_user = getpwuid($<);
	if($current_user ne "cloudadm"){
		return ($GeneralDefine::FAILED, "Can only cloudadm reset the password.");
	}
	my ($ldapc) = LDAPUtils->new( {'host'=>$::LDAPHost, 'dn'=>'cn=Manager,dc=thcloud.com', 'pw'=>$::LDAPADMPASSWD} );

	$ldapc->connect();
	($rc)=$ldapc->bind();

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "LDAP failed to bind for user $uid.");
		return ($GeneralDefine::FAILED, "LDAP failed to bind for user $uid");
	}

	($rc) = $ldapc->modUserPassword($uid, $resetpwd);

	if ($rc != 0) {
		Logger->get()->error('User', "$uid", "Failed to add user $uid, due to LDAP error.");
		$ldapc->unbind();
		return ($GeneralDefine::FAILED, "Failed to add user $uid, due to LDAP error.");
	}

	$ldapc->unbind();
	return ($GeneralDefine::SUCCESS);
}

sub getTrueHostName{
	my ($class, $hostname) = @_;
	if(!defined($hostname)){
		return undef;
	}
	my $confd = GeneralDAO->new();
	my @conditions = ("using_hostname,$hostname");
	my @qeuryField = ("machine_name");
	my ($rc, $param) = $confd->query("request_machine", "machine_name", \@qeuryField, \@conditions, "and");
	if($rc != $GeneralDefine::SUCCESS){
		return $hostname;
	}else{
		my @nameArray = keys(%$param);
		return $nameArray[0];
	}
}

END
{
	if ($::DBH) {
		$::DBH->disconnect; #or log $DBI::errstr
	}
}

1;
