#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/common/CmdUtils.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.61 1/31/12 04:25:29";

package CmdUtils;
use Getopt::Long;
use strict;
use warnings;
use XML::Simple;
use Time::Local;
use Utils::Logger;
#we need semaphoe lock to protect some actions
use Fcntl qw(:flock);

#-------------------------------------------------------------------------------
=head1  CmdUtils
=head2    Package Description
  HPCC CmdUtils module. Some commands such as rpower have different implementations  for different hardware. This module holds the usage string for these kind
of commands so that the usage can be referenced from different modules.
=cut
#-------------------------------------------------------------------------------

%CmdUtils::NameConvert = (
# User:
    'user.name' => 'user_name',
    'user.comments' => 'comments',
    'user.department' => 'department',
    'user.last_name' => 'LN',
    'user.first_name' => 'FN',

# UserGroup:
    'user_group.name' => 'group_name',
    'user_group.id' => 'id',
    'user_group.owner' => 'owner',

# Capability:
    'capability.name' => 'name',

# Role:
    'role.name' => 'name',
    'role.id' => 'id',

# Image:
    'image.arch' => 'arch',
    'image.base' => 'base',
	'image.profile' => 'profile',
    'image.id' => 'id',
    'image.name' => 'name',
    'image.approved_by' => 'approver',
    'image.captured_by' => 'capturer',
    'image.captured_on' => 'source',
    'image.captured_time' => 'create',
    'image.description' => 'description',
    'image.status' => 'status',
    'image.pkglist_path' => 'pkglist',
    'image.pkg_location' => 'pkg_location',
    'image.scheduler' => 'scheduler',
    'image.parallel_env' => 'parallel_env',
    'image.pre_scripts' => 'pre_scripts',
    'image.post_scripts' => 'post_scripts',
    'image.interconnect' => 'interconnect',
    'image.shared_fs' => 'shared_fs',
    'image.install_to_disk' => 'install_to_disk',

# Event:
    'event.id' => 'id',
    'event.message' => 'message',
    'event.object' => 'object',
    'event.operater' => 'operater',
    'event.originator' => 'originator',
    'event.time' => 'time',
    'event.type' => 'type',

# Log:
    'log.caller' => 'caller',
    'log.id' => 'id',
    'log.level' => 'level',
    'log.message' => 'message',
    'log.object' => 'object',
    'log.time' => 'time',
    'log.type' => 'type',
    'log.pid' => 'pid',

# MachineGroup:
    'machine_group.id' => 'id',
    'machine_group.name' => 'name',
    'machine_group.memory_size' => 'memory_size',
    'machine_group.num_ethernet_interfaces' => 'num_ethernet_interfaces',
    'machine_group.num_high_speed_interfaces' => 'num_high_speed_interfaces',
    'machine_group.catalog' => 'catalog',
    'machine_group.architecture' => 'arch',
    'machine_group.core_count' => 'core_count',
    'machine_group.cpu_count' => 'cpu_count',
    'machine_group.description' => 'description',
    'machine_group.high_speed_interface_type' => 'high_speed_interface_type',

# Machine:
    'machine.id' => 'id',
    'machine.name' => 'name',
    'machine.cpu_load_avg' => 'cpu_load_avg',
    'machine.fail_count' => 'fail_count',
    'machine.hostname' => 'hostname',
    'machine.hypervisor' => 'hypervisor',
    'machine.hypv_flag' => 'hypv_flag',
    'machine.interfaces' => 'interfaces',
    'machine.ip' => 'ip',
    'machine.machine_group' => 'machine_group',
    'machine.physicial_name' => 'physicial_name',
    'machine.power_state' => 'power_state',
    'machine.provision_mode' => 'provision_mode',
	'machine.status' => 'status',
    'machine.swap_space_used' => 'swap_space_used',
    'machine.location' => 'location',
    'machine.virtual_memory_used' => 'virtual_memory_used',
	'machine.arch' => 'arch',
	'machine.type' => 'type',
	'machine.default_os' => 'default_os',
	'machine.current_os' => 'current_os',

# EnergyPolicy:
    'energy_scheme.id' => 'id',
    'energy_policy.name' => 'name',
    'energy_policy.mach_status' => 'mach_status',
    'energy_policy.prov_mode' => 'prov_mode',
    'energy_policy.mach_group' => 'mach_group',
    'energy_policy.power_status' => 'power_status',
    'energy_policy.energy_scheme' => 'energy_scheme',
    'energy_policy.flag' => 'flag',
    'energy_policy.' => 'description',

# EnergyScheme:
    'energy_scheme.id' => 'id',
    'energy_scheme.name' => 'name',
    'energy_scheme.owner' => 'owner',
    'energy_scheme.cat' => 'cat',
    'energy_scheme.cpu_freq_fixed' => 'cpu_freq_fixed',
    'energy_scheme.cpu_freq_min' => 'cpu_freq_min',
    'energy_scheme.cpu_freq_max' => 'cpu_freq_max',
    'energy_scheme.threshold_up' => 'threshold_up',
    'energy_scheme.threshold_down' => 'threshold_down',
    'energy_scheme.sampling_rate' => 'sampling_rate',
    'energy_scheme.freq_step' => 'freq_step',
    'energy_scheme.description' => 'description',

# EnergyUsage:
    'energy_usage.id' => 'id',
    'energy_usage.machine_name' => 'machine_name',
    'energy_usage.request_name' => 'request_name',
    'energy_usage.avg_power' => 'avg_power',
    'energy_usage.acc_power' => 'acc_power',
    'energy_usage.sampling_times' => 'sampling_times',
    'energy_usage.start_time' => 'start_time',
    'energy_usage.start_energy' => 'start_energy',
    'energy_usage.pre_energy' => 'pre_energy',
    'energy_usage.cur_energy' => 'cur_energy',
    'energy_usage.update_interval' => 'update_interval',

# EschemeMachine:
    'escheme_machine.id' => 'id',
    'escheme_machine.machine_name' => 'machine_name',
    'escheme_machine.is_machine_group' => 'is_machine_group',
    'escheme_machine.energy_scheme' => 'energy_scheme',

# ProvisionMachineHistory:
    'provision_machine_history.id' => 'id',
    'provision_machine_history.machine_name' => 'machine_name',
    'provision_machine_history.request_name' => 'request_name',
    'provision_machine_history.arch' => 'arch',
    'provision_machine_history.cat' => 'cat',
    'provision_machine_history.os' => 'os',
    'provision_machine_history.ip' => 'ip',
    'provision_machine_history.prov_start_time' => 'prov_start_time',
    'provision_machine_history.prov_stop_time' => 'prov_stop_time',
    'provision_machine_history.energy' => 'energy',
	'provision_machine_history.vlan_id' => 'vlan_id',
	'provision_machine_history.vlan_flag' => 'vlan_flag',

# switch releated table
	'switchconfig.id' => 'id',
	'switchconfig.switch_name' => 'switch_name', 
	'switchconfig.port_number' => 'port_number', 
	'switchconfig.ipaddress' => 'ipaddress',  
	'switchconfig.description' => 'description',
	'switchportconfig.id' => 'id',
	'switchportconfig.node_name' => 'node_name',
	'switchportconfig.switch_name' => 'switch_name', 
	'switchportconfig.bandwidth' => 'bandwidth', 
	'switchportconfig.port_id' => 'port_id',  
	'switchportconfig.vlan_id' => 'vlan_id',  
	'switchportconfig.description' => 'description',

# pkglist
	'pkglist.id' => 'id',
	'pkglist.name' => 'name',
	'pkglist.path_orig' => 'path_orig',
	'pkglist.status' => 'status',
	'pkglist.creator' => 'creator',
	'pkglist.description' => 'description',
	
# script
	'script.id' => 'id',
	'script.name' => 'name',
	'script.flag' => 'flag',
	'script.status' => 'status',
	'script.description' => 'description',

# infrastr_script
	'infrastr_script.id' => 'id',
	'infrastr_script.name' => 'name',
	'infrastr_script.type' => 'type',
	'infrastr_script.pre_scripts' => 'pre_scripts',
	'infrastr_script.post_scripts' => 'post_scripts',
	
#request_machine
	'request_machine.hypervisor' => 'hypervisor',
	'request_machine.status' => 'status',
	'request_machine.inuse_time' => 'inuse_time',
);

my %usage = (
    "hcmkreq" => 
"Usage:
	{ [-h | --help]
	{ {--machtype=<machine_type> --machnum=<machine_num>} | --machlist=<machine1[,machine2[,machine3[,..]]]>}
	--startdate=<\"mm-dd-yyyy hh:mm:ss\"> --enddate=<\"mm-dd-yyyy hh:mm:ss\"> | --duration=<duration>
	--osimage=<os_image> [--users=<user1[,user2[,user3[,..]]]>] [--groups=<group1[,group2[,group3[,..]]]>]
	[--cluster=<yes|no>] }",

    "hcchreq" => 
"Usage:
	{ [-h | --help] --reqid=<request_id> 
	[--addmach=<machine1[,machine2[,machine3[,..]]]> | --rmmach=<machine1[,machine2[,machine3[,..]]]> |
	--machlist=<machine1[,machine2[,machine3[,..]]]> | --addmachnum=<Requested new machine number>]
	[--enddate=<\"mm-dd-yyyy hh:mm:ss\"> | --duration=<duration>]
	[--startdate=<\"mm-dd-yyyy hh:mm:ss\">]
	[ [--adduser=<user1[,user2[,user3[,..]]]>] [--rmuser=<user1[,user2[,user3[,..]]]>] | --users=<user1[,user2[,user3[,..]]]> ]
	[ [--addgroup=<group1[,group2[,group3[,..]]]>] [--rmgroup=<group1[,group2[,group3[,..]]]>] | --groups=<group1[,group2[,group3[,..]]]> ] }",

    "hclsreq" =>
"Usage:
	{ [-h | --help] { --all } | --reqid=<request_id> | --userid=<user_name> [--status=<status1[,status2[,...]]>] | --status=<status1[,status2[,...]]> }",

    "hcrmreq" =>
"Usage:
	{ [-h | --help] --reqid=<request_id> }",

    "hcmkhypv" =>
"Usage:
	{ [-h | --help] | --hypv=<hypervisor_list> [--os=<base_operating_system>] }",

    "hclshypv" =>
"Usage:
	{ [-h | --help] | --all | --hypv=<hypervision_list> }",

    "hcmkvm" =>
"Usage:
	{ [-h | --help] | --vmtype=<IBM_KVM> --vmcount=<vm_count>
	--startdate=<\"mm-dd-yyyy hh:mm:ss\">  
	--enddate=<\"mm-dd-yyyy hh:mm:ss\"> | --duration=<duration>
	--osimage=<os_image> [--cpus=<cpu_count>] [--memory=<memory_count>]
	[--disk=<disk_quota>] [--users=<user1[,user2[,user3[,..]]]>] [--groups=<group1[,group2[,group3[,..]]]>]
	[--cluster=<yes|no>] }",

    "hcchvm" =>
"Usage:
	{ [-h | --help] |  --reqid=<request_id>  [--vmcount=<vm_count>]
	[--cpus=<cpu_num>] [--memory=<memory_size>] [--disk=<disk_size>]
	[--enddate=<\"mm-dd-yyyy hh:mm:ss\">  |  --duration=<duration>]
	[--startdate=<\"mm-dd-yyyy hh:mm:ss\">]
	[ [--adduser=<user1[,user2[,user3[,..]]]>] [--rmuser=<user1[,user2[,user3[,..]]]>] | --users=<user1[,user2[,user3[,..]]]>]
	[ [--addgroup=<group1[,group2[,group3[,..]]]>] [--rmgroup=<group1[,group2[,group3[,..]]]>] | --groups=<group1[,group2[,group3[,..]]]>] }",

    "hclsvm" =>
"Usage:
	{ [-h | --help] | --all | --reqid=<request_id> | --vnode=<KVM machine name> }",

    "hcrmvm" =>
"Usage:
	{ [-h | --help] | --reqid=<request_id> }",

    "hcupenergy" =>
"Usage:
	{ [-h | --help] | --reqid=<request_id> [--duration=<duration>] [--interval=<interval>] }",

    "hclsenergy" =>
"Usage:
	{ [-h | --help] | --reqid=<request_id> [--machname=<machine_name>] }",

    "hcmkescheme" =>
"Usage:
	{ [-h | --help] | --owner=<scheme_owner> --name=<sheme_name> [--desc=<description>]
	{[--type=0] --cpu-freq-fixed=<cpu_freq>} |
	{ --type=1 --cpu-freq-min=<cpu_freq> --cpu-freq-max=<cpur_freq> [--threshold-up=<threshold>] [--threshold-down=<threshold>]
	[--sampling-rate=<rate>] [--freq-step=<freq_step>]} }",

    "hcchescheme" =>
"Usage:
	{ [-h | --help] | --owner=<scheme_owner> --name=<scheme_name> [--type=<type>]
	[--cpu-freq-fixed=<cpu_freq>] [--cpu-freq-min=<cpu_freq>] [--cpu-freq-max=<cpur_freq>] [--threshold-up=<threshold>] [--threshold-down=<threshold>]
	[--sampling-rate=<rate>] [--freq-step=<freq_step>] [--desc=<remark>] [--force] }",

    "hclsescheme" =>
"Usage:
	{ [-h | --help] | [--owner=<scheme_owner> [--name=<scheme_name>]] }",

    "hcrmescheme" =>
"Usage:
	{ [-h | --help] | --owner=<scheme_owner> [--name=<scheme_name>] }",

    "hcapplyescheme" =>
"Usage:
	{ [-h | --help] | --owner=<scheme_owner> --name=<scheme_name> --machlist=<machine1[,machine2[,machine3[,..]]]> }",

    "hcreleaseescheme" =>
"Usage:
	{ [-h | --help] | --owner=<scheme_owner> --name=<scheme_name> --machlist=<machines> }",

    "hclsavailcpufreq" =>
"Usage:
	{ [-h | --help] }",

    "hcmkepolicy" =>
"Usage:
	{ [-h | --help] [--name=<policy_name>] --machstatus=<machine_status> [--provmode=<prov_mode>]
	[--machgroup=<machine_group>] --energyscheme=<energy_scheme> [--desc=<description>] }",

    "hcchepolicy" =>
"Usage:
	{ [-h | --help] --id=<energy_policy_id> [--name=< policy_name>] [--machstatus=<machine_status> ]
	[--provmode=<prov_mode>] [--machgroup=<mach_group>] [--energyscheme=<energy_scheme>] [--desc=<description>] }",

    "hclsepolicy" =>
"Usage:
	{ [-h | --help] [--id=<energy_policy_id>] | [--name=<energy_policy_name>] }",

    "hcrmepolicy" =>
"Usage:
	{ [-h | --help] --id=<energy_policy_id> }",

    "hcmkimage" =>
"Usage:
	{ [-h | --help] | --profile=<profile_name> --os=<base_operating_system> --arch=<architecture> --desc=<description>
	[--pkglist=<kernel_pkglist_file>] [--pkglocation=<pkg_directory>] [--scheduler=<loadl|lsf|pbs|sge|others>] [--parallelenv=<pe|openmpi|others>]
	[--interconnect=<eth|ib|others>] [--sharedfs=<nfs|gpfs|others>] [--prescripts=<script_names>] [--postscripts=<script_names>] [--installtodisk=<yes|no>] }",

    "hcchimage" =>
"Usage:
	{ [-h | --help] | --label=<image_name> [--grouplist=<grp1,grp2,...>] [--approve=yes|no] }",

    "hclsimage" =>
"Usage:
	{ [-h | --help] | [--name=<image_name>] [--profile=<profile_name>] [--os=<base_operating_system>] [--arch=<architecture>] [--group=<user_group_id>]
	[--status=<INIT|AVAILABLE|SETUP>] [--scheduler=<loadl|lsf|pbs|sge|others>] [--parallelenv=<pe|openmpi|others>] [--interconnect=<eth|ib|others>]
	[--sharedfs=<nfs|gpfs|others>] }",

    "hcrmimage" =>
"Usage:
	{ -h | --help | --profile=<profile_name> --os=<base_operating_system> --arch=<architecture> }",

    "hceximage" =>
"Usage:
	{ [-h | --help] | --image=<image_name> --action=<extract | pack> }",

    "hccapimage" =>
"Usage:
	{ [-h | --help] | --mach=<mach_name>  --profile=<profile_name> --os=<base_operating_system> --arch=<architecture> 
	--desc=<description> [--pkglist=<kernel_pkglist_file>] [--scheduler=<loadl|lsf|pbs|sge|others>] [--parallelenv=<pe|openmpi|others>]
	[--interconnect=<eth|ib|others>] [--sharedfs=<nfs|gpfs|others>] [--postscripts=<script_names>] }",

	"hcmkpkglist" =>
"Usage:
	{ [-h | --help] | --path=<pkglist_full_path> --desc=<description> [--newname=<pkglist_file_name>] }",
	"hcchpkglist" =>
"Usage:
	{ [-h | --help] | --name=<pkglist_file_name> [--status=<INIT|APPROVED>] [--desc=<description>] }",
	"hclspkglist" =>
"Usage:
	{ [-h | --help] | [--creator=<user_name>] [--status=<INIT|APPROVED|REMOVED>] [--name=<pkglist_file_name>] }",
	"hcrmpkglist" =>
"Usage:
	{ [-h | --help] | {[--creator=<user_name>] [--status=<INIT|APPROVED>] [--name=<pkglist_file_name>]} | [--all] }",
	"hcmkscript" =>
"Usage:
	{ [-h | --help] | --path=<script_full_path> --flag=<PRE|POST> --desc=<description> [--newname=<new_script_name>] }",
	"hcchscript" =>
"Usage:
	{ [-h | --help] | --name=<script_file_name> [--status=<INIT|APPROVED>] [--desc=<description>] }",
	"hclsscript" =>
"Usage:
	{ [-h | --help] | [--flag=<PRE|POST>] [--status=<INIT|APPROVED|REMOVED>] [--name=<script_file_name>] }",
	"hcrmscript" =>
"Usage:
	{ [-h | --help] | {[--flag=<PRE|POST>] [--status=<INIT|APPROVED>] [--name=<script_file_name>]} | [--all] }",

    "hclsuser" =>
"Usage:
	{ [-h | --help] | --all | --userid=<user_id> }",

    "hcmkuser" =>
"Usage:
	{ [-h | --help] | --userid=<user_id> --password=<password> [--firstname=<first_name>] [--lastname=<last_name>]
	[--email=<email>] [--phone=<phone>] [--department=<department>] [--organization=<organization>] [--balance=<balance>]
	[--shell=<bash|csh|dash|ksh|sh|tcsh>] [--comments=<comments>] }",

    "hcchuser" =>
"Usage:
	{ [-h | --help] | --userid=<user_id> [--oldpassword=<old_password> --newpassword=<new_password>]
	[--firstname=<first_name>] [--lastname=<last_name>] [--email=<email>] [--phone=<phone>]
	[--department=<department>] [--organization=<organization>] [--recharge=<recharge>]
	[--shell=<bash|csh|dash|ksh|sh|tcsh>] [--comments=<comments>] }",

    "hcrmuser" =>
"Usage:
	{ [-h | --help] | --userid=<userid> }",

    "hcresetuser" =>
"Usage:
	{ [-h | --help] | --userid=<userid> --rstpwd=<password> }",

    "hcmkgroup" =>
"Usage:
	{ [-h | --help] | --groupid=<group_id> --ownerid=<ownwer_id> --userlist=<user1[,user2[,user3[,..]]]> [--comments=<comments>] }",

    "hcchgroup" =>
"Usage:
	{ [-h | --help] | --groupid=<group_id> --userlist=<user1[,user2[,user3[,..]]]> [--comments=<comments>] }",

    "hclsgroup" =>
"Usage:
	{ [-h | --help] | --all | --groupid=<group_id> }",

    "hcrmgroup" =>
"Usage:
	{ [-h | --help] | --groupid=<group_id> }",

    "hcsubmitjob" =>
"Usage:
	{ [-h | --help] | [--reqid=<request_id>] --jcf=<jcf_path> }",

    "hclsjob" =>
"Usage:
	{ [-h | --help] | --all | --reqid=<request_id> | --jobid=<job_id> }",

    "hcaddrole" =>
"Usage:
	{ [-h | --help] | --role=<role_name> --comments=<comments>] }",

    "hclsrole" =>
"Usage:
	{ [-h | --help] | --all | --role=<role_name> }",

    "hcrmrole" =>
"Usage:
	{ [-h | --help] | --role=<role_name> }",

    "hcassignrole" =>
"Usage:
	{ [-h | --help] | --role=<role_name> --user=<user_name> }",

    "hcaddcapability" =>
"Usage:
	{ [-h | --help] | --role=<role_name> --capability=<capability_name> }",

    "hclscapability" =>
"Usage:
	{ [-h | --help] | --all }",

    "hcrmcapability" =>
"Usage:
	{ [-h | --help] | --role=<role_name> --capability=<cap_name> }",

    "hclsstatus" =>
"Usage:
	{ [-h | --help] | --all | --machgroup=<machine_group> | --machname=<machine_name> }",

    "hclsevent" =>
"Usage:
	{ [-h | --help] [--start=<\"mm/dd/yyyy [hh:mm:[ss]]\">] [--end=<\"mm/dd/yyyy [hh:mm:[ss]]\">] [--object=<object_id>] [--level=<log_level>] }",

    "hcversion" =>
"Usage:
	[-h | --help]",
    "hcmkinfrastructure" =>
"Usage:
	{ [-h | --help] | --name=<infra_name> --type=<infra_type> [--prescripts=<pre_scripts_name>] [--postscripts=<post_scripts_name>] }",
    "hclsinfrastructure" =>
"Usage:
	{ [-h | --help] | [--name=<infra_name>] [--type=<infra_type>] }",
    "hcchinfrastructure" =>
"Usage:
	{ [-h | --help] | [--name=<infralist_name>] [--type=<infra_type>] [--prescripts=<pre_scripts_name>] [--postscripts=<post_scripts_name>] }",
    "hcrminfrastructure" =>
"Usage:
	{ [-h | --help] | [--name=<infralist_name>] [--type=<infra_type>] | [--all] }",
	"hcchnode2switch" =>
"Usage:
	{ [-h|--help] | --nodeName=<node name> [--switchName=<switch name> --port=<switch port id> --bandwidth=<Ethernet  bandwidth> --comments=<comments>] }",
	"hcchswitch" =>
"Usage:
	{ [-h|--help] | --switchName=<switch name> [--ipAddress=<ip address> --switchLongName=<switch.cluster.com name> --snmpVersion=<1|2|3> --userName=<snmp login user> --password=<password> --authMethod=<SHA> --linkPorts=<port_number:switch> --switchPortNum=<switch port number> --comments=<comments>]}",
	"hclsswitch" =>
"Usage:
	{ [-h|--help] | --switchname=<switch name> }",
	"hclsswitchport"=>
"Usage:
	{ [-h|--help] | --nodename=<node name>}",
	"hcmknode2switch"=>
"Usage:
	{ [-h|--help] | --nodeName=<node name> --switchName=<switch name> --port=<switch port id> [--bandwidth=<Ethernet  bandwidth> --comments=<comments>] }",
	"hcmkswitch"=>
"Usage:
	{ [-h|--help] | --switchName=<switch name> --ipAddress=<ip address> [--switchLongName=<switch.cluster.com name> --snmpVersion=<1|2|3> --userName=<snmp login user> --password=<password> --authMethod=<SHA> --linkPorts=<port_number:switch> --switchPortNum=<switch port number> --comments=<comments>]}",
	"hcrmnode2switch"=>
"Usage:
	{ [-h|--help] | --nodeName=<node name> }",
	"hcrmswitch"=>
"Usage:
	{ [-h|--help] | --switchName=<switch name> }",
);

#--------------------------------------------------------------------------------
=head3   getUsage
      It returns the usage string for the given command.
    Arguments:
        command
    Returns:
        the usage string for the command.
=cut
#-------------------------------------------------------------------------------
sub getUsage {
   my ($class, $command)=@_;
   if (exists($usage{$command})) { return $usage{$command};}  
   else { return "Usage for command $command cannot be found\n"; }
}


#--------------------------------------------------------------------------------
=head3   findName
      Internal function.
      get the converted key name
    Arguments:
        key name
    Returns:
        the converted key name
=cut
#-------------------------------------------------------------------------------
sub _findName {
    my ($keyName, $namespace) = @_;
    return $CmdUtils::NameConvert{$keyName};
}

#--------------------------------------------------------------------------------
=head3   convertHashObj
      Internal function.
      Iterate all keys in the hash object, and convert each key names;
    Arguments:
        Hash handle
    Returns:
        none
=cut
#-------------------------------------------------------------------------------
sub _convertHashObj {
    my ($hash) = @_;
    my $i;
    my %newNames = ();
    foreach $i (keys(%$hash)) {
        my $newName = _findName($i);
        if (defined($newName)) {
            $newNames{$i} = $newName if ($newName ne $i);
        }

        _convertObject($hash->{$i});
    }
    foreach $i (keys(%newNames)) {
        $hash->{$newNames{$i}} = $hash->{$i};
        delete $hash->{$i};
    }
}

#--------------------------------------------------------------------------------
=head3   convertArrayObj
      Internal function.
      Iterate all elements in the hash object, and convert each key names;
    Arguments:
        Array handle
    Returns:
        none
=cut
#-------------------------------------------------------------------------------
sub _convertArrayObj {
    my ($array) = @_;
    foreach my $i (@$array) {
        _convertObject($i);
    }
}

#--------------------------------------------------------------------------------
=head3   convertObject
      Internal function.
      Visit the hash/array object, and convert each key names;
    Arguments:
        object handle
    Returns:
        none
=cut
#-------------------------------------------------------------------------------
sub _convertObject {
    my ($hdl) = @_;
    if (ref($hdl) eq 'HASH') {
        _convertHashObj($hdl);
    } elsif (ref($hdl) eq 'ARRAY') {
        _convertArrayObj($hdl);
    }
}

#--------------------------------------------------------------------------------
=head3   convertNames
      Internal function.
      Visit the resulte object, and convert each key names;
    Arguments:
        result handle
    Returns:
        none
=cut
#-------------------------------------------------------------------------------
sub _convertNames {
    my ($hdl) = @_;
    _convertObject($hdl);
}

#--------------------------------------------------------------------------------
=head3   printXML
      It display the res model in XML.
    Arguments:
        res model hash ref, root name 
    Returns:
        the res model in XML.
=cut
#-------------------------------------------------------------------------------
sub printXML {
   my ($class, $resmodel, $rtname)=@_;

   # convert key names;
   _convertNames($resmodel);

   # create a res model object
   my $xml = new XML::Simple (NoAttr=>1, RootName=>$rtname);

   # convert Perl hash ref into XML document
   my $res = $xml->XMLout($resmodel);

   # print XML res model 
   print $res;

}

sub printSuc {
   my ($class, $result, $status)=@_;
  
   my $res;
   my $ops;
   my @atts = ();

   $res->{'status'} = $status;
   foreach my $opt (@$result) {
       my ($key, $value) =  each(%$opt);

       my $att;
       $att->{'name'} = $key;
       $att->{'value'} = $value;
       push(@atts, $att);
   }

   $ops = \@atts;
   $res->{'options'} = {'option' => $ops};

   CmdUtils->printXML($res, "result"); 
}

sub printErr {
	my ($class, $result)=@_;

	my $att;
	my @atts = ();

	foreach my $err (@$result) {
		my $att;
		$att->{'status'} = "failure";
		$att->{'reasonMessage'} = $err->{'errmsg'};
		$att->{'reasonCode'} = $err->{'errcode'};
		push(@atts, $att);
	}

	my $ops = \@atts;
	my $res->{'result'} = $ops;
	CmdUtils->printXML($res, "results");
}

sub error {
   my ($class, $errmsg, $errcode)=@_;

   my $err;
   my @errs = ();
   $err->{'errmsg'} = $errmsg;
   $err->{'errcode'} = $errcode; # need to define
   push(@errs, $err);

   CmdUtils->printErr(\@errs);
   Logger->get()->warning('CmdUtils', $errcode, $errmsg);
   exit(1);
}

#--------------------------------------------------------------------------------
=head3   convertDate
      It convert the date.
    Arguments:
        the date
    Returns:
        the seconds.
=cut
#-------------------------------------------------------------------------------
sub convertDate {
   my ($class, $date, $flag)=@_;

   my ($timelc,$sec,$min,$hour,$mday,$mon,$year);
   my @time = split(/-|:|\s|\//,$date);

   foreach(@time){
      s/^0//;
      $timelc .= $_.",";
   }

   $timelc =~ s/,$//;

   ($mon, $mday, $year, $hour, $min, $sec) = split /,/,$timelc;
   
   if ($flag) { return( [$mon, $mday, $year, $hour, $min, $sec] ); }

   $year -= 1900;
   $mon -= 1;

   $timelc = timelocal($sec,$min,$hour,$mday,$mon,$year);

   return($timelc);
}
#-------------------------------------------------------------------------------
# JO: 1
# PO: 2
# J+P: 3
# failure: 0
sub convertProvMode {
    my ($class, $strPM) = @_;

    if ($strPM eq "JO") {
        return 1;
    }
    if ($strPM eq "PO") {
        return 2;
    }
    if ($strPM eq "J+P") {
        return 3;
    }

    return 0;
}

sub getVersion {
    # The correct version and build date will be replaced during pkg.
    # The version string will look like:
    # Version 1.0.0.0 (s18, built Mon Jul  4 03:09:43 EDT 2011)

    my $version = 'Version ' . '1.0.0.2' . ' (rbeis002a, built Wed Feb  8 11:55:44 EST 2012)';
    return $version;
}

sub runxcatcmd {
	my ($class, $cmd, $admin, $retry) = @_;

	# if the command is not used the full path name
	if ($cmd !~ /^\//) {
		my ($base, @unused) = split(' ', $cmd);

		if (-f "/opt/xcat/bin/$base") {
			$cmd = "/opt/xcat/bin/" . $cmd;
		}
		elsif (-f "/opt/xcat/sbin/$base") {
			$cmd = "/opt/xcat/sbin/" . $cmd;
		}
	}

	$retry = defined($retry) ? $retry : 1;
	my $rv = 0;
	my @out;

	do {
		($rv, @out) = CmdUtils->runcmd($cmd, $admin);
	} while(($rv != 0) && (--$retry));

	return ($rv, @out);
}

sub runloadlcmd {
	my ($class, $cmd, $admin) = @_;

	# if the command is not used the full path name
	if ($cmd !~ /^\//) {
		$cmd = "/usr/bin/" . $cmd;
	}

	return CmdUtils->runcmd($cmd, $admin);
}

sub runpecmd {
	my ($class, $cmd, $admin) = @_;

	# if the command is not used the full path name
	if ($cmd !~ /^\//) {
		$cmd = "/usr/bin/" . $cmd;
	}

	return CmdUtils->runcmd($cmd, $admin);
}

sub runcmd {
	my ($class, $cmd, $admin) = @_;
	my @out;

	my @cmdline=split(" ", $cmd);
	if ($admin) {
		$cmd = "/opt/ibm/hpccloud/cloudbin/utils/thcsu -u $admin '$cmd'";
	}

	use File::Basename;

	if (!is_hccmd(basename($0))) {
		print("$cmd\n");
	}
	my $lockflag = is_lockcmd(basename($cmdline[0]));
	
	if($lockflag){
		$lockflag = CmdUtils->takelock(basename($cmdline[0]), -1);
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
		$year += 1900;
		$mon += 1;
		my $timestr = sprintf("%02d.%02d.%4d_%02d:%02d:%02d", $mon, $mday, $year, $hour, $min, $sec);
		print "get lock[$lockflag] for $cmdline[0] at $timestr\n";
	}

	eval {
		@out = `$cmd 2>&1`;
	};
	if($lockflag){
		CmdUtils->givelock($lockflag);
	}
	if($@) {
		Logger->get()->error("CmdUtils", "[$cmd] error", "@out");
		return(-1, @out);
	}

	my $rv = $? >> 8;
	if ($rv) {
		Logger->get()->error("CmdUtils", "[$cmd] failed", "@out");
	}
	return($rv, @out);
}

sub is_lockcmd {
	my ($base) = @_;
	my %lockcmd = (
		'mkvlan' => 1,
	);

	return $lockcmd{$base};
}

sub is_hccmd {
	my ($base) = @_;

	my %hccmd = (
		# prov
		'hcmkreq' => 1,
		'hcmkhypv' => 1,
		'hcmkvm' => 1,
		'hcchreq' => 1,
		'hcchvm' => 1,
		'hclsevent' => 1,
		'hclshypv' => 1,
		'hclsjob' => 1,
		'hclsreq' => 1,
		'hclsstatus' => 1,
		'hclsvm' => 1,
		'hcrmreq' => 1,
		'hcsubmitjob' => 1,

		# energy
		'hclsavailcpufreq' => 1,
		'hcmkescheme' => 1,
		'hclsescheme' => 1,
		'hcchescheme' => 1,
		'hcrmescheme' => 1,
		'hcapplyescheme' => 1,
		'hcreleaseescheme' => 1,
		'hcmkepolicy' => 1,
		'hclsepolicy' => 1,
		'hcchepolicy' => 1,
		'hcrmepolicy' => 1,
		'hclsenergy' => 1,
		'hcupenergy' => 1,

		# image
		'hcmkimage' => 1,
		'hclsimage' => 1,
		'hcchimage' => 1,
		'hcrmimage' => 1,
		'hccapimage' => 1,
		'hceximage' => 1,
		'hcmkpkglist' => 1,
		'hclspkglist' => 1,
		'hcchpkglist' => 1,
		'hcrmpkglist' => 1,
		'hcmkscript' => 1,
		'hclsscript' => 1,
		'hcchscript' => 1,
		'hcrmscript' => 1,
		'hcmkinfrastructure' => 1,
		'hclsinfrastructure' => 1,
		'hcchinfrastructure' => 1,
		'hcrminfrastructure' => 1,

		# sysconfig
		'hcmkswitch' => 1,
		'hclsswitch' => 1,
		'hcchswitch' => 1,
		'hcrmswitch' => 1,
		'hcmknode2switch' => 1,
		'hclsswitchport' => 1,
		'hcchnode2switch' => 1,
		'hcrmnode2switch' => 1,

		# usermgt
		'hcaddrole' => 1,
		'hclsrole' => 1,
		'hcrmrole' => 1,
		'hcassignrole' => 1,
		'hcmkgroup' => 1,
		'hclsgroup' => 1,
		'hcchgroup' => 1,
		'hcrmgroup' => 1,
		'hcmkuser' => 1,
		'hclsuser' => 1,
		'hcchuser' => 1,
		'hcrmuser' => 1,
		'hcaddcapability' => 1,
		'hclscapability' => 1,
		'hcrmcapability' => 1,

		'dumpmach' => 1,
		'addmach2cloud' => 1,
		'addvm2cloud' => 1,
		'portal.cgi' => 1,
		'GetInfoXML.cgi' => 1,
	);

	return $hccmd{$base};
}

sub print_val {
	my ($class, $ref) = @_;

	if (!defined($ref)) {
		return ;
	}

	my $ref_type = ref($ref);

	if ($ref_type eq 'SCALAR') {
		print("$$ref\n");
	}
	elsif ($ref_type eq 'ARRAY') {
		print_array($ref);
	}
	elsif ($ref_type eq 'HASH') {
		print_hash($ref);
	}
	else {
		print("$ref\n");
	}

	return ;
}

sub print_array {
	my ($array_ref) = @_;

	foreach my $ref (@$array_ref) {
		CmdUtils->print_val($ref);
	}
}

sub print_hash {
	my ($hash_ref) = @_;

	while(my ($key, $val) =  each(%$hash_ref)) {
		CmdUtils->print_val($key);
		print("=>\n");
		CmdUtils->print_val($val);
	}
}

sub takelock{
	my ($class, $lock_name, $timer) = @_;
	my $lock_file = "/tmp/.lock_$lock_name";
	`mkdir -p /tmp/`;
	my $LOCK;
	my $flag = 1;
	my $count = 0;
	if(!defined($timer)){
		$timer = 0;
	}
	if(!defined($lock_name)){
		return undef;
	}
	do{
		my $getlockflag=0;
		if(open($LOCK, ">$lock_file")){
    		if(flock($LOCK, LOCK_EX)){
				`chmod 777 $lock_file`;
				print $LOCK $$;
				$getlockflag=1;
			}else{
				close($LOCK);
			}
		}

		if($getlockflag){
			$flag = 0;
		}else{
			if($timer >= 0){
				if($count < $timer){
					print "sleep 5 seconds and retake the lock $lock_name\n";
					sleep(5);
					$count += 5;
				}else{
					$flag = 0;
					$LOCK = undef;
				}
			}else{
				sleep(5);
			}
		}
	}while($flag);
	return $LOCK;
}

sub givelock{
	my ($class, $lock_id) = @_;
	close($lock_id);
}
1;
