# sccsid = "@(#) src/hpcc/common/db/EnergyDAO.pm, hpc.cloud.energy, hpcc_rbei, rbeis002a 1.10 12/28/11 03:54:27";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2011 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package EnergyDAO;

use strict;
use warnings;
require Exporter;
require DB::DBA;
our @ISA = qw(Exporter DAO);
require DB::DAO;
use Utils::Logger;

sub new {
	my $self = DAO->new();
	bless ($self);
	return $self;
}


sub queryESchemeByMachStatAndProMod {
	my ($class, $machstatus, $provmode) = @_;
	my @result;
	my @args;


	if (!$machstatus || !$provmode) { return -1; }

	my $sql = 'select #>energy_scheme.owner<#, #>energy_scheme.name<# from #>energy_policy<# join #>energy_scheme<# on (#>energy_policy.energy_scheme<#=#>energy_scheme.name<#) where #>energy_policy.mach_status<# = ? and #>energy_policy.prov_mode<# = ?';

	push(@args,[$machstatus]);
	push(@args,[$provmode]);

	my ($rc, $param) = $class->{'dba'}->querySQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', 'NA', "Failed to query energy scheme. Reason: $param");
		return (-1, "Failed to query energy scheme. Reason: $param");
	}
	($rc, $param) = $class->{'dba'}->fetchrow_arrayref();

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', 'NA', "Failed to query energy scheme. Reason: $param");
		return (-1, "Failed to query energy scheme. Reason: $param");
	}

	$class->{'dba'}->finish();

	@result = @$param;

	if (!scalar(@$param)){
		Logger->get()->info('EnergyDAO', 'NA', "No record.");
		return (100, "No record.");
	}

	return (0, \@result);
}


sub insertEnergyScheme {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>energy_scheme.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_scheme.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>energy_scheme<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$name", "Failed to create new energy scheme. Reason: $param");
		return (-1, "Failed to create new energy scheme. Reason: $param");
	}

	return (0);
}

sub insertEnergyPolicy {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>energy_policy.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_policy.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>energy_policy<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$name", "Failed to create new energy policy. Reason: $param");
		return (-1, "Failed to create new energy policy. Reason: $param");
	}

	return (0);
}

sub queryEnergyPolicy {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;

	foreach my $key (keys %$args) {
		$index++;

		if ($index !=1) {
			$where .= ' and';
		}

		$where .= " #>energy_policy.$key<#=?";
		push(@$bind, [$args->{$key}]);
	}

	if (!scalar(%$args)) {
		$sql = 'select #>energy_policy.*<# from #>energy_policy<#';
	} else {
		$sql = 'select #>energy_policy.*<# from #>energy_policy<# where' . $where;
	}

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy policy info. Reason: $param");
		return (-1, "Failed to query energy policy info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>energy_policy.id<#', '#>energy_policy<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy policy info. Reason: $param");
		return (-1, "Failed to query energy policy info. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}
	return (0, $result);
}

sub queryEnergyScheme {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;
	my $name_specified = 0;
	my $id_specified = 0;

	foreach my $key (keys %$args) {
		$index++;

		if ($index !=1) {
			$where .= ' and';
		}

		$where .= " #>energy_scheme.$key<#=?";
		push(@$bind, [$args->{$key}]);
		if ($key eq "name") {
			$name_specified = 1;
		}
		if ($key eq "id") {
			$id_specified = 1;
		}
	}

	if (!scalar(%$args)) {
		$sql = 'select #>energy_scheme.*<# from #>energy_scheme<#';
	} else {
		if ($name_specified != 1 && $id_specified != 1) {
			my $defaultOwner = "default";
			$where .= " or #>energy_scheme.owner<#=?";
			push(@$bind, [$defaultOwner]);
		}
		$sql = 'select #>energy_scheme.*<# from #>energy_scheme<# where' . $where;
	}

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme info. Reason: $param");
		return (-1, "Failed to query energy scheme info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>energy_scheme.id<#', '#>energy_scheme<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme info. Reason: $param");
		return (-1, "Failed to query energy scheme info. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	return (0, $result);
}

sub modifyEnergyScheme {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	my $index = 0;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_scheme.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>energy_scheme.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	my $sql = sprintf('update #>energy_scheme<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to update energy scheme. Reason: $param");
		return (-1, "Failed to update energy scheme. Reason: $param");
	}

	return (0);
}

sub deleteEnergyScheme {
	my ($self, $id) = @_;
	my $sql = '';

	$sql = 'delete from #>escheme_machine<# join #>energy_scheme<# on #>escheme_machine.energy_scheme<#=#>energy_scheme.name<# where #>energy_scheme.id<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$id", "Failed to delete energy scheme machine relate to energy scheme. Reason: $param");
		return (-1, "Failed to delete energy scheme machine relate to energy scheme. Reason: $param");
	}

	$sql = 'delete from #>energy_policy<# join #>energy_scheme<# on #>energy_policy.energy_scheme<#=#>energy_scheme.name<# where #>energy_scheme.id<#=?';

	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$id", "Failed to delete energy policy relate to energy scheme. Reason: $param");
		return (-1, "Failed to delete energy policy relate to energy scheme. Reason: $param");
	}

	$sql = 'delete from #>energy_scheme<# where #>energy_scheme.id<#=?';

	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$id", "Failed to delete energy scheme. Reason: $param");
		return (-1, "Failed to delete energy scheme. Reason: $param");
	}

	return (0);
}

sub deleteEnergySchemeByName {
	my ($self, $name) = @_;
	my $sql = '';

	$sql = 'delete from #>escheme_machine<# where #>escheme_machine.energy_scheme<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$name", "Failed to delete energy scheme machine relate to energy scheme. Reason: $param");
		return (-1, "Failed to delete energy scheme machine relate to energy scheme. Reason: $param");
	}

	$sql = 'delete from #>energy_policy<# where #>energy_policy.energy_scheme<#=?';

	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$name", "Failed to delete energy policy relate to energy scheme. Reason: $param");
		return (-1, "Failed to delete energy policy relate to energy scheme. Reason: $param");
	}

	$sql = 'delete from #>energy_scheme<# where #>energy_scheme.name<#=?';

	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$name", "Failed to delete energy scheme. Reason: $param");
		return (-1, "Failed to delete energy scheme. Reason: $param");
	}

	return (0);
}

sub insertESchemeMachine {
	my ($self, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	if (!defined($attrs)) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to insert energy scheme machine. Reason: No parameter");
		return (-1, "Failed to insert energy scheme machine. Reason: No parameter");
	}

	foreach my $key (keys(%$attrs)) {
		push(@cols, "#>escheme_machine.$key<#");
		push(@vals, '?');
		push(@args, [$attrs->{$key}]);
	}

	my $sql = sprintf('insert into #>escheme_machine<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to create new energy scheme machine. Reason: $param");
		return (-1, "Failed to create new energy scheme machine. Reason: $param");
	}

	return (0);

}

sub queryESchemeByMachine {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;

	foreach my $key (keys %$args) {
		$index++;

		if ($index != 1) {
			$where .= ' and';
		}

		$where .= " #>escheme_machine.$key<#=?";
		push(@$bind, [$args->{$key}]);
	}

	$sql = 'select #>energy_scheme.*<# from #>escheme_machine<# join #>energy_scheme<# on #>energy_scheme.name<#=#>escheme_machine.energy_scheme<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme machine info. Reason: $param");
		return (-1, "Failed to query energy scheme machine info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>energy_scheme.id<#', '#>energy_scheme<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme machine info. Reason: $param");
		return (-1, "Failed to query energy scheme machine info. Reason: $param");

	}

	$self->{'dba'}->finish();

	my $result = $param;
	if (!scalar(%$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	return (0, $result);
}

sub queryEScheme {
	my ($self) = @_;
	my $sql;
	my $bind = [];

	$sql = 'select #>escheme_machine.*<# from #>escheme_machine<#';

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme machine info. Reason: $param");
		return (-1, "Failed to query energy scheme machine info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>escheme_machine.id<#', '#>escheme_machine<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy scheme machine info. Reason: $param");
		return (-1, "Failed to query energy scheme machine info. Reason: $param");
	}

	if (!scalar(%$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	return (0, $param);
}

sub queryMachinesByEScheme {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;

	foreach my $key (keys %$args) {
		$index++;

		if ($index != 1) {
			$where .= ' and';
		}

		$where .= " #>energy_scheme.$key<#=?";
		push(@$bind, [$args->{$key}]);
	}

	$sql = 'select #>escheme_machine.*<# from #>escheme_machine<# join #>energy_scheme<# on #>energy_scheme.name<#=#>escheme_machine.energy_scheme<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query machine info by energy scheme. Reason: $param");
		return (-1, "Failed to query machine info by energy scheme. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>escheme_machine.id<#','#>escheme_machine<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query machine info by energy scheme. Reason: $param");
		return (-1, "Failed to query machine info by energy scheme. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	return (0, $result);
}

sub modifyESchemeMachine {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	my $index = 0;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>escheme_machine.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>escheme_machine.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	my $sql = sprintf('update #>escheme_machine<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to update energy scheme machine. Reason: $param");
		return (-1, "Failed to update energy scheme machine. Reason: $param");
	}

	return (0);
}

sub deleteESchemeMachine {
	my ($self, $attrs) = @_;

	my $where = '';
	my @args = ();
	my $index = 0;

	if (defined($attrs)) {
		foreach my $key (keys %$attrs) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>escheme_machine.$key<#=?";
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = 'delete from #>escheme_machine<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to delete energy scheme machine. Reason: $param");
		return (-1, "Failed to delete energy scheme machine. Reason: $param");
	}

	return 0;
}

sub modifyEnergyPolicy {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	my $index = 0;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_policy.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>energy_policy.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	my $sql = sprintf('update #>energy_policy<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to update energy policy. Reason: $param");
		return (-1, "Failed to update energy policy. Reason: $param");
	}

	return (0);
}

sub deleteEnergyPolicy {
	my ($self, $query) = @_;
	my $sql = '';
	my $index = 0;
	my @args = ();
	my $where = '';

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>energy_policy.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	$sql = 'delete from #>energy_policy<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to delete energy policy. Reason: $param");
		return (-1, "Failed to delete energy policy. Reason: $param");
	}

	return (0);
}

sub insertEnergyUsage {
	my ($self, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_usage.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>energy_usage<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to create new energy usage. Reason: $param");
		return (-1, "Failed to create new energy usage. Reason: $param");
	}

	return (0);
}

sub deleteEnergyUsage {
	my ($self, $query) = @_;
	my $sql = '';
	my $index = 0;
	my @args = ();
	my $where = '';

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>energy_usage.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	$sql = 'delete from #>energy_usage<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to delete energy usage. Reason: $param");
		return (-1, "Failed to delete energy usage. Reason: $param");
	}

	return (0);
}

sub queryEnergyFromProvMachLog {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;
	foreach my $key (keys %$args) {
		$index++;
		if ($index != 1) {
			$where .= ' and';
		}

		$where .= " #>provision_machine_history.$key<#=?";
		push(@$bind, [$args->{$key}]);
	}

	$sql = 'select #>provision_machine_history.*<# from #>provision_machine_history<# where' . $where;
	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy info by provision machine log. Reason: $param");
		return (-1, "Failed to query energy info by provision machine log. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>provision_machine_history.id<#','#>provision_machine_history<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy info by provision machine log. Reason: $param");
		return (-1, "Failed to query energy info by provision machine log. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)){
		return (100, "No record.");
	}

	return (0, $result);
}

sub queryEnergyUsage {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $index = 0;

	foreach my $key (keys %$args) {
		$index++;

		if ($index !=1) {
			$where .= ' and';
		}

		$where .= " #>energy_usage.$key<#=?";
		push(@$bind, [$args->{$key}]);
	}

	$sql = 'select #>energy_usage.*<# from #>energy_usage<# where' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy usage info. Reason: $param");
		return (-1, "Failed to query energy usage info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>energy_usage.id<#', '#>energy_usage<#');

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to query energy usage info. Reason: $param");
		return (-1, "Failed to query energy usage info. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)){
		return (100, "No record.");
	}

	return (0, $result);
}

sub queryProvisionIdByName {
	my ($self, $provisionId) = @_;
	my $sql;
	my $bind = [];
	my $name;

	$sql = 'select #>thc_request.name<# from #>thc_request<# where #>thc_request.id<#=?';
	push(@$bind, [$provisionId]);

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$provisionId", "Failed to query provision name. Reason: $param");
		return (-1, "Failed to query provision name. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_arrayref();

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$provisionId", "Failed to query provision name. Reason: $param");
		return (-1, "Failed to query provision name. Reason: $param");
	}

	if(!scalar(@$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	foreach my $i (@$param) {
		$name = $i;
		last;
	}

	return (0, $name);
}

sub queryProvisionNamebyId {
	my ($self, $provisionName) = @_;
	my $sql;
	my $bind = [];
	my $id;

	$sql = 'select #>thc_request.id<# from #>thc_request<# where #>thc_request.name<#=?';
	push(@$bind, [$provisionName]);

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$provisionName", "Failed to query provision id. Reason: $param");
		return (-1, "Failed to query provision id. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_arrayref();

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$provisionName", "Failed to query provision id. Reason: $param");
		return (-1, "Failed to query provision id. Reason: $param");
	}

	if (!scalar(@$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}

	foreach my $i (@$param) {
		$id = $i;
	}

	return (0, $id);
}

sub modifyEnergyUsage {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	my $index = 0;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>energy_usage.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		foreach my $key (keys %$query) {
			$index++;

			if ($index != 1) {
				$where .= ' and';
			}

			$where .= " #>energy_usage.$key<#=?";
			push(@args, [$query->{$key}]);
		}
	}

	my $sql = sprintf('update #>energy_usage<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "NA", "Failed to update energy usage. Reason: $param");
		return (-1, "Failed to update energy usage. Reason: $param");
	}

	return (0);
}

sub querySchemeNameById {
	my ($self, $schemeId) = @_;
	my $sql;
	my $bind = [];
	my $name;

	$sql = 'select #>energy_scheme.name<# from #>energy_scheme<# where #>energy_scheme.id<#=?';
	push(@$bind, [$schemeId]);

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$schemeId", "Failed to query scheme name. Reason: $param");
		return (-1, "Failed to query scheme name. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_arrayref();

	if ($rc != 0) {
		Logger->get()->warning('EnergyDAO', "$schemeId", "Failed to query scheme name. Reason: $param");
		return (-1, "Failed to query scheme name. Reason: $param");
	}

	$self->{'dba'}->finish();
	if (!scalar(@$param)){
		Logger->get()->info('EnergyDAO', "NA", "No record.");
		return (100, "No record.");
	}
	$name = $param->[0];

	return (0, $name);
}

1;
