# sccsid = "@(#) src/hpcc/common/db/GeneralDAO.pm, hpc.cloud.sysmgt, hpcc_rbei, rbeis002a 1.5 12/6/11 05:20:51";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2011 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package GeneralDAO;

use warnings;
use strict;
use GeneralDefine;
use Utils::Logger;
require Exporter;
require DB::DBA;
require DB::DAO;
our @ISA = qw(Exporter DAO);

sub new {
    my $self = DAO->new();
    bless ($self);
    return $self;
}

=comments
	$table: qeury table Name;
	$reference: The key name for the selection result. String
	$query_keys: speicify the query content of the table. Array
	$conditions: specify the query condition. Array/"condName,condValue", "condName,condValue"/
	$condLogical: specify the logical relationship among the conditions. String
=cut
sub query {
	my ($self, $table, $reference, $query_keys, $conditions, $condLogical) = @_;
	if(!defined($table) || !defined($reference)){
		Logger->get()->warning('db', 'query', "Not speicify the queried table name or reference name");
		return ($GeneralDefine::FAILED, "Not speicify the queried table name or reference name");
    }   
	if(!defined ($condLogical)){
		$condLogical = "and";
	}
	my $queryStr="#>$table.*<#";
	if(defined($query_keys) && scalar(@$query_keys)>0){
		my @findKey = grep {$_ eq $reference;}  @$query_keys;
		if(scalar(@findKey) ne '1'){
			Logger->get()->warning('db', 'query', "reference name does not be included in $query_keys");
			return ($GeneralDefine::FAILED, "reference name does not be included in $query_keys");
       	}
        	$queryStr='';
        	foreach(@$query_keys){
        		if($queryStr eq ''){
        			$queryStr="#>$table.$_<#";
        		}else{
        			$queryStr .=",#>$table.$_<#";
        		}
        	}
    }
    my $tabStr = "#>$table<#";
    my $condStr = '';
    my $bind = [];
    my $sql = "select $queryStr from $tabStr"; 
    if(defined($conditions) && scalar($conditions)){
       	foreach my $whereKey(@$conditions){
			my($tmpKey, $tmpValue)=split(",", $whereKey);
			if(($tmpKey eq "")||($tmpValue eq "")){
				next;
			}
       		if($condStr eq ''){
       			$condStr = "#>$table.$tmpKey<#=?";
       		}else{
       			$condStr .= " $condLogical #>$table.$tmpKey<#=?";
       		}
       		push(@$bind, [$tmpValue]);
       	}
		if($condStr ne ''){
       		$sql .= " where $condStr";
		}
	}
	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query', "Failed to $sql. Reason: $param");
		return ($GeneralDefine::FAILED, "Failed to $sql. Reason: $param");
	}
	($rc, $param) = $self->{'dba'}->fetchall_hashref("#>$table.$reference<#", "#>$table<#");
	if ($rc != 0) {
		Logger->get()->warning('db', 'query', "Failed to fetchall_hashref $table.$reference: $param");
		return ($GeneralDefine::FAILED, "Failed to fetchall_hashref $table.$reference: $param");
	}
    
	my $result = $param;

	if (!scalar(%$param)) {
		Logger->get()->info('db', 'query', "No record for $table where $condStr.");
		return ($GeneralDefine::NORECORD, "No record for $sql.");
	}

	return ($GeneralDefine::SUCCESS, $result);
}
=comments
	$table: qeury table Name;
	$modifyList: speicify the modified content of the table. Hash
	$conditions: specify the query condition. Array/"condName,condValue", "condName,condValue"/
	$condLogical: specify the logical relationship among the conditions. String
=cut
sub modify {
	my ($self, $table, $modifyList, $conditions, $condLogical) = @_;
	if(!defined($table) || !defined($modifyList)
		|| !defined($conditions)||scalar($modifyList) eq "0" 
		||scalar($conditions) eq "0"){
		Logger->get()->warning('db', 'modify', "Not speicify the queried table name or modify list or conditions");
		return ($GeneralDefine::FAILED, "Not speicify the queried table name or modify list or conditions");
    }
    my @modifyKey;
    my @Value;
	if(!defined ($condLogical)){
		$condLogical = "and";
	}
    foreach my $key (keys(%$modifyList)){
		push(@modifyKey, "#>$table.$key<#=?");
		push(@Value, [$modifyList->{$key}]);
    }
    my @conditionKey;
    foreach my $key (@$conditions){
		my ($tmpKey, $tmpValue) = split(",", $key);
		if($tmpKey eq "" || $tmpValue eq ""){
			next;
		}
		push(@conditionKey, "#>$table.$tmpKey<#=?");
		push(@Value, [$tmpValue]);
    }
    my $tabStr = "#>$table<#";
	my $sql = sprintf("update $tabStr set %s where %s", join(',', @modifyKey), join(" $condLogical ", @conditionKey));
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@Value);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify', "Failed to modify $tabStr. Reason: $param");
		return ($GeneralDefine::FAILED, "Failed to $sql. Reason: $param");
	}
	return ($GeneralDefine::SUCCESS);
}
=comments
	$table: qeury table Name;
	$insertList: speicify the inserted content of the table. Hash
=cut
sub insert {
	my ($self, $table, $insertList) = @_;
	if(!defined($table) || !defined($insertList)){
		Logger->get()->warning('db', 'insert', "Not speicify the queried table name or insert list");
		return ($GeneralDefine::FAILED, "Not speicify the queried table name or insert list");
    }
	my $tabStr = "#>$table<#";
	my @insertKey;
	my @qMask;
    my @Value;
    foreach my $key (keys(%$insertList)){
		push(@insertKey, "#>$table.$key<#");
		push(@qMask, "?");
		push(@Value, [$insertList->{$key}]);
    }
	my $sql = sprintf("insert into  $tabStr (%s) values (%s)", join(',', @insertKey), join(',', @qMask));
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@Value);
	if ($rc != 0) {
		Logger->get()->warning('db', 'insert', "Failed to insert $tabStr. Reason: $param");
		return ($GeneralDefine::FAILED, "Failed to $sql. Reason: $param");
	}

	return ($GeneralDefine::SUCCESS);
}

=comments
	$table: qeury table Name;
	$conditions: specify the query condition. Array/"condName,condValue", "condName,condValue"/
	$condLogical: specify the logical relationship among the conditions. String
=cut
sub delete {
	my ($self, $table, $conditions, $condLogical) = @_;
	my @conditionKey;
    my @Value;
	if(!defined($table)){ 
		Logger->get()->warning('db', 'delete', "Not speicify the table name");
		return ($GeneralDefine::FAILED, "Not speicify the table name");
    }
	if(!defined ($condLogical)){
		$condLogical = "and";
	}
	if(defined($conditions)){
    	foreach my $key (@$conditions){
			my($tmpKey, $tmpValue)=split(",", $key);
			if($tmpKey eq "" || $tmpValue eq ""){
				next;
			}
			push(@conditionKey, "#>$table.$tmpKey<#=?");
			push(@Value, [$tmpValue]);
		}
    }
    my $tabStr = "#>$table<#";
	my $sql;
	my $rc;
	my $param;
   	if(scalar(@conditionKey)){
		$sql = sprintf("delete from  $tabStr where %s", join(" $condLogical ", @conditionKey));
		($rc, $param) = $self->{'dba'}->execSQL($sql, \@Value);
	}else{
		$sql = "delete from  $tabStr ";
		($rc, $param) = $self->{'dba'}->execSQL($sql);
	}
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete', "Failed to $sql Reason: $param");
		return ($GeneralDefine::FAILED, "Failed to $sql. Reason: $param");
	}
	
	return ($GeneralDefine::SUCCESS);
}

