# sccsid = "@(#) src/hpcc/common/db/ImageDAO.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.7 11/1/11 16:20:20";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2011 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package ImageDAO;

# --------------------------- #
#
# Image:
#    id
#    name
#    status
#    base_os
#    architecture
#    captured_time
#    captured_on
#    captured_by
#    approved_by
#    description
#    authorized-user-group-list:
#    pkglist_path
#    pkg_location
#    scheduler
#    parallel_env
#    pre_scripts
#    post_scripts
#    interconnect
#    shared_fs
#        gname1
#        gname2
#        ...
#        gnameN
#    install_to_disk
#
# --------------------------- #

use warnings;
use strict;
require Exporter;
require DB::DBA;
our @ISA = qw(Exporter DAO);

require DB::DAO;

use Utils::Logger;

sub new {
    my $self = DAO->new();
    bless ($self);
    return $self;
}

sub queryImage {
    my ($self, $args) = @_;

    my $sql;
    my $bind = [];
    my $where = '';
	my $image_where = '';

	#support more kinds of query

	my @whereKeyList= keys(%$args);

	foreach my $where_key (@whereKeyList){
		if($where eq ""){
			$where = 'where #>image.'.$where_key.'<#=?';
		}
		else{
			$where .= ' and #>image.'.$where_key.'<#=?';
		}
		push(@$bind, [$args->{$where_key}]);
	}
	if( scalar(@whereKeyList)==0){
		$sql = 'select #>image.*<# from #>image<# ';
	} else {
		$sql = 'select #>image.*<# from #>image<# ' . $where;
	}
    my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);

    if ($rc != 0) {
	    Logger->get()->warning('ImageDAO', 'query image', "Failed to query image. Reason: $param");
        return (-1, "Failed to query image. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>image.name<#', '#>image<#');
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query image', "Failed to query image info. Reason: $param");
        return (-1, "Failed to query image info. Reason: $param");
    }

    my $result = $param;

    if (!scalar(%$param)) {
	    Logger->get()->warning('ImageDAO', 'query image', "No record.");
    	return (100, "No record.");
    }
    # get user group info;

    $bind = [];
	foreach my $image_name (keys(%$param)) {
		if($image_where eq ""){
			$image_where = 'where #>image_user_group.image_name<#=?';
		}
		else{
			$image_where .= ' or #>image_user_group.image_name<#=?';
		}
		push(@$bind, [$image_name]);
	}


    $sql = 'select #>image_user_group.id<#,#>image_user_group.image_name<#,#>image_user_group.user_group_name<# from #>image_user_group<# ' . $image_where;
    ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query image', "Failed to query image info. Reason: $param");
        return (-1, "Failed to query image info. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>image_user_group.id<#', '#>image_user_group<#');
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query image', "Failed to query image info. Reason: $param");
        return (-1, "Failed to query image info. Reason: $param");
    }

    my $ugroup = $param;
    foreach my $uguid (keys(%$ugroup)) {
        my $iname = $ugroup->{$uguid}->{'image_user_group.image_name'};
        my $gname = $ugroup->{$uguid}->{'image_user_group.user_group_name'};
        my $hdl;
        if (defined($result->{$iname}->{'authroized-user-group-list'})) {
            $hdl = $result->{$iname}->{'authroized-user-group-list'}->{'user_group'};
        } else {
            $hdl = [];
            $result->{$iname}->{'authroized-user-group-list'} = {'user_group' => $hdl};
        }
        push(@$hdl, {'user_group.name' => $gname});
    }
    
    return (0, $result);
}

sub queryAllImage {
    my ($self) = @_;

    my ($rc, $param);
    my $st;
    my $result;

    # get all user info;

    my $sql = 'select #>image.*<# from #>image<#';

    ($rc, $param) = $self->{'dba'}->querySQL($sql);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query all image', "Failed to query all image info. Reason: $param");
        return (-1, "Failed to query all image info. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>image.name<#', '#>image<#');
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query all image', "Failed to query all user info. Reason: $param");
        return (-1, "Failed to query all user info. Reason: $param");
    }
    
    $result = $param;

    if (!scalar(%$param)) {
    	Logger->get()->warning('ImageDAO', 'query all image', "No record.");
    	return (100, "No record.");
    }

    $sql = 'select #>image_user_group.id<#,#>image_user_group.image_name<#,#>image_user_group.user_group_name<# from #>image_user_group<#';

    ($rc, $param) = $self->{'dba'}->querySQL($sql);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query all image', "Failed to query image authorized user group info. Reason: $param");
        return (-1, "Failed to query image authorized user group info. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>image_user_group.id<#', '#>image_user_group<#');
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'query all image', "Failed to query image authorized user group info. Reason: $param");
        return (-1, "Failed to query image authorized user group info. Reason: $param");
    }

    my $ugroup = $param;
    foreach my $uguid (keys(%$ugroup)) {
        my $iname = $ugroup->{$uguid}->{'image_user_group.image_name'};
        my $gname = $ugroup->{$uguid}->{'image_user_group.user_group_name'};
        my $hdl;
        if (defined($result->{$iname}->{'authroized-user-group-list'})) {
            $hdl = $result->{$iname}->{'authroized-user-group-list'}->{'user_group'};
        } else {
            $hdl = [];
            $result->{$iname}->{'authroized-user-group-list'} = {'user_group' => $hdl};
        }
        push(@$hdl, {'user_group.name' => $gname});
    }

    return (0, $result);
}

sub insertImage {
    my ($self, $name, $attrs) = @_;

    my @members = ();
    my @cols = ();
    my @vals = ();
    my @args = ();
    my $hdl;

    push(@cols, "#>image.name<#");
    push(@vals, '?');
    push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			if ($key eq 'authroized-user-group-list') {
				$hdl = $attrs->{$key};
				foreach my $i (@$hdl) {
					push(@members, $i);
				}
			} else {
				push(@cols, "#>image.$key<#");
				push(@vals, '?');
				push(@args, [$attrs->{$key}]);
			}
		}
	}

    my $sql = sprintf('insert into #>image<# (%s) values (%s)', join(',', @cols), join(',', @vals));

    my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'insert image', "Failed to create new image. Reason: $param");
        return (-1, "Failed to create new image. Reason: $param");
    }

    foreach my $group_name (@members) {
        $sql = sprintf('insert into #>image_user_group<# (#>image_user_group.image_name<#, #>image_user_group.user_group_name<#) values (?, ?)');
        @args = ([$name], [$group_name]);
        ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
        if ($rc != 0) {
	        Logger->get()->warning('ImageDAO', 'insert image', "Failed to create image group list. Reason: $param");
            return (-1, "Failed to create image group list. Reason: $param");
        }
    }

    return (0);
}

sub modifyImage {
    my ($self, $query, $attrs) = @_;

    my @cols = ();
    my @args = ();
    my $where = '';
    
    if (defined($attrs)) {
        foreach my $key (keys(%$attrs)) {
            push(@cols, "#>image.$key<#=?");
            push(@args, [$attrs->{$key}]);
        }
    }

    if (defined($query)) {
        if (defined($query->{'id'})) {
            $where = "#>image.id<#=?";
            push(@args, [$query->{'id'}]);
        } elsif (defined($query->{'name'})) {
            $where = "#>image.name<#=?";
            push(@args, [$query->{'name'}]);
        } else {
    	    Logger->get()->warning('ImageDAO', 'modify image', "Unsuported user select.");
            return (-1, "Unsuported user select.");
        }
    }

    my $sql = sprintf('update #>image<# set %s where %s', join(',', @cols), $where);

    my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'modify image', "Failed to updagte image info. Reason: $param");
        return (-1, "Failed to updagte image info. Reason: $param");
    }

    return (0);
}

sub modifyImageAuthroizedUserGroups {
    my ($self, $name, $attrs) = @_;

    my @cols = ();
    my @args = ();
    my $where = '';
    my @members = ();
    my $hdl;

    if (defined($attrs)) {
        foreach my $key (keys(%$attrs)) {
            if ($key eq 'authroized-user-group-list') {
                $hdl = $attrs->{$key};
                foreach my $i (@$hdl) {
                    push(@members, values(%$i));
                }
            }
        }
    }

    my $sql = 'delete from #>image_user_group<# where #>image_user_group.image_name<#=?';
    @args = ([$name]);
    my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'modify image authorized users and groups', "Failed to updagte image info. Reason: $param");
        return (-1, "Failed to updagte image info. Reason: $param");
    }

    foreach my $group_name (@members) {
       $sql = 'insert into #>image_user_group<# (#>image_user_group.image_name<#, #>image_user_group.user_group_name<#) VALUES (?, ?)';
       @args = ([$name], [$group_name]);
       ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
       if ($rc != 0) {
    	   Logger->get()->warning('ImageDAO', 'modify image authorized users and groups', "Failed to updagte image info. Reason: $param");
           return (-1, "Failed to updagte image info. Reason: $param");
       }
    }

    return (0);
}

sub deleteImage {
    my ($self, $id) = @_;

    my $sql = 'delete from #>image<# where #>image.id<#=?';
    my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'delete image', "Failed to delete image. Reason: $param");
        return (-1, "Failed to delete image. Reason: $param");
    }

    return (0);
}

sub deleteImageByName {
    my ($self, $name) = @_;

    my $sql = 'delete from #>image<# where #>image.name<#=?';
    my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
    if ($rc != 0) {
    	Logger->get()->warning('ImageDAO', 'delete image by name', "Failed to delete image. Reason: $param");
        return (-1, "Failed to delete image. Reason: $param");
    }

    return (0);
}

1;
