# sccsid = "@(#) src/hpcc/common/db/MachineDAO.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.19 1/17/12 04:46:41";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package MachineDAO;

# --------------------------- #
#
# Machine:
#    id
#    name
#    provision_mode
#    status
#    power_status
#    fail_count
#    hypv_flag
#    interfaces
#    location
#    hostname
#    physicial_name
#    hypervisor
#    cpu_load_avg
#    virtual_memory_used
#    swap_space_used
#    machine_group
#
# MachineGroup:
#    id
#    name
#    catalog
#    core_count
#    cpu_count
#    memory_size
#    arch
#    num_ethernet_interfaces
#    num_high_speed_interfaces
#    high_speed_interface_type
#    description
#    machine-list
#        machine1
#        machine2
#        ...
#        machineN
#
# --------------------------- #

use warnings;
use strict;
require Exporter;
require DB::DBA;
our @ISA = qw(Exporter DAO);
require DB::DAO;

use Utils::Logger;

sub new {
    my $self = DAO->new();
    bless ($self);
    return $self;
}

sub queryAllMachineName {
	my ($self) = @_;

	my $sql = "select #>machine.name<# from #>machine<#";

	my ($rc, $param) = $self->{'dba'}->querySQL($sql);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'NA', "Failed to query machine info. Reason: $param");
		return (-1, "Failed to query machine info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'NA', "Failed to fetch machine info. Reason: $param");
		return (-1, "Failed to fetch machine info. Reason: $param");
	}

	return (0, [sort(keys(%$param))]);
}

sub queryMachine {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	my $mach_key;
	if (defined($args->{'id'})) {
		$where = ' where #>machine.id<#=?';
		push(@$bind, [$args->{'id'}]);
		$mach_key = $args->{'id'};
	}
	elsif (defined($args->{'name'})) {
		$where = ' where #>machine.name<#=?';
		push(@$bind, [$args->{'name'}]);
		$mach_key = $args->{'name'};
	} else {
		Logger->get()->warning('MachineDAO', 'query machine', "Unknown query.");
		return (-1, "Unknown query.");
	}

	if (defined($args->{'hypv_flag'})) {
		$where .= ' and #>machine.hypv_flag<#=?';
		push(@$bind, [$args->{'hypv_flag'}]);
	}

	if (defined($args->{'type'})) {
		$where .= ' and #>machine.type<#=?';
		push(@$bind, [$args->{'type'}]);
	}

	$sql = 'select #>machine.*<#, #>request_machine.request_name<#, #>request.start_time<#, #>request.status<# from #>machine<# left join #>request_machine<# on #>machine.name<#=#>request_machine.machine_name<# left join #>request<# on #>request_machine.request_name<#=#>request.name<#' . $where;
	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'query machine', "Failed to query machine. Reason: $param");
		return (-1, "Failed to query machine. Reason: $param");
	}
=description:
the code change is a temporay solution, now, we do not fetch all but fetch the the one which start time is the most clost to current time
the best method is use fetchall to get all information of one machine, but it seems the machine name can not be key otherwise, it can only
get one record.
=cut
	my $record_num=0;
	my $tmp_param=0;
	my $latest_start_time= 0;
	while($rc == 0) {
		($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>machine.name<#', '#>machine<#');
		if (($rc != 0)&&($record_num == 0)) {
			Logger->get()->warning('MachineDAO', 'query machine', "Failed to fetch machine info. Reason: $param");
			return (-1, "Failed to fetch machine info. Reason: $param");
		}
		if(!exists($param->{$mach_key}) || !$param->{$mach_key}->{'machine.name'}){
			last;
		}

		# Find out the first request for the machine.
		if (defined($param->{$mach_key}->{'request.start_time'})) {
			my $tmp_start_time = $param->{$mach_key}->{'request.start_time'};
			if($tmp_param == 0){
				$tmp_param = $param;
				$latest_start_time = $tmp_start_time;
			}
			else {
				if($tmp_start_time < $latest_start_time){
					$latest_start_time = $tmp_start_time;
					$tmp_param = $param;
				}
				else{
					$record_num++;
					next;
				}
			}
		} else { # There is no request info for this machine.
			$tmp_param = $param;	
		}

		$record_num++;
	}
	$self->{'dba'}->finish();
	if($tmp_param !=0){
		$param = $tmp_param;
	}
    my $result = $param;
    if (!scalar(%$param)) {
    	Logger->get()->info('MachineDAO', 'query machine', "No record.");
    	return (100, "No record.");
    }
    return (0, $result);
}

sub queryAllMachine {
    my ($self) = @_;

    my ($rc, $param);
    my $st;
    my $result;

    # get all machine info;

    my $sql = 'select #>machine.name<#, #>machine.machine_group<# from #>machine<#';

    ($rc, $param) = $self->{'dba'}->querySQL($sql);
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query alll machine', "Failed to query all machine info. Reason: $param");
        return (-1, "Failed to query all machine info. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query alll machine', "Failed to query all user info. Reason: $param");
        return (-1, "Failed to query all user info. Reason: $param");
    }
    
    $self->{'dba'}->finish();

    $result = $param;
	if (!scalar(%$param)){
		Logger->get()->info('MachineDAO', 'query alll machine', "No record.");
	    return (100, "No record.");
	}

    return (0, $result);
}

sub insertMachine {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>machine.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>machine.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>machine<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', "insertMachine $name", "Failed to create new machine. Reason: $param");
		return (-1, "Failed to create new machine. Reason: $param");
	}

	return (0);
}

sub deleteMachine {
	my ($self, $args) = @_;

	my $where = '';
	my $bind = [];

	if (defined($args->{'id'})) {
		$where = ' #>machine.id<#=?';
		push(@$bind, [$args->{'id'}]);
	}
	elsif (defined($args->{'name'})) {
		$where = ' #>machine.name<#=?';
		push(@$bind, [$args->{'name'}]);
	}
	else {
		Logger->get()->warning('MachineDAO', "deleteMachine", "Unknown delete.");
		return (-1, "Unknown delete.");
	}

	my $sql = 'delete from #>machine<# where ' . $where;
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', "deleteMachine", "Failed to delete machine. Reason: $param");
		return (-1, "Failed to delete machine. Reason: $param");
	}

	return (0);
}

sub modifyMachine {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>machine.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		if (defined($query->{'id'})) {
			$where = "#>machine.id<#=?";
			push(@args, [$query->{'id'}]);
		}
		elsif (defined($query->{'name'})) {
			$where = "#>machine.name<#=?";
			push(@args, [$query->{'name'}]);
		}
		else {
			Logger->get()->warning('MachineDAO', "modifyMachine", "Unsuported machine select.");
			return (-1, "Unsuported machine select.");
		}
	}

	my $sql = sprintf('update #>machine<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', "modifyMachine", "Failed to modify machine. Reason: $param");
		return (-1, "Failed to modify machine. Reason: $param");
	}

	return (0);
}

sub queryMachineGroup {
    my ($self, $args) = @_;

    my $sql;
    my $where = '';
    my $bind = [];
	my $where1 = '';
	my $where2 = '';
    if (defined($args->{'id'})) {
        $where1 = ' where #>machine_group.id<#=?';
        $where2 = ' join #>machine_group<# on #>machine_group.name<#=#>machine.machine_group<# where #>machine_group.id<#=?';
        push(@$bind, [$args->{'id'}]);
    }
    elsif (defined($args->{'name'})) {
        $where1 = ' where #>machine_group.name<#=?';
        $where2 = ' where #>machine.machine_group<#=?';
        push(@$bind, [$args->{'name'}]);
    } else {
    	Logger->get()->warning('MachineDAO', 'query machine group', "Unknown query.");
        return (-1, "Unknown query.");
    }

    $sql = 'select #>machine_group.*<# from #>machine_group<# ' . $where1;

    my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query machine group', "Failed to query machine group. Reason: $param");
        return (-1, "Failed to query machine group. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>machine_group.name<#', '#>machine_group<#');
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query machine group', "Failed to fetch machine group info. Reason: $param");
        return (-1, "Failed to fetch machine group info. Reason: $param");
    }
    $self->{'dba'}->finish();

    my $result = $param;

    if (!scalar(%$param)) {
		Logger->get()->info('MachineDAO', 'queryMachineGroup', "No record.");
    	return (100, "No record.");
    }

    # get machine-list
    
    $sql = 'select #>machine.name<# from #>machine<# ' . $where2;
    ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query machine group', "Failed to query machine group. Reason: $param");
        return (-1, "Failed to query machine group. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query machine group', "Failed to fetch machine group info. Reason: $param");
        return (-1, "Failed to fetch machine group info. Reason: $param");
    }

    my $hdl = [];
    foreach (values(%$result)) {
        $_->{'machine-list'}->{'machine'} = $hdl;
    }
    foreach my $i (keys(%$param)) {
        push(@$hdl, {'machine.name' => $i});
    }
    $self->{'dba'}->finish();

    return (0, $result);
}

sub queryAllMachineGroup {
    my ($self, $args) = @_;
    #Currently we do not support group, do not use it
    my $sql = 'select #>machine_group.*<# from #>machine_group<#';
    my ($rc, $param) = $self->{'dba'}->querySQL($sql);
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query all machine group', "Failed to query machine group. Reason: $param");
        return (-1, "Failed to query machine group. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine_group.name<#', '#>machine_group<#');
    if ($rc != 0) {
    	Logger->get()->warning('MachineDAO', 'query all machine group', "Failed to fetch machine group info. Reason: $param");
        return (-1, "Failed to fetch machine group info. Reason: $param");
    }

    $self->{'dba'}->finish();

    my $result = $param;
	if (!scalar(%$param)){
		Logger->get()->info('MachineDAO', 'queryAllMachineGroup', "No record.");
	    return (100, "No record.");
	}

    return (0, $result);
}

sub insertMachineGroup {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>machine_group.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>machine_group.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>machine_group<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'insertMachineGroup',"Failed to create new machine_group. Reason: $param"); 
		return (-1, "Failed to create new machine_group. Reason: $param");
	}

	return (0);
}

sub deleteMachineGroup {
	my ($self, $args) = @_;

	my $where = '';

	if (defined($args->{'id'})) {
		$where = ' #>machine_group.id<#=?';
	}
	elsif (defined($args->{'name'})) {
		$where = ' #>machine_group.name<#=?';
	}
	else {
		Logger->get()->warning('MachineDAO', 'deleteMachineGroup', "Unknown delete.");
		return (-1, "Unknown delete.");
	}

	my $sql = 'delete from #>machine_group<# where ' . $where;

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, $args);
	if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'deleteMachineGroup', "Failed to delete machine_group. Reason: $param");
		return (-1, "Failed to delete machine_group. Reason: $param");
	}

	return (0);
}

sub queryAllPMach {
    my ($self) = @_;

    my $sql = 'select #>machine.*<# #>energy_usage.acc_power<# #>energy_usage.start_time<# #>energy_usage.sampling_times<# #>energy_usage.avg_power<# from #>machine<# left join #>energy_usage<# on (#>machine.name<#=#>energy_usage.machine_name<#) where #>machine.provision_mode<# in (2,3) and #>machine.hypv_flag<#=0';

    my ($rc, $param) = $self->{'dba'}->querySQL($sql);

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMach', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMach', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    $self->{'dba'}->finish();

	if (!scalar(%$param)){
		Logger->get()->info('MachineDAO', 'queryAllMachineGroup', "No record.");
    	return (100, "No record.");
	}

    return (0, $param);
}

sub queryAllPMachName {
    my ($self) = @_;

    my $sql = 'select #>machine.name<# from #>machine<# where #>machine.provision_mode<# in (2,3) and #>machine.hypv_flag<#=0';

    my ($rc, $param) = $self->{'dba'}->querySQL($sql);

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMachName', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMachName', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    $self->{'dba'}->finish();
	if (!scalar(%$param)){
		Logger->get()->info('MachineDAO', 'queryAllPMachName', "No record.");
   	 	return (100, "No record.");
	}

    return (0, $param);
}

sub queryAllPhyMachName{
    my ($self) = @_;

    my $sql = 'select #>machine.name<# from #>machine<# where #>machine.hypv_flag<#!=-1';

    my ($rc, $param) = $self->{'dba'}->querySQL($sql);

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMachName', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    ($rc, $param) = $self->{'dba'}->fetchall_hashref('#>machine.name<#', '#>machine<#');

    if ($rc != 0) {
		Logger->get()->warning('MachineDAO', 'queryAllPMachName', "Failed to query machine. Reason: $param");
        return (-1, "Failed to query machine. Reason: $param");
    }

    $self->{'dba'}->finish();
	if (!scalar(%$param)){
		Logger->get()->info('MachineDAO', 'queryAllPMachName', "No record.");
   	 	return (100, "No record.");
	}

    return (0, $param);
}
1;
