# sccsid = "@(#) src/hpcc/common/db/NameUtils.pm, hpc.cloud.comm, hpcc_rbei, rbeis001a 1.4 6/15/11 11:34:43";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2011 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package NameUtils;

use DB::NameSchema;

use strict;
use warnings;

#
# DESC:
#   prepare the SQL statement, replace the table/column names
#   with the real table/column names;
#
#   one prepareable pattern should like this #>...<#
#   the ... can be either:
#       logical-table-name
#     or
#       logical-table-name.logical-column-name
#
#     one example:
#      <input>
#       insert into #>role<# (#>role.name<#) values ('leader');
#      <output>
#       insert into thc_role (role_name) values ('leader');
# 
# INPUT:
#   param1: package name;
#   param2: the SQL statement;
# OUTPUT:
#   ret1: return code
#         0 - SUCC
#         others - FAIL
#   ret2 :
#         SUCC - new SQL statment;
#         FAIL - failure reason;
#
sub prepare {
    my ($package, $sql) = @_;

    my $key;
    my $tab;
    my $col;

    my $rc;
    my $param;

    my $r_tab_name;
    my $r_col_name;

    my $temp;

    my @replacement = ();

    while ($sql =~ /#>([^#<>]+)<#/) {
        $key = $1;
        ($tab, $col) = split(/\./, $key);
        if (!defined($col)) {
            ($rc, $param) = NameSchema->getTableRealName($tab);
            if ($rc != 0) {
                return (-1, $param);
            }
            $r_tab_name = $param;
            $temp = $r_tab_name;
        } else {
            ($rc, $param) = NameSchema->getTableRealName($tab);
            if ($rc != 0) {
                return (-1, $param);
            }
            $r_tab_name = $param;

            if ($col ne '*') {
                ($rc, $param) = NameSchema->getColumnRealName($tab, $col);
                if ($rc != 0) {
                    return (-1, $param);
                }
                $r_col_name = $param;

                push(@replacement, $key);
            } else {
                $key =~ s/\*/\\\*/g;
                $r_col_name = '*';

                my $cols = NameSchema->getTableColumnExternalNames($tab);
                push(@replacement, @$cols);
            }
            $temp = $r_tab_name . '.' . $r_col_name;
        }

        $sql =~ s/#>$key<#/$temp/g;
    }

    return (0, $sql, \@replacement);
}

#
# DESC:
#   fetch the query result and pack the SQL query result,
#   replace the physical table/column names with the logical
#   table/column names;
#
#     one example:
#      <physical>
#        { 'yujing' = > {
#            'id' => 1,
#            'user_name' => 'yujing',
#            'uid' => '1001',
#            'role_id' => '1',
#            'first_name' => 'Jing',
#            'last_name' => 'Yu',
#            'email' => 'jyuj@cn.ibm.com',
#            'phone' => '(+86)13811613359',
#            'department' => 'LoadLeveler DEV, CSTL',
#            'organization' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#          'huxiaoying' = > {
#            'id' => 2,
#            'user_name' => 'huxiaoying',
#            'uid' => '1002',
#            'role_id' => '1',
#            'first_name' => 'Xiaoying',
#            'last_name' => 'Hu',
#            'email' => 'huxiaoy@cn.ibm.com',
#            'phone' => '(+86)1NNNNNNNNNN',
#            'department' => 'LoadLeveler DEV, CSTL',
#            'organization' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#        }
#      <logical>
#        { 'yujing' = > {
#            'id' => 1,
#            'name' => 'yujing',
#            'uid' => '1001',
#            'rid' => '1',
#            'fn' => 'Jing',
#            'ln' => 'Yu',
#            'email' => 'jyuj@cn.ibm.com',
#            'phone' => '(+86)13811613359',
#            'dept' => 'LoadLeveler DEV, CSTL',
#            'org' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#          'huxiaoying' = > {
#            'id' => 2,
#            'name' => 'huxiaoying',
#            'uid' => '1002',
#            'rid' => '1',
#            'fn' => 'Xiaoying',
#            'ln' => 'Hu',
#            'email' => 'huxiaoy@cn.ibm.com',
#            'phone' => '(+86)1NNNNNNNNNN',
#            'dept' => 'LoadLeveler DEV, CSTL',
#            'org' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#        }
#
# INPUT:
#   param1: package name;
#   param2: the SQL query statement;
#   param3: key column name (logical name);
#   param4: default table name (logical name);
# OUTPUT:
#   ret1: return code
#         0 - SUCC
#         others - FAIL
#   ret2 :
#         SUCC - the result hash handle;
#         FAIL - failure reason;
#
sub fetchall_hashref {
    my ($package, $st, $primary_key, $default_table, $column_names) = @_;
    
    my $i_table_name;
    my $i_column_name;
    my $e_table_name;
    my $e_column_name;
    
    my $rc;
    my $param;

    # Remove the leading [> and tailing <]
    $primary_key =~ s/^#>//;
    $primary_key =~ s/<#$//;

    $default_table =~ s/^#>//;
    $default_table =~ s/<#$//;

    ($e_table_name, $e_column_name) = split(/\./, $primary_key);
    
    unless (defined($e_column_name)) {
        $primary_key = $default_table . '.' . $primary_key;
    }
    
    my %mapping = ();
    my $result = {};
    my $array;
    while ($array = $st->fetchrow_arrayref()) {
        my $hash = {};
        my $index = 0;
        foreach my $itor (@$array) {
            if (defined($itor)) {
                my $temp = $column_names->[$index];
                $hash->{$temp} = $itor;
                if ($temp eq $primary_key) {
                    $result->{$itor} = $hash;
                }
            }
            $index++;
        }
    }
    
    return (0, $result);
}

#
# DESC:
#   fetch one row result and pack the SQL query result,
#   replace the physical table/column names with the logical
#   table/column names;
#
#     one example:
#      <physical>
#        { 'yujing' = > {
#            'id' => 1,
#            'user_name' => 'yujing',
#            'uid' => '1001',
#            'role_id' => '1',
#            'first_name' => 'Jing',
#            'last_name' => 'Yu',
#            'email' => 'jyuj@cn.ibm.com',
#            'phone' => '(+86)13811613359',
#            'department' => 'LoadLeveler DEV, CSTL',
#            'organization' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#        }
#      <logical>
#        { 'yujing' = > {
#            'id' => 1,
#            'name' => 'yujing',
#            'uid' => '1001',
#            'rid' => '1',
#            'fn' => 'Jing',
#            'ln' => 'Yu',
#            'email' => 'jyuj@cn.ibm.com',
#            'phone' => '(+86)13811613359',
#            'dept' => 'LoadLeveler DEV, CSTL',
#            'org' => 'IBM comp',
#            'comments' => 'HPC Cloud developer',
#          },
#        }
#
# INPUT:
#   param1: package name;
#   param2: the SQL query statement;
#   param3: key column name (logical name);
#   param4: default table name (logical name);
# OUTPUT:
#   ret1: return code
#         0 - SUCC
#         others - FAIL
#   ret2 :
#         SUCC - the result hash handle;
#         FAIL - failure reason;
#
sub fetchrow_hashref {
    my ($package, $st, $primary_key, $default_table, $column_names) = @_;

    my $i_table_name;
    my $i_column_name;
    my $e_table_name;
    my $e_column_name;

    my $rc;
    my $param;

    # Remove the leading [> and tailing <]
    $primary_key =~ s/^#>//;
    $primary_key =~ s/<#$//;

    $default_table =~ s/^#>//;
    $default_table =~ s/<#$//;

    ($e_table_name, $e_column_name) = split(/\./, $primary_key);

    unless (defined($e_column_name)) {
        $primary_key = $default_table . '.' . $primary_key;
    }

    my %mapping = ();
    my $result = {};
    my $array;
    if ($array = $st->fetchrow_arrayref()) {
        my $hash = {};
        my $index = 0;
        foreach my $itor (@$array) {
            if (defined($itor)) {
                my $temp = $column_names->[$index];
                $hash->{$temp} = $itor;
                if ($temp eq $primary_key) {
                    $result->{$itor} = $hash;
                }
            }
            $index++;
        }
    }

    return (0, $result);

}

sub fetchrow_arrayref {
    my ($package, $st) = @_;

    my $result = $st->fetchrow_arrayref();

    if(!defined($result)) {
        my $param = "Error to fetchrow-arrayref!";
        return (-1, $param);
    }

    return (0, $result);
}

sub fetchall_arrayref {
    my ($package, $st) = @_;

    my $result = $st->fetchall_arrayref();

    if(!defined($result)) {
        my $param = "Error to fetchall-arrayref!";
        return (-1, $param);
    }

    return (0, $result);
}

1;
