# sccsid = "@(#) src/hpcc/common/db/UserDAO.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.7 1/6/12 04:12:22";
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package UserDAO;

# --------------------------- #
#
# User:
#	id
#	uname
#	uid
#	role
#	email
#	FN
#	LN
#	phone
#	dept
#	org
#	comments
#	group-list:
#		gname1
#		gname2
#		...
#		gnameN
#
# --------------------------- #

# --------------------------- #
#
# Group:
#	id
#	gname
#	owner
#	desc
#	user-list:
#		uname1
#		uname2
#		...
#		unameN
#
# --------------------------- #

# --------------------------- #
#
# Role:
#	id
#	name
#	desc
#	cap-list:
#		cap_name1
#		cap_name2
#		...
#		cap_nameN
#
# --------------------------- #

# --------------------------- #
#
# Capability:
#	id
#	name
#	value
#
# --------------------------- #

use warnings;
use strict;
require Exporter;
require DB::DBA;
require DB::DAO;
our @ISA = qw(Exporter DAO);

use Utils::Logger;

sub new {
	my $self = DAO->new();
	bless ($self);
	return $self;
}

# ========================
# User related operations
# ========================
sub queryUser {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	if (defined($args->{'id'})) {
		$where = ' #>user.id<#=?';
		push(@$bind, [$args->{'id'}]);
	}
	elsif (defined($args->{'name'})) {
		$where = ' #>user.name<#=?';
		push(@$bind, [$args->{'name'}]);
	}
	elsif (defined($args->{'uid'})) {
		$where = ' #>user.uid<#=?';
		push(@$bind, [$args->{'uid'}]);
	} else {
		return (-1, "Unknown query.");
	}
	
	$sql = 'select #>user.*<# from #>user<# where ' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query user info. Reason: $param");
		return (-1, "Failed to query user info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>user.name<#', '#>user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query user info. Reason: $param");
		return (-1, "Failed to query user info. Reason: $param");
	}

	my $result = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query user', "No record.");
		return (100, "No record.");
	}

	# get user group info;

	$sql = 'select #>user_group_user.id<#,#>user_group_user.group_name<#,#>user_group_user.user_name<# from #>user_group_user<# join #>user<# on #>user_group_user.user_name<#=#>user.name<# where ' . $where;

	($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user_group_user.id<#', '#>user_group_user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	my $ugroup = $param;
	foreach my $uguid (keys(%$ugroup)) {
		my $gname = $ugroup->{$uguid}->{'user_group_user.group_name'};
		my $uname = $ugroup->{$uguid}->{'user_group_user.user_name'};
		my $hdl;
		if (defined($result->{$uname}->{'group-list'})) {
			$hdl = $result->{$uname}->{'group-list'}->{'group'};
		} else {
			$hdl = [];
			$result->{$uname}->{'group-list'} = {'group' => $hdl };
		}
		push(@$hdl, {'group.name' => $gname});
	}

	# get user role info;
	$sql = 'select #>role_capability.id<#, #>user.name<#, #>role_capability.role_name<#,#>capability.name<# from ((#>role_capability<# join #>capability<# on #>role_capability.cap_name<#=#>capability.name<#) left join #>role<# on #>role_capability.role_name<#=#>role.name<#) left join #>user<# on #>user.role<#=#>role.name<# where ' . $where;

	($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>role_capability.id<#', '#>role_capability<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	my $urole = $param;
	foreach my $i (keys(%$urole)) {
		my $role_name = $urole->{$i}->{'role_capability.role_name'};
		my $uname2 = $urole->{$i}->{'user.name'};
		my $cap_name = $urole->{$i}->{'capability.name'};
		my $hdl;
		$result->{$uname2}->{'role.name'} = $role_name;
		if (defined($result->{$uname2}->{'cap-list'})) {
			$hdl = $result->{$uname2}->{'cap-list'}->{'capability'};
		} else {
			$hdl = [];
			$result->{$uname2}->{'cap-list'} = {'capability' => $hdl};
		}
		push(@$hdl, {'capability.name'=>$cap_name});
	}
	
	return (0, $result);
}

sub queryAllUser {
	my ($self) = @_;

	my ($rc, $param);
	my $st;
	my $result;
	my $ugroup;

	# get all user info;

	my $sql = 'select #>user.*<# from #>user<#';

	($rc, $param) = $self->{'dba'}->querySQL($sql);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user.name<#', '#>user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}
	
	$result = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query all user', "No record.");
		return (100, "No record.");
	}

	# get all user group info;
	$sql = 'select #>user_group_user.id<#,#>user_group_user.group_name<#,#>user_group_user.user_name<# from #>user_group_user<#';

	($rc, $param) = $self->{'dba'}->querySQL($sql);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	$st = $param;
	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user_group_user.id<#', '#>user_group_user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all user', "Failed to query all user info. Reason: $param");
		return (-1, "Failed to query all user info. Reason: $param");
	}

	$ugroup = $param;
	foreach my $uguid (keys(%$ugroup)) { 
		my $gname = $ugroup->{$uguid}->{'user_group_user.group_name'};
		my $uname = $ugroup->{$uguid}->{'user_group_user.user_name'};
		my $hdl;
		if (defined($result->{$uname}->{'group-list'})) {
			$hdl = $result->{$uname}->{'group-list'}->{'group'};
		} else {
			$hdl = [];
			$result->{$uname}->{'group-list'} = {'group' => $hdl};
		}
		push(@$hdl, {'group.name' => $gname});
	}

	return (0, $result);
}

sub insertUser {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>user.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>user.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>user<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'insert user', "Failed to create new user. Reason: $param");
		return (-1, "Failed to create new user. Reason: $param");
	}

	return (0);
}

sub modifyUser {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	
	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>user.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}
  	my $userid=""; 
	if (defined($query)) {
		if (defined($query->{'id'})) {
			$where = "#>user.id<#=?";
			push(@args, [$query->{'id'}]);
			$userid=$query->{'id'};
		} elsif (defined($query->{'name'})) {
			$where = "#>user.name<#=?";
			push(@args, [$query->{'name'}]);
			$userid=$query->{'name'};
		} elsif (defined($query->{'uid'})) {
			$where = "#>user.uid<#=?";
			push(@args, [$query->{'uid'}]);
			$userid=$query->{'uid'};
		} else {
			return (-1, "Unsuported user select.");
		}
	}

	my $sql = sprintf('update #>user<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify user', "Failed to update the user $userid. Reason: $param");
		return (-1, "Failed to update the user $userid. Reason: $param");
	}

	return (0);
}

=comments=======================================
Function Define: ($rc, $param) = removeUserRole($username)
Description: Remove the role for the specified user
Input: the user name
Return:
	$rc: 0 - succ, -1 - fail
	$param: failure reason
=cut
sub removeUserRole {
	my ($self, $uname) = @_;

	my @args = ();
	push(@args, [$uname]);
	my $where = "#>user.name<#=?";

	my $sql = sprintf('update #>user<# set role_name=NULL where %s', $where);
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'removeUserRole', "Failed to remove the role for user $uname. Reason: $param");
		return (-1, "Failed to remove the role for user $uname. Reason: $param");
	}

	return (0);
}

sub deleteUser {
	my ($self, $id) = @_;

	my $sql = 'delete from #>user<# where #>user.id<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete user', "Failed to delete user. Reason: $param");
		return (-1, "Failed to delete user. Reason: $param");
	}

	return (0);
}

sub deleteUserByName {
	my ($self, $name) = @_;

	my $sql = 'delete from #>user<# where #>user.name<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete user by name', "Failed to delete user. Reason: $param");
		return (-1, "Failed to delete user. Reason: $param");
	}

	return (0);
}


# ==========================
# Group related operations
# ==========================

sub queryGroup {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $group_info;
	
	my $bind = [];
	if (defined($args->{'id'})) {
		$where = "where #>user_group.id<#=?";
		push(@$bind, [$args->{'id'}]);
	}
	elsif (defined($args->{'name'})) {
		$where = "where #>user_group.name<#=?";
		push(@$bind, [$args->{'name'}]);
	} else {
		Logger->get()->warning('db', 'query group', "Unknown query.");
		return (-1, "Unknown query.");
	}

	# query group info
	$sql = 'select #>user_group.*<# from #>user_group<# ' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query group', "Failed to query user group basic info. Reason: $param");
		return (-1, "Failed to query user group basic info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>user_group.name<#', '#>user_group<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query group', "Failed to query user group basic info. Reason: $param");
		return (-1, "Failed to query user group basic info. Reason: $param");
	}

	$group_info = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query group', "No record.");
		return (100, "No record.");
	}

	my $group_name;
	foreach my $i (keys(%$group_info)) {
		$group_name = $i;
		last;
	}
	
	# query members 
	$sql = 'select #>user_group_user.user_name<# from #>user_group_user<# where #>user_group_user.group_name<#=?';
	$bind = [[$group_name]];
	($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query group', "Failed to query user group member info. Reason: $param");
		return (-1, "Failed to query user group member info. Reason: $param");
	}
	
	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user_group_user.user_name<#', '#>user_group_user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query group', "Failed to query user group member info. Reason: $param");
		return (-1, "Failed to query user group member info. Reason: $param");
	}
	
	my $hdl = [];
	$group_info->{$group_name}->{'user-list'} = {'user' => $hdl};
	foreach my $i (keys(%$param)) {
		push(@$hdl, {'user.name'=>$i});
	}
	
	return (0, $group_info);
}

sub queryAllGroup {
	my ($self, $query) = @_;

	my $where1 = '';
	my $where2 = '';
	my @bind = ();

	my $bind = [];
	if (defined($query->{'owner'})) {
		$where1 = "where #>user_group.owner<#=?";
		$where2 = "join #>user_group<# on #>user_group_user.group_name<#=#>user_group.name<# where #>user_group.owner<#=?";
		push(@bind, [$query->{'owner'}]);
	} elsif (scalar(%$query) > 0) {
		Logger->get()->warning('db', 'query all group', "Unknown query.");
		return (-1, "Unknown query.");
	}

	my ($rc, $param);
	my $st;
	my $result;
	my $ugroup;

	# get all user group info;
	my $sql = 'select #>user_group.*<# from #>user_group<# '. $where1;
	($rc, $param) = $self->{'dba'}->querySQL($sql, \@bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all group', "Failed to query all user group info. Reason: $param");
		return (-1, "Failed to query all user group info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user_group.name<#', '#>user_group<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all group', "Failed to query all user group info. Reason: $param");
		return (-1, "Failed to query all user group info. Reason: $param");
	}

	$result = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query all group', "No record.");
		return (100, "No record.");
	}

	$sql = 'select #>user_group_user.id<#,#>user_group_user.group_name<#,#>user_group_user.user_name<# from #>user_group_user<# '. $where2;
	($rc, $param) = $self->{'dba'}->querySQL($sql, \@bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all group', "Failed to query all user group info. Reason: $param");
		return (-1, "Failed to query all user group info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>user_group_user.id<#', '#>user_group_user<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all group', "Failed to query all user group info. Reason: $param");
		return (-1, "Failed to query all user group info. Reason: $param");
	}

	foreach my $i (keys%$param) {
		my $group_name = $param->{$i}->{'user_group_user.group_name'};
		my $user_name = $param->{$i}->{'user_group_user.user_name'};
		my $hdl;
		if (defined($result->{$group_name}->{'user-list'})) {
			$hdl = $result->{$group_name}->{'user-list'}->{'user'};
		} else {
			$hdl = [];
			$result->{$group_name}->{'user-list'} = {'user' => $hdl};
		}
		push(@$hdl, {'user.name'=>$user_name});
	}


	return (0, $result);
}

sub insertGroup {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>user_group.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	my @members = ();
	my $hdl;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			if ($key eq 'user-list') {
				$hdl = $attrs->{$key};
				foreach my $i (@$hdl) {
					push(@members, values(%$i));
				}
			} else {
				push(@cols, "#>user_group.$key<#");
				push(@vals, '?');
				push(@args, [$attrs->{$key}]);
			}
		}
	}

	my $sql = sprintf('insert into #>user_group<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'insert group', "Failed to create new user group. Reason: $param");
		return (-1, "Failed to create new user group. Reason: $param");
	}

	foreach my $user_name(@members) {
		$sql = sprintf('insert into #>user_group_user<# (#>user_group_user.group_name<#, #>user_group_user.user_name<#) values (?, ?)');
		@args = ([$name], [$user_name]);
		($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
		if ($rc != 0) {
			Logger->get()->warning('db', 'insert group', "Failed to create user group member list. Reason: $param");
			return (-1, "Failed to create user group member list. Reason: $param");
		}
	}

	return (0);
}

sub modifyGroup {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>user_group.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		if (defined($query->{'id'})) {
			$where = "#>user_group.id<#=?";
			push(@args, [$query->{'id'}]);
		} elsif (defined($query->{'name'})) {
			$where = "#>user_group.name<#=?";
			push(@args, [$query->{'name'}]);
		} else {
			Logger->get()->warning('db', 'modify group', "Unsuported user group select.");
			return (-1, "Unsuported user group select.");
		}
	}

	my $sql = sprintf('update #>user_group<# set %s where %s', join(',', @cols), $where);
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify group', "Failed to update user group info. Reason: $param");
		return (-1, "Failed to update user group info. Reason: $param");
	}

	return (0);
}

sub modifyGroupMembers {
	my ($self, $group_name, @members) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';

	my $sql = sprintf('delete from #>user_group_user<# where #>user_group_user.group_name<#=?');
	push(@args, [$group_name]);
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify group members', "Failed to update user group member list. Reason: $param");
		return (-1, "Failed to update user group member list. Reason: $param");
	}

	foreach my $user_name (@members) {
		$sql = sprintf('insert into #>user_group_user<# (#>user_group_user.group_name<#, #>user_group_user.user_name<#) values (?, ?)');
		@args = ([$group_name], [$user_name]);
		($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
		if ($rc != 0) {
			Logger->get()->warning('db', 'modify group members', "Failed to update user group member list. Reason: $param");
			return (-1, "Failed to update user group member list. Reason: $param");
		}
	}
	
	return (0);
}

sub deleteGroup {
	my ($self, $id) = @_;
	my $sql = '';

	$sql = 'delete from #>user_group_user<# join #>user_group<# on #>user_group_user.group_name<#=#>user_group.name<# where #>user_group.id<#=?';
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete group', "Failed to delete user group. Reason: $param");
		return (-1, "Failed to delete user group. Reason: $param");
	}

	$sql = 'delete from #>user_group<# where #>user_group.id<#=?';

	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete group', "Failed to delete user group. Reason: $param");
		return (-1, "Failed to delete user group. Reason: $param");
	}

	return (0);
}

sub deleteGroupByName {
	my ($self, $name) = @_;

	my $sql = 'delete from #>user_group_user<# where #>user_group_user.group_name<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete group by name', "Failed to delete user group. Reason: $param");
		return (-1, "Failed to delete user group. Reason: $param");
	}

	$sql = 'delete from #>user_group<# where #>user_group.name<#=?';
	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete group by name', "Failed to delete user group. Reason: $param");
		return (-1, "Failed to delete user group. Reason: $param");
	}

	return (0);
}

# ==========================
# Role related operations
# ==========================

sub queryRole {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $role_info;
	my ($rc, $param);
	
	my $bind = [];
	if (defined($args->{'id'})) {
		$where = "where #>role.id<#=?";
		push(@$bind, [$args->{'id'}]);
	} elsif (defined($args->{'name'})) {
		$where = "where #>role.name<#=?";
		push(@$bind, [$args->{'name'}]);
	} else {
		return (-1, "Unknown query.");
	}

	# query group info
	$sql = 'select #>role.*<# from #>role<# ' . $where;

	($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query role', "Failed to query role basic info. Reason: $param");
		return (-1, "Failed to query role basic info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>role.name<#', '#>role<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query role', "Failed to query role basic info. Reason: $param");
		return (-1, "Failed to query role basic info. Reason: $param");
	}

	$role_info = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query role', "No record.");
		return (100, "No record.");
	}

	my $role_name = 0;
	
	foreach my $i (keys(%$role_info)) {
		$role_name = $i;
		last;
	}
	
	# query members 
	$sql = 'select #>capability.name<# from #>role_capability<# join #>capability<# on #>role_capability.cap_name<#=#>capability.name<# where #>role_capability.role_name<#=?';
	$bind = [[$role_name]];
	($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query role', "Failed to query capability info. Reason: $param");
		return (-1, "Failed to query capability info. Reason: $param");
	}
	
	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>capability.name<#', '#>capability<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query role', "Failed to query capability info. Reason: $param");
		return (-1, "Failed to query capability info. Reason: $param");
	}
	
	my $hdl = [];
	$role_info->{$role_name}->{'cap-list'} = {'capability' => $hdl};
	foreach my $i (keys(%$param)) {
		my $cap_name = $i;
		push(@$hdl, {'capability.name'=>$cap_name});
	}
	
	return (0, $role_info);
}

sub queryAllRole {
	my ($self) = @_;

	my ($rc, $param);
	my $st;
	my $result;
	my $ugroup;

	# get all user info;

	my $sql = 'select #>role.*<# from #>role<#';

	($rc, $param) = $self->{'dba'}->querySQL($sql);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all role', "Failed to query all user role info. Reason: $param");
		return (-1, "Failed to query all user role info. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>role.name<#', '#>role<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all role', "Failed to query all user role info. Reason: $param");
		return (-1, "Failed to query all user role info. Reason: $param");
	}

	$result = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query all role', "No record.");
		return (100, "No record.");
	}

	return (0, $result);
}

sub insertRole {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>role.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	my @members = ();
	my $hdl;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			if ($key eq 'cap-list') {
				$hdl = $attrs->{$key};
				foreach my $i (@$hdl) {
					push(@members, values(%$i));
				}
			} else {
				push(@cols, "#>role.$key<#");
				push(@vals, '?');
				push(@args, [$attrs->{$key}]);
			}
		}
	}

	my $sql = sprintf('insert into #>role<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'insert role', "Failed to create new user group. Reason: $param");
		return (-1, "Failed to create new user group. Reason: $param");
	}
	
	foreach my $cap_name (@members) {
		$sql = sprintf('insert into #>role_capability<# (#>role_capability.role_name<#, #>role_capability.cap_name<#) values (?, ?)');
		@args = ([$name], [$cap_name]);
		($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
		if ($rc != 0) {
			Logger->get()->warning('db', 'insert role', "Failed to create role capability list. Reason: $param");
			return (-1, "Failed to create role capability list. Reason: $param");
		}
	}

	return (0);
}

sub modifyRole {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>role.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		if (defined($query->{'id'})) {
			$where = "#>role.id<#=?";
			push(@args, [$query->{'id'}]);
		} elsif (defined($query->{'name'})) {
			$where = "#>role.name<#=?";
			push(@args, [$query->{'name'}]);
		} else {
			Logger->get()->warning('db', 'modify role', "Unsuported role select.");
			return (-1, "Unsuported role select.");
		}
	}

	my $sql = sprintf('update #>role<# set %s where %s', join(',', @cols), $where);
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify role', "Failed to update role info. Reason: $param");
		return (-1, "Failed to update role info. Reason: $param");
	}

	return (0);
}

sub modifyRoleCapabilities {
	my ($self, $role_name, @caps) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';
	
	my $sql = sprintf('delete from #>role_capability<# where #>role_capability.role_name<#=?');
	push(@args, [$role_name]);
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify role capabilities', "Failed to modify role capability list. Reason: $param");
		return (-1, "Failed to modify role capability list. Reason: $param");
	}

	foreach my $cap_name (@caps) {
		$sql = sprintf('insert into #>role_capability<# (#>role_capability.role_name<#, #>role_capability.cap_name<#) values (?, ?)');
		@args = ([$role_name], [$cap_name]);
		($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
		if ($rc != 0) {
			Logger->get()->warning('db', 'modify role capabilities', "Failed to modify user role capability list. Reason: $param");
			return (-1, "Failed to modify user role capability list. Reason: $param");
		}
	}

	return (0);
}

sub deleteRole {
	my ($self, $id) = @_;

	my $sql = 'delete from #>role_capability<# join #>role<# on #>role_capability.role_name<#=#>role.name<# where #>role.id<#=?';
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete role', "Failed to delete user role. Reason: $param");
		return (-1, "Failed to delete user role. Reason: $param");
	}

	$sql = 'delete from #>role<# where #>role.id<#=?';
	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete role', "Failed to delete user role. Reason: $param");
		return (-1, "Failed to delete user role. Reason: $param");
	}

	return (0);
}

sub deleteRoleByName {
	my ($self, $name) = @_;

	my $sql = 'delete from #>role_capability<# where #>role_capability.role_name<#=?';
	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete role by name', "Failed to delete user role. Reason: $param");
		return (-1, "Failed to delete user role. Reason: $param");
	}

	$sql = 'delete from #>role<# where #>role.name<#=?';
	($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete role by name', "Failed to delete user role. Reason: $param");
		return (-1, "Failed to delete user role. Reason: $param");
	}

	return (0);
}

# ==========================
# Role related operations
# ==========================

sub queryCapability {
	my ($self, $args) = @_;

	my $sql;
	my $where = '';
	my $bind = [];
	if (defined($args->{'id'})) {
		$where = ' #>capability.id<#=?';
		push(@$bind, [$args->{'id'}]);
	}
	elsif (defined($args->{'name'})) {
		$where = ' #>capability.name<#=?';
		push(@$bind, [$args->{'name'}]);
	} else {
		Logger->get()->warning('db', 'query capability', "Unknown query.");
		return (-1, "Unknown query.");
	}


	$sql = 'select #>capability.*<# from #>capability<# where ' . $where;

	my ($rc, $param) = $self->{'dba'}->querySQL($sql, $bind);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query capability', "Failed to query capability. Reason: $param");
		return (-1, "Failed to query capability. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchrow_hashref('#>capability.name<#', '#>capability<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query capability', "Failed to query capability. Reason: $param");
		return (-1, "Failed to query capability. Reason: $param");
	}
	
	$self->{'dba'}->finish();

	my $result = $param;
	
	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query capability', "No record.");
		return (100, "No record.");
	}

	return (0, $result);
}

sub queryAllCapability {
	my ($self, $args) = @_;

	my $sql = 'select #>capability.*<# from #>capability<#';
	my ($rc, $param) = $self->{'dba'}->querySQL($sql);
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all capability', "Failed to query capability. Reason: $param");
		return (-1, "Failed to query capability. Reason: $param");
	}

	($rc, $param) = $self->{'dba'}->fetchall_hashref('#>capability.name<#', '#>capability<#');
	if ($rc != 0) {
		Logger->get()->warning('db', 'query all capability', "Failed to query capability. Reason: $param");
		return (-1, "Failed to query capability. Reason: $param");
	}
	
	my $result = $param;

	if (!scalar(%$param)) {
		Logger->get()->warning('db', 'query all capability', "No record.");
		return (100, "No record.");
	}

	$self->{'dba'}->finish();
	return (0, $result);
}

sub insertCapability {
	my ($self, $name, $attrs) = @_;

	my @cols = ();
	my @vals = ();
	my @args = ();

	push(@cols, "#>capability.name<#");
	push(@vals, '?');
	push(@args, [$name]);

	my @members = ();
	my $hdl;

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>capability.$key<#");
			push(@vals, '?');
			push(@args, [$attrs->{$key}]);
		}
	}

	my $sql = sprintf('insert into #>capability<# (%s) values (%s)', join(',', @cols), join(',', @vals));

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'insert capability', "Failed to create new capability. Reason: $param");
		return (-1, "Failed to create new capability. Reason: $param");
	}
	
	return (0);

}

sub modifyCapability {
	my ($self, $query, $attrs) = @_;

	my @cols = ();
	my @args = ();
	my $where = '';

	if (defined($attrs)) {
		foreach my $key (keys(%$attrs)) {
			push(@cols, "#>capability.$key<#=?");
			push(@args, [$attrs->{$key}]);
		}
	}

	if (defined($query)) {
		if (defined($query->{'id'})) {
			$where = "#>capability.id<#=?";
			push(@args, [$query->{'id'}]);
		} elsif (defined($query->{'name'})) {
			$where = "#>capability.name<#=?";
			push(@args, [$query->{'name'}]);
		} else {
			Logger->get()->warning('db', 'modify capability', "Unsuported user select.");
			return (-1, "Unsuported user select.");
		}
	}

	my $sql = sprintf('update #>capability<# set %s where %s', join(',', @cols), $where);

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, \@args);
	if ($rc != 0) {
		Logger->get()->warning('db', 'modify capability', "Failed to modify capability. Reason: $param");
		return (-1, "Failed to modify capability. Reason: $param");
	}

	return (0);
}

sub deleteCapability {
	my ($self, $id) = @_;

	my $sql = 'delete from #>capability<# where #>capability.id<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$id]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete capability', "Failed to delete role capability. Reason: $param");
		return (-1, "Failed to delete role capability. Reason: $param");
	}

	return (0);
}

sub deleteCapabilityByName {
	my ($self, $name) = @_;

	my $sql = 'delete from #>capability<# where #>capability.name<#=?';

	my ($rc, $param) = $self->{'dba'}->execSQL($sql, [[$name]]);
	if ($rc != 0) {
		Logger->get()->warning('db', 'delete capability by name', "Failed to delete role capability. Reason: $param");
		return (-1, "Failed to delete role capability. Reason: $param");
	}

	return (0);
}

1;
