#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/energy/xcat/EnergyUtils.pm, hpc.cloud.energy, hpcc_rbei, rbeis002a 1.29 1/5/12 22:54:21";

use strict;
use warnings;
package EnergyUtils;
use GeneralDefine;
use CloudUtils;
use DB::EnergyDAO;
use CmdUtils;

# Add a record into thc_energy_scheme.
# Return: 1- success, undef - fail
sub addEnergyScheme {
	my ($class, $owner, $name, $type, $fixedCPUFreq, $minCPUFreq, $maxCPUFreq, $upThreshold, $downThreshold, $samplingRate, $freqStep, $desc) = @_;

	# owner and name are required.
	if (!$owner || !$name) {return;}

	if (!$type) { $type = 0; }
	if ($type != 0 && $type != 1) {
		# log "invalid type"
		return;
	}

	my $attr = {};
	my $db = EnergyDAO->new();

	$attr->{'owner'} = $owner;
	$attr->{'cat'} = $type;
	$attr->{'cpu_freq_fixed'} = $fixedCPUFreq;
	$attr->{'cpu_freq_min'} = $minCPUFreq;
	$attr->{'cpu_freq_max'} = $maxCPUFreq;
	$attr->{'threshold_up'} = $upThreshold;
	$attr->{'threshold_down'} = $downThreshold;
	$attr->{'sampling_rate'} = $samplingRate;
	$attr->{'freq_step'} = $freqStep;
	$attr->{'description'} = $desc;

	my ($rc, $param) = $db->insertEnergyScheme($name, $attr);

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}
# Get all information for schemes from thc_energy_scheme and thcr_escheme_machine.
# Input: none or
#		owner or
#		owner name
# Return: a hash ref - ok, undef - no record or fail
sub getEnergySchemeAllInfo {
	my ($class, $owner, $name) = @_;
	my $query = {};
	my $escheme_machine;

	if ($owner) {
		$query->{'owner'} = $owner;
	}

	if ($name) {
		$query->{'name'} = $name;
	}

	my $db = EnergyDAO->new();

	my ($rc, $scheme_hash) = $db->queryEnergyScheme($query);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	# Check if the scheme is applied to any machine.
	($rc, $escheme_machine) = $db->queryEScheme();

	if ($rc == 100) {
		# The scheme is not applied to any machine, return the scheme information dirctly.
		return $scheme_hash;
	}
	if ($rc != 0) {
		return;
	}

	# The scheme is applied to some machines, set the appliedTo field.
	foreach my $id (keys %$escheme_machine) {
		my $schemeName = $escheme_machine->{$id}->{'escheme_machine.energy_scheme'};

		foreach my $sch_id (keys %$scheme_hash) {
			if ($scheme_hash->{$sch_id}->{'energy_scheme.name'} eq $schemeName) {
				if (defined($scheme_hash->{$sch_id}->{'appliedTo'})) {
					$scheme_hash->{$sch_id}->{'appliedTo'} = $scheme_hash->{$sch_id}->{'appliedTo'} . "," . $escheme_machine->{$id}->{'escheme_machine.machine_name'};
				} else {
					$scheme_hash->{$sch_id}->{'appliedTo'} = $escheme_machine->{$id}->{'escheme_machine.machine_name'};
				}
			}
		}
	}

	return $scheme_hash;
}


# Get record(s) from thc_energy_scheme.
# Input: none or
#		owner or
#		owner name
# Return: a hash ref - success, undef - no record or fail
sub getEnergyScheme {
	my ($class, $owner, $name) = @_;
	my $condition = {};

	if ($owner) {
		$condition->{'owner'} = $owner;
	}

	if ($name) {
		$condition->{'name'} = $name;
	}

	if (!scalar(%$condition)) {
		return;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyScheme($condition);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	return $param;
}

# Get the record from thc_energy_scheme by id.
# Input: id 
# Return: a hash ref - success, undef - fail
sub getEnergySchemeById {
	my ($class, $id) = @_;
	my $condition = {};
	my $tmp;

	if (!$id) { return; }

	if ($id) {
		$condition->{'id'} = $id;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyScheme($condition);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	foreach my $i (keys %$param) {
		$tmp = $i;
		last;
	}

	return $param->{$tmp};
}

# Update the specified record in thc_energy_scheme.
# Input: owner name opts
# Return: 1 - sucess, undef - fail
sub updateEnergyScheme {
	my ($class, $owner, $name, $opts) = @_;
	my $attr = {};
	my $query = {};

	if (!$owner || !$name || !$opts) { return; }

	if (exists($opts->{'type'})) { $attr->{'cat'} = $opts->{'type'}; }
	if (exists($opts->{'cpu-freq-fixed'})) { $attr->{'cpu_freq_fixed'} = $opts->{'cpu-freq-fixed'}; }
	if (exists($opts->{'cpu-freq-min'})) { $attr->{'cpu_freq_min'} = $opts->{'cpu-freq-min'}; }
	if (exists($opts->{'cpu-freq-max'})) { $attr->{'cpu_freq_max'} = $opts->{'cpu-freq-max'}; }
	if (exists($opts->{'threshold-up'})) { $attr->{'threshold_up'} = $opts->{'threshold-up'}; }
	if (exists($opts->{'threshold-down'})) { $attr->{'threshold_down'} = $opts->{'threshold-down'}; }
	if (exists($opts->{'sampling-rate'})) { $attr->{'sampling_rate'} = $opts->{'sampling-rate'}; }
	if (exists($opts->{'freq-step'})) { $attr->{'freq_step'} = $opts->{'freq-step'}; }
	if (exists($opts->{'desc'})) { $attr->{'description'} = $opts->{'desc'}; }

	if (!scalar(%$attr)) {
		return;
	}

	if ($owner) {
		$query->{'owner'} = $owner;
	}

	if ($name) {
		$query->{'name'} = $name;
	}

	my $db = EnergyDAO->new();
	my ($rc, $param) = $db->modifyEnergyScheme($query,$attr);

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}

# Remove record(s) from thc_energy_scheme.
# Input: owner [name]
# Return: 1 - success, undef - fail
sub rmEnergyScheme {
	my ($class, $owner, $name) = @_;
	my $energy_item;
	my $db;
	my ($rc, $param);

	if (!$owner && !$name) {
		return;
	}

	if ($name) {
		$db = EnergyDAO->new();
		($rc, $param) = $db->deleteEnergySchemeByName($name);

		if ($rc == 0) {
			$db->commit();
			return (1);
		} else {
			$db->rollback();
			return;
		}
	} else {
		my ($tmp_rc, $tmp_param);
		$tmp_rc = -1;
		$db = EnergyDAO->new();

		# Get all schemes of the specified owner.
		($rc, $param) = $db->queryEnergyScheme({'owner'=>$owner});
		if($rc == 0) {
			foreach $energy_item (keys %$param) {
				($tmp_rc, $tmp_param) = $db->deleteEnergySchemeByName($param->{$energy_item}->{'energy_scheme.name'});
				if($tmp_rc != 0) {
					$db->rollback();
					return;
				}
			}

			if($tmp_rc == 0) {
				$db->commit();
			}
		} else {
			return;
		}
	}

	return 1;
}

# Add a new record into thcr_escheme_machine.
# Input: machine schemeName
# Return: 1 - sucess, undef - fail
sub addESchemeMachine {
	my ($class, $machine, $schemeName) = @_;

	if (!$machine || !$schemeName) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->insertESchemeMachine({'machine_name'=>$machine, 'energy_scheme'=>$schemeName});

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	} 

	return 1;
}

# Get the energy scheme ID of the specified machine
# Input: machinename flag
# Return: energy_scheme_id - sucess, undef - fail
# Note: flag == 1 will return name instead of id
sub getESchemeByMachine {
	my ($class, $machine, $flag) = @_;
	my $energy_scheme_id;
	my $scheme_name;

	if (!$machine) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryESchemeByMachine({'machine_name'=>$machine});

	if ($rc == 0) {
		foreach my $i (keys(%$param)) {
			$energy_scheme_id = $i;
			last;
		}
		if ($flag) {
			return $param->{$energy_scheme_id}->{'energy_scheme.name'};
		}
		return $energy_scheme_id;
	} else {
		if ($rc == 100) {
			return;
		}
		return;
	}
}

# Get the machines to which the specified energy scheme is applied.
# Input: owner [name]
# Return: a hash ref - sucess, undef - fail 
sub getMachinesByEScheme {
	my ($class, $owner, $name) = @_;
	my $query;
	my @result;

	if (!$owner) { return; }

	if ($name) {
		$query = {'owner'=>$owner,'name'=>$name};
	} else {
		$query = {'owner'=>$owner};
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryMachinesByEScheme($query);

	if ($rc == 100) {
		# No record
		return;
	}
	if ($rc != 0) {
		return;
	}

	foreach my $id (keys %$param) {
		push(@result, $param->{$id}->{'escheme_machine.machine_name'});
	}

	return \@result;
}

# Update thcr_escheme_machine
# Input: machine schemeName
# Return: 1 - success, undef - fail
# TODO: As designed in DB, (machine_name, is_machine_group) is unique.
# When is_machine_group is implemented, we need to enhance this function
# to accept is_machine_group as the input.
sub updateESchemeMachine {
	my ($class, $machine, $schemeName) = @_;
	if (!$machine || !$schemeName) { return 0; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryESchemeByMachine({'machine_name'=>$machine});

	if ($rc == -1) {
		return;
	} elsif ($rc == 100) {
		# No record for the specified machine is found, insert a new one.
		my ($tmp_rc, $tmp_param) = $db->insertESchemeMachine({'machine_name'=>$machine, 'energy_scheme'=>$schemeName});
		if ($tmp_rc == 0) {
			$db->commit();
			return 1;
		} else {
			$db->rollback();
			return;
		}
	}

	# Modify the existing record.
	($rc, $param) = $db->modifyESchemeMachine({'machine_name'=>$machine}, {'energy_scheme'=>$schemeName});

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}

# Remove record(s) from thcr_escheme_machine by machine(s)
# Input: a list of comma-delimited machines [and a schemeId]
# Return: 1 - success, undef - fail
sub rmESchemeMachineByM {
	my ($class, $machines, $schemeId) = @_;
	my $cond;
	my ($rc, $param);
	$rc = -1;

	if (!$machines) { return; }

	my $db = EnergyDAO->new();

	if ($schemeId) {
		($rc, $param) = $db->querySchemeNameById($schemeId);
		if ($rc == 100) {
			return;
		}
		if ($rc != 0) {
			return;
		}
		$cond = {'energy_scheme'=>$param};
	}

	foreach my $mach (@$machines) {
		$cond->{'machine_name'} = $mach;
		($rc, $param) = $db->deleteESchemeMachine($cond);
		if ($rc != 0) {
			$db->rollback();
			return;
		}
	}

	if ($rc == 0) {
		$db->commit();
	}

	return 1;
}

# Remove record(s) from thcr_escheme_machine by energy scheme(s)
# Input: schemenames
# Return: 1 - success, undef - fail
sub rmESchemeMachineByS {
	my ($class, $schemes) = @_;
	my ($rc, $param);
	$rc = -1;

	if (!$schemes) { return; }

	my @schemeNames = split /,/, $schemes;

	my $db = EnergyDAO->new();

	foreach my $schemeName (@schemeNames) {
		($rc, $param) = $db->deleteESchemeMachine({'energy_scheme'=>$schemeName});
		if ($rc != 0) {
			$db->rollback();
			return;
		}
	}

	if ($rc == 0) {
		$db->commit();
	}

	return 1;
}


# Get the available cpu frequencies
# Return: fail -- 0
#		 succ -- A string including blank-delimited freqencies
sub getAvailCPUFreq {
	chomp(my $freqs = `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies 2>/dev/null`);
	my $ret = `echo $?`;
	if ($ret != 0) {
		# log: cannot read file /sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies
		return 0;
	}
	return $freqs;
}

=comments==============================================
Function Define: $freq = mapCpuFreq($freq);
Description: Map the input cpu freq.
Input:
	$freq: A string or an integer
Return:
	$freq: A mapped integer
=cut
sub mapCpuFreq {
	my ($class, $freq) = @_;
	if ($freq eq "MIN") {
		$freq = $GeneralDefine::MIN_AVAIL_FREQ;
	} elsif ($freq eq "MAX") {
		$freq = $GeneralDefine::MAX_AVAIL_FREQ;
	}
	return $freq;
}

# Verify if the specified CPU frequency is valid
# Return: 0 - invalid, 1 - valid
sub verifyCPUFreq {
	my ($class, $freq) = @_;
	if (!$freq) { return 0; }

	if ($freq == $GeneralDefine::MIN_AVAIL_FREQ || $freq == $GeneralDefine::MAX_AVAIL_FREQ) {
		return 1;
	}

	my $availFreqs = EnergyUtils->getAvailCPUFreq();
	if (!$availFreqs) {
		# log the error
		return 0;
	}
	my @freqs = split / /, $availFreqs;
	foreach my $f (@freqs) {
		if ($f eq $freq) {
			return 1;
		}
	}
	return 0;
}

# Add a record into thc_energy_policy.
# Return: 1 - success, undef - fail
sub addEnergyPolicy {
	my ($class, $name, $mach_status, $prov_mode, $mach_group, $power_status, $energy_scheme, $desc, $flag) = @_;

	if (!$mach_status || !$prov_mode || !$mach_group || !$power_status) {
		# log the error: mach_status, prov_mode, mach_group and power_status have to be specified
		return;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->insertEnergyPolicy($name, {'mach_status'=>$mach_status, 'prov_mode'=>$prov_mode, 'mach_group'=>$mach_group, 'power_status'=>$power_status, 'energy_scheme'=>$energy_scheme, 'flag'=>$flag, 'description'=>$desc});

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}

# Get all energy policies.
# Input: id
# Return: a hash ref - sucess, undef - fail
sub getEnergyPolicybyName {
	my ($class, $name) = @_;
	my $tmp = {};

	if ($name) {
		$tmp->{'name'} = $name;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyPolicy($tmp);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	return $param;
}
sub getEnergyPolicy {
		my ($class, $id) = @_;
		my $tmp = {};

		if ($id) {
			$tmp->{'id'} = $id;
		}

		my $db = EnergyDAO->new();

		my ($rc, $param) = $db->queryEnergyPolicy($tmp);

		if ($rc == 100) {
		# No record.
		 return;
		}
		if ($rc != 0) {
		   return;
		}

	   return $param;
}
# Get policies that using the specified scheme
# Input: energySchemeName
# Return: an arrany ref - success, undef - fail
sub getEnergyPolicyByScheme {
	my ($class, $sname) = @_;
	my @PolicyIds;

	if (!$sname) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyPolicy({'energy_scheme'=>$sname});

	if ($rc == 100) {
		# No record
		return;
	}
	if ($rc != 0) {
		return;
	}

	foreach my $id (keys %$param) {
		push(@PolicyIds, "$id");
	}

	return \@PolicyIds;
}

# Update the specified policy 
# Input: id opts
# 1 - success, undef - fail
sub updateEnergyPolicy {
	my ($class, $id, $opts) = @_;
	my $attr = {};
	my $query = {};

	if (!$id || !$opts) {
		# log the error: id and opts are required
		return;
	}

	if (exists($opts->{'name'})) { $attr->{'name'} = $opts->{'name'}; }
	if (exists($opts->{'machstatus'})) { $attr->{'mach_status'} = $opts->{'machstatus'}; }
	if (exists($opts->{'provmode'})) { $attr->{'prov_mode'} = $opts->{'provmode'}; }
	if (exists($opts->{'machgroup'})) { $attr->{'mach_group'} = $opts->{'machgroup'}; }
	if (exists($opts->{'energyscheme'})) { $attr->{'energy_scheme'} = $opts->{'energyscheme'}; }
	if (exists($opts->{'desc'})) { $attr->{'description'} = $opts->{'desc'}; }

	if (!scalar(%$attr)) {
		return;
	}

	if ($id) {
		$query->{'id'} = $id;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->modifyEnergyPolicy($query,$attr);

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}

# Remove a record from thc_energy_policy.
# Input: id
# Return: 1 - success, undef - fail 
sub rmEnergyPolicy {
	my ($class, $id) = @_;
	if (!$id) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->deleteEnergyPolicy({'id'=>$id});

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}


# Add a record into thc_energy_usage.
# startEnergy, preEnergy, curEnergy and updateInterval are new introduced for iDataPlex.
# rvitals nodelist wattage does not work well for all nodes, we'll calculate power from preEnergy and curEnergy
# Return: 1 - success, undef - fail
sub addEnergyUsage {
	my ($class, $machineName, $provName, $avgPower, $samplingTimes, $startTime, $accPower, $startEnergy, $preEnergy, $curEnergy, $updateInterval) = @_;

	# machineName and provName are required
	if (!$machineName || !$provName) { return; }

	if (!$samplingTimes) {
		$samplingTimes = 1;
	}
	if (!$startTime) {
		$startTime = time();
	}
	if (!$accPower) {
		$accPower = $avgPower;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->insertEnergyUsage({'machine_name'=>$machineName, 'request_name'=>$provName, 'avg_power'=>$avgPower, 'acc_power'=>$accPower, 'sampling_times'=>$samplingTimes, 'start_time'=>$startTime, 'start_energy'=>$startEnergy, 'pre_energy'=>$preEnergy, 'cur_energy'=>$curEnergy, 'update_interval'=>$updateInterval});

	if ($rc == 0) {
		$db->commit();
		EnergyUtils->createRRDEntry($machineName, ($startTime - 120));
	} else {
		$db->rollback();
		return;
	}

	return 1;
}


# Remove a record from thc_energy_usage.
# Input: machineName provName
# Return: 1 - success, undef - fail
sub rmEnergyUsageByMachProv {
	my ($class, $machineName, $provName) = @_;
	if (!$machineName || !$provName) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->deleteEnergyUsage({'machine_name'=>$machineName, 'request_name'=>$provName});

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}

sub getMachMgType{
	my ($class, $machineName) = @_;
	my $user = getpwuid($<);
	my($rt, @out) = CmdUtils->runxcatcmd("gettab node=$machineName nodehm.mgt", $user);
	if(!$rt){
		if(scalar(@out)>0){
		   chomp($out[0]);
		   if($out[0] ne ""){
				return $out[0];
		   }
		}
	}
#try to find the group information
	($rt, @out) = CmdUtils->runxcatcmd("gettab node=$machineName nodelist.groups", $user);
	if(($rt)||(scalar(@out))==0){#failed implement the command or no group information
		return undef; 
	}else{
		chomp($out[0]); 
		my @groups = split(",", $out[0]);
		if(scalar(@groups)==0){
			return undef;
		}
		foreach my $group (@groups){
			$rt = EnergyUtils->getMachMgType($group);
			if(defined($rt) and ($rt ne "")){
				return $rt;
			}
		}
	}
	return undef;
}
# Get the energy of specified ($machineName, $provId) from thc_energy_usage.
# Return: the energy in kWh
sub getEnergyOfMachProv {
	my ($class, $machineName, $provId) = @_;

	if (!$machineName || !$provId) {
		return (0);
	}

	# Intercept the mgt type of the provisioned nodes.
	my $mgt = EnergyUtils->getMachMgType($machineName);
	if (!defined($mgt)) {
	   Logger->get()->warning('EnergyUtils', "$provId", "getEnergyOfMachProv: Failed to get the mgt type for node $machineName");
	   return (0);
	} else {
		SWITCH: {
		$mgt eq "blade" && do {
			return (EnergyUtils->getEnergyOfMachProv_blade($machineName, $provId));
		};
		$mgt eq "ipmi" && do {
			return (EnergyUtils->getEnergyOfMachProv_iDPlx($machineName, $provId));
		};
		DEFAULT:	Logger->get()->warning('EnergyUtils', "$provId", "getEnergyOfMachProv: The mgt is $mgt, no matched mgt for node $machineName") and return (0);
		}
	}
}


# Get the energy of the specified (provisionId, machineNanme) from thc_provision_machine_history.
# Return: a hash ref - success, undef - fail
sub getEnergyFromProvMachLogByPM {
	my ($class, $provisionName, $machName) = @_;
	my $query = {};
	my $result = {};

	if (!$provisionName) { return; }

	if ($provisionName) {
		$query->{'request_name'} = $provisionName;
	}

	if ($machName) {
		$query->{'machine_name'} = $machName;
	}

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyFromProvMachLog($query);

	if ($rc == 100) {
		return;
	}
	if ($rc != 0) {
		return;
	}
	return $param;
}

# Get the energy of the specified provision.
# Return: the energy in kWh
sub getEnergyOfProv {
	my ($class, $provisionId) = @_;
	if (!$provisionId) { return 0; }
		# If provision status is "Expired", just get the data from thc_request.
		# If provision status is "Ready", get data from thc_provision_machine_history and thc_energy_usage
		# - Search thc_provision_machine_history to get the energy values and accumulate.
		#   Say the accumulated value is $sum_a;
		# - Search thc_energy_usage by machine.
		# - sum_a + sum_b
		# Otherwise, return 0.

	my $prov = CloudUtils->getProvInfoById($provisionId, 1);

	# The request does not exist or it is a VMs request.
	if (!defined($prov) || $prov->{'request.hypv_flag'} == -1) {
		return 0;
	}

	if (not defined $prov->{'request.status'} or $prov->{'request.status'} eq "Expired" || $prov->{'request.status'} eq "Canceled") {
		return $prov->{'request.energy'};
	} elsif ($prov->{'request.status'} eq "Ready") {
		my $machEnLog = EnergyUtils->getEnergyFromProvMachLogByPM($provisionId);
		my $sum_a = 0; my $sum_b = 0;

		foreach my $mach (keys %$machEnLog) {
			if (defined($machEnLog->{$mach}->{'provision_machine_history.energy'}) && $machEnLog->{$mach}->{'provision_machine_history.energy'} > 0) {
				$sum_a += $machEnLog->{$mach}->{'provision_machine_history.energy'};
			}
		}
		foreach my $mach (keys %{$prov->{'machList'}}) {
			$sum_b += EnergyUtils->getEnergyOfMachProv($mach, $provisionId);
		}

		my $en = $sum_a + $sum_b;
		my $result;
		if ($en > 10000 || $en < 0.01) {
			$result = sprintf("%.2e", $en);
		} else {
			$result = sprintf("%.2f", $en);
		}
		return $result;
	} else {
		return 0;
	}
}


# create RRD table
sub createRRDEntry {
	my ($class, $name, $time) = @_;

	my $rrdPath = "/hpcc/var/rrd";
	$name =~ s/:/_/g;
	$rrdPath .= "/${name}.rrd";

	open(RRDP, '>'.${rrdPath}) or die "Error: failed to create file $${rrdPath}\n$!";
	close(RRDP);

	system("/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root 'rm -f ${rrdPath}'");
	my $cmd = "rrdtool create ${rrdPath} -b ${time} -s 120 DS:energy:GAUGE:240:0:U RRA:LAST:0.5:1:720 RRA:LAST:0.5:30:96 RRA:LAST:0.5:720:356";
	print("$cmd\n");
	system($cmd);
	system("/opt/ibm/hpccloud/cloudbin/utils/thcsu -u root 'chmod 666 ${rrdPath}'");
}


# update RRD table
sub updateRRDEntry {
	my ($class, $name, $power) = @_;
	my $rrdPath = "/hpcc/var/rrd";
	$name =~ s/:/_/g;
	$rrdPath .= "/${name}.rrd";

	unless (-f $rrdPath) {
		my $now = time();
		EnergyUtils->createRRDEntry($name, ($now-120));
	}

	my $energy = $power * 1000;
	my $cmd = sprintf("rrdtool update %s N:%d", $rrdPath, $energy);
	print("$cmd\n");
	return system($cmd);
}



################ three APIs for iDataPlex ################

# This API is for iDataPlex
# Query thc_energy_usage by machineName and provisionName.
# Return the array ref of ($power, $times, $startTime, $startEnergy, $preEnergy, $curEnergy, $updateInterval)
# The return data will be used to calculating the power and energy consumed by the machine.
sub getEnergyUsageByMachProv_iDPlx {
	my ($class, $machineName, $provName) = @_;
	if (!$machineName || !$provName) { return; }

	my $query = {};
	my @energyUsage;

	$query->{'machine_name'} = $machineName;
	$query->{'request_name'} = $provName;

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryEnergyUsage($query);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	foreach my $id (keys %$param) {
		push(@energyUsage,$param->{$id}->{'energy_usage.avg_power'});
		push(@energyUsage,$param->{$id}->{'energy_usage.sampling_times'});
		push(@energyUsage,$param->{$id}->{'energy_usage.start_time'});
		push(@energyUsage,$param->{$id}->{'energy_usage.start_energy'});
		push(@energyUsage,$param->{$id}->{'energy_usage.pre_energy'});
		if (!($param->{$id}->{'energy_usage.cur_energy'})) {
			return;
		}
		push(@energyUsage,$param->{$id}->{'energy_usage.cur_energy'});
		push(@energyUsage,$param->{$id}->{'energy_usage.update_interval'});
	}

	return \@energyUsage;
}


# Update a record in thc_energy_usage.
# startTime is never updated
# accPower is ignored on iDataPlex
# input: machineName, provId, curEnergy, updateInterval
sub updateEnergyUsageByMachProv_iDPlx {
	my $rv = 0;
	my $pre_record_ref;
	my @pre_record;
	my $new_times = 0;
	my $new_pre_energy = 0;
	my $new_cur_energy = 0;
	my $new_power = 0;
	my ($rc, $param);
	my $query = {};
	my $attr = {};

	my ($class, $machineName, $provId, $curEnergy, $updateInterval) = @_;
	if (!$machineName || !$provId || !$updateInterval) { return; }
	# print "in updateEnergyUsageByMachProv_iDPlx, curEnergy is $curEnergy, updateInterval is $updateInterval\n";

	my $prov = CloudUtils->getProvInfoById($provId, 1);
	if ($prov->{'request.status'} ne "Ready") {
		print "Provision $provId is NOT in Ready state, need not to update energy usage info into thc_energy_usage.\n";
		return;
	}

	my $db = EnergyDAO->new();

	$pre_record_ref = EnergyUtils->getEnergyUsageByMachProv_iDPlx($machineName, $provId);

	# if not found, invoke addEnergyUsage to add a record
	if (!defined($pre_record_ref)) {
		my $startTime = time();
		# Set power and accPower to 0
		# Set $startEnergy = $preEnergy = $curEnergy initially
		$rv = EnergyUtils->addEnergyUsage($machineName, $provId, 0, 1, $startTime, 0, $curEnergy, $curEnergy, $curEnergy, $updateInterval);
		# print "addEnergyUsage returns $rv\n";
		return $rv;
	}

	@pre_record = @$pre_record_ref;
	# @pre_record is ($power, $times, $startTime, $startEnergy, $preEnergy, $curEnergy, $updateInterval)

	$new_times = $pre_record[1] + 1;;
	$new_pre_energy = $pre_record[5]; # the old curEnergy is now the new preEnergy
	if ($curEnergy > 0) {
		# If the input curEnergy >0, set it as the new curEnergy. Then calculate the average power.
		# energy is in kWh, updateInterval is in seconds, power is in watt
		$new_cur_energy = $curEnergy;
		$new_power = ($new_cur_energy - $new_pre_energy) * 1000 /($updateInterval/3600);
	} else {
		# Else, set new power the same as the previous one. Then estimate the new_cur_energy based on power.
		$new_power = $pre_record[0];
		$new_cur_energy = $new_pre_energy + ($new_power * $updateInterval)/3600000;
	}

	# once the record is created, the interval cannot be updated
	print "new_power is $new_power, new_times is $new_times, new_pre_energy is $new_pre_energy, new_cur_energy is $new_cur_energy\n";

	$query->{'machine_name'} = $machineName;
	$query->{'request_name'} = $provId;

	$attr->{'avg_power'} = $new_power;
	$attr->{'sampling_times'} = $new_times;
	$attr->{'pre_energy'} = $new_pre_energy;
	$attr->{'cur_energy'} = $new_cur_energy;

	($rc, $param) = $db->modifyEnergyUsage($query,$attr);

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}


# Get the energy of specified ($machineName, $provId) from thc_energy_usage.
# Return: the energy in kWh
sub getEnergyOfMachProv_iDPlx {
	my ($class, $machineName, $provId) = @_;
	if (!$machineName || !$provId) { return 0; }

	my $res = EnergyUtils->getEnergyUsageByMachProv_iDPlx($machineName, $provId);
	if (!defined($res)) {
		# no matched record returned.
		return 0;
	}
	my ($power, $times, $startTime, $startEnergy, $preEnergy, $curEnergy, $updateInterval) = @$res;

	if ($startEnergy <= 0 || $curEnergy <= 0) {
		# internal error
		return 0;
	}

	my $en = $curEnergy - $startEnergy;
	my $result;
	if ($en > 10000 || $en < 0.01) {
		$result = sprintf("%.2e", $en);
	} else {
		$result = sprintf("%.2f", $en);
	}
	return $result;
}
################ END three APIs for iDataPlex ################


################ START three APIs for blade ##################

# This API is for blade only.
# Query thc_energy_usage by machineName and provisionId.
# Return the array ref of ($avgpower, $times, $startTime, $accPower)
# The return data will be used to calculating the energy consumed by the machine.
sub getEnergyUsageByMachProv_blade {
	my ($class, $machineName, $provId) = @_;
	if (!$machineName || !$provId) { return; }

	my $query = {};
	my @energyUsage;
	my $param;

	my $db = EnergyDAO->new();

	$query->{'machine_name'} = $machineName;
	$query->{'request_name'} = $provId;
	my $rc;
	($rc, $param) = $db->queryEnergyUsage($query);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	foreach my $id (keys %$param) {
		if (!($param->{$id}->{'energy_usage.avg_power'}) && !($param->{$id}->{'energy_usage.sampling_times'})) {
			return;
		}

		push(@energyUsage,$param->{$id}->{'energy_usage.avg_power'});
		push(@energyUsage,$param->{$id}->{'energy_usage.sampling_times'});
		push(@energyUsage,$param->{$id}->{'energy_usage.start_time'});
		push(@energyUsage,$param->{$id}->{'energy_usage.acc_power'});
	}

	return \@energyUsage;
}


# Update a record in thc_energy_usage.
# Input: machineName, provId, avgpower, updateInterval
# startTime is never updated
sub updateEnergyUsageByMachProv_blade {
	my $rv = 0;
	my $pre_record_ref;
	my @pre_record;
	my $new_times = 0;
	my $new_acc_power = 0;

	my ($class, $machineName, $provId, $avgpower, $updateInterval) = @_;
	if (!$machineName || !$provId) { return; }

	$pre_record_ref = EnergyUtils->getEnergyUsageByMachProv_blade($machineName, $provId);

	# if not found, invoke addEnergyUsage to add a record
	if (!defined($pre_record_ref)) {
		my $startTime = time();
		$rv = EnergyUtils->addEnergyUsage($machineName, $provId, $avgpower, 1, $startTime, 0, 0, 0, 0, $updateInterval);
		return $rv;
	}

	@pre_record = @$pre_record_ref;
	# @pre_record is ($power, $times, $startTime, $accPower)
	# times++ and accumulate the accPower
	$new_times = $pre_record[1] + 1;
	$new_acc_power = $pre_record[3] + $avgpower;

	my $query = {};
	my $attr = {};
	my $param;
	my $db = EnergyDAO->new();

	$query->{'machine_name'} = $machineName;
	$query->{'request_name'} = $provId;

	$attr->{'avg_power'} = $avgpower;
	$attr->{'sampling_times'} = $new_times;
	$attr->{'acc_power'} = $new_acc_power;
	my $rc;
	($rc, $param) = $db->modifyEnergyUsage($query,$attr);

	if ($rc == 0) {
		$db->commit();
	} else {
		$db->rollback();
		return;
	}

	return 1;
}


# Get the energy of specified ($machineName, $provId) from thc_energy_usage.
# Return the energy in kWh
sub getEnergyOfMachProv_blade {
	my ($class, $machineName, $provId) = @_;
	if (!$machineName || !$provId) { return 0; }

	my $res = EnergyUtils->getEnergyUsageByMachProv_blade($machineName, $provId);
	if (!defined($res)) {
		# no matched record returned.
		return 0;
	}
	my ($power, $times, $startTime, $accPower) = @$res;
	my $dur = time - $startTime;

	if (!$times) {
		# internal error, times is 0.
		return 0;
	}

	# convert it to kWh
	my $en = ($accPower / $times * $dur) / 3600000;
	my $result;
	if ($en > 10000 || $en < 0.01) {
		$result = sprintf("%.2e", $en);
	} else {
		$result = sprintf("%.2f", $en);
	}
	return $result;
}
################ END  three APIs for blade ###################

# Get scheme owner and scheme name by machine status and provision mode.
sub getESchemeByMachAndPM {
	my ($class, $machstatus, $provmode) = @_;

	if(!$machstatus || !$provmode) { return; }

	my $db = EnergyDAO->new();

	my ($rc, $param) = $db->queryESchemeByMachStatAndProMod($machstatus,$provmode);

	if ($rc == 100) {
		# No record.
		return;
	}
	if ($rc != 0) {
		return;
	}

	return $param;
}

1;
