#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/common/LoadL.pm, hpc.cloud.comm, hpcc_rbei, rbeis002a 1.11 1/17/12 05:53:32";

package LoadL;

use CloudUtils;
use strict;
use warnings;

%::JobStatusL2S = ( "Starting" => "ST", "Running" => "R", "Checkpointing" => "CK", "Complete" => "C",
                  "Complete Pending" => "CP", "Pending" => "P", "User Hold" => "H", "System Hold" => "S",
                  "System User Hold" => "HS", "Not Queued" => "NQ", "Deferred" => "D", "Preempted" => "E",
                  "Preempted Pending" => "EP", "Resume Pending" => "MP", "Idle" => "I", "Canceled" => "CA",
                  "Vacated" => "V", "Rejected" => "X", "Vacate Pending" => "VP", "Reject Pending" => "XP",
                  "Remove Pending" => "RP", "Removed" => "RM", "Not Run" => "NR", "Terminated" => "TX",
                  "Submission Error" => "SX" );

=comment
Description:
	Get job information. ($rc, $jobs) = getJobs($requestid)
input:
	$requestid: the request id, it is optional
return:
	$rc: $GeneralDefine::FAILED or $GeneralDefine::SUCCESS
	$jobs: A hash of jobs if SUCCESS; reason message if FAILED
=cut
sub getJobs {
	my ($class, $requestid, $uid) = @_;
	if (!defined($uid)) {
		$uid= (getpwuid($<))[0];
	}
	my $isAdmin = CloudUtils->isAdmin($uid);
	my $requests = {};
	my %alljobs;
	my %res2req;

	my $fuzzy = $GeneralDefine::QUERY_NAME_ONLY; # Do not query ***a or ***d
	my $flag = 1; # get machines in hash
	if (defined($requestid)) {
		my $request = CloudUtils->getProvInfoById($requestid, $flag, $fuzzy);
		if (!$request) {
			return ($GeneralDefine::FAILED, "Failed to find request $requestid.");
		}
		my $user = CloudUtils->getUserByName($uid);
		if (!$user || CloudUtils->checkProvAuth($request, $user->{$uid}) == 0) {
			return ($GeneralDefine::FAILED, "User $uid is not authorized on request $requestid.");
		}
		$requests->{$requestid} = $request;
	} else {
		if ($isAdmin) {
			my $status = "Reserved,Setup,Verifying,Waiting,Ready";
			$requests = CloudUtils->getProvAndEnergyInfoByStatus($status, $fuzzy, $flag);
		} else {
			$requests = CloudUtils->getProvByUser($uid, $fuzzy, $flag);
		}
		if (!$requests || !scalar(keys %$requests)) {
			goto JOINANDPUBLIC;
		}
	}
	
	my $query="/usr/bin/llq -l";
	my $connectcmd="/usr/bin/ssh";
	my $rv = 0;
	my @outs = ();
	while (my ($reqid, $req) = each(%$requests)) {
		if (($req->{'request.cluster'} eq 'Sub') && ($req->{'request.status'} eq 'Ready')) {
   			# query mn_node which belong to this request.
			my $nodeHash = $req->{'machList'};
			my @nodes = keys %$nodeHash;
			if (!scalar(@nodes)) {
				Logger->get()->error('Request', $reqid, "The machList of the request is empty.");
				return ($GeneralDefine::FAILED, "Internal error.");
			}
			($rv, @outs) = CmdUtils->runcmd("$connectcmd $nodes[0] $query", 'cloudadm');

			if ($rv == 0 && scalar(@outs) > 0 && $outs[0] !~ /no job/) {
				%alljobs = (%alljobs, parse_job(\@outs, $reqid, $uid));
			}
		}
		else { # Join
			# save resid => reqid for later.
			$res2req{$req->{'request.ll_resid'}} = $reqid;
		}
	}

JOINANDPUBLIC:
	($rv, @outs) = CmdUtils->runcmd("llq -l");
	if ($rv == 0 && $outs[0] !~ /no job/) {
		%alljobs = (%alljobs, parse_job(\@outs, \%res2req, $uid));
	}

	if (%alljobs && scalar(keys %alljobs) > 0) {
		return ($GeneralDefine::SUCCESS, \%alljobs);
	}
	return ($GeneralDefine::FAILED, "Failed to get any job information for user $uid.");
}

sub parse_job {
	my ($outs, $poly, $uid) = @_;

	my %alljobs;

	my ($jobstep, $attr, $value);

	my $request_id;
	my $res2req;
	if (!defined($uid)) {
		$uid= (getpwuid($<))[0];
	}
	my $isAdmin = CloudUtils->isAdmin($uid);

	ref($poly) eq 'HASH' ? $res2req = $poly : $request_id = $poly;

	foreach my $line (@$outs) {
		if ($line =~ /^=====\s*Job Step\s+(\S+)\s*=====$/) {
			$jobstep = $1;
		}
		elsif ($jobstep && $line =~ /\s*(\S[^:\+]*\S)\s*:\s+(.*)\s*$/ && $line !~ /\+/) {
			$attr = $1; $value = $2;
			$alljobs{$jobstep}->{$attr} = $value;
		}
		elsif ($jobstep && $line =~ /\s*\+\s*(\S*)/) {
			$alljobs{$jobstep}->{$attr} .= $1;
		}
	}

	foreach $jobstep (keys(%alljobs)) {
		# filter the jobstep for provision.
		if ($alljobs{$jobstep}->{'Job Name'} eq 'CloudSolution') {
			delete($alljobs{$jobstep});
			next;
		}

		# filter jobs not owned by the user
		if (!$isAdmin and $alljobs{$jobstep}->{'Owner'} ne $uid) {
			delete($alljobs{$jobstep});
			next;
		}

		# llq command will remain a white character in the end.
		my $resid = $alljobs{$jobstep}->{'Reservation ID'};
		$resid =~ s/\s*$//;

		# add a new field for request id.
		if ($request_id) {
			# for job submit on subcluster.
			$alljobs{$jobstep}->{'Request ID'} = $request_id;
		}
		elsif (exists($res2req->{$resid})) {
			# for job submit on join machine.
			$alljobs{$jobstep}->{'Request ID'} = $res2req->{$resid};
		}
		else {
			# for public area job.
			$alljobs{$jobstep}->{'Request ID'} = '';
		}
	}

	return %alljobs;
}

sub genJobXml {
    my ($class, $alljobs) = @_;

    my $returnXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    $returnXML .= "<JobGraphicsViewData>";
    $returnXML .= "<jobs>";
    $returnXML .= "<timeDateStamp>" . scalar (localtime) . "</timeDateStamp>";
    foreach my $key (sort keys %$alljobs) {
        my $jobElem = $$alljobs{$key};
        $returnXML .= "<job>";
        $returnXML .= "<jobID>" . $$jobElem{"Job Step Id"} . "</jobID>";
        $returnXML .= "<jobName>" . $$jobElem{"Job Name"} . "</jobName>";
        $returnXML .= "<stepName>" . $$jobElem{"Step Name"} . "</stepName>";
        $returnXML .= "<owner>" . $$jobElem{"Owner"} . "</owner>";
        $returnXML .= "<queueDate>" . $$jobElem{"Queue Date"} . "</queueDate>";
        $returnXML .= "<holdUntilDate>" . $$jobElem{"Hold Job Until"} . "</holdUntilDate>";
        $returnXML .= "<dispatchedDate>" . $$jobElem{"Dispatch Time"} . "</dispatchedDate>";
        $returnXML .= "<state>" . $::JobStatusL2S{$$jobElem{"Status"}} . "</state>";
        $returnXML .= "<priority>" . $$jobElem{"User Priority"} . "</priority>";
        $returnXML .= "<jobClass>" . $$jobElem{"Class"} . "</jobClass>";
        $returnXML .= "<group>" . $$jobElem{"LoadLeveler Group"} . "</group>";
        $returnXML .= "<jobType>" . $$jobElem{"Step Type"} . "</jobType>";
        $returnXML .= "<hostsRunning>";
        if ($$jobElem{"Allocated Host"} ne "") {
            $returnXML .= "<hostRunning>" . $$jobElem{"Allocated Host"} . "</hostRunning>";
        } else {
            my @hosts = split /::/, $$jobElem{"Allocated Hosts"};
            foreach my $host (@hosts) {
                $returnXML .= "<hostRunning>" . $host . "</hostRunning>" if ($host ne "");
            }
        }
        $returnXML .= "</hostsRunning>";
        $returnXML .= "<completedDate>" . $$jobElem{"Completion Date"} . "</completedDate>";
        $returnXML .= "<completionCode>" . $$jobElem{"Completion Code"} . "</completionCode>";
        $returnXML .= "<accountNumber>" . $$jobElem{"Account"} . "</accountNumber>";
        $returnXML .= "<unixGroup>" . $$jobElem{"Unix Group"} . "</unixGroup>";
        $returnXML .= "<imageSize>" . $$jobElem{"Virtual Image Size"} . "</imageSize>";
        $returnXML .= "<reservationID>" . $$jobElem{"Reservation ID"} . "</reservationID>";
        $returnXML .= "<schedulingCluster>" . $$jobElem{"Scheduling Cluster"} . "</schedulingCluster>";
        $returnXML .= "<submittingCluster>" . $$jobElem{"Submitting Cluster"} . "</submittingCluster>";
        $returnXML .= "<sendingCluster>" . $$jobElem{"Sending Cluster"} . "</sendingCluster>";
        $returnXML .= "<requestedCluster>" . $$jobElem{"Requested Cluster"} . "</requestedCluster>";
        $returnXML .= "<submittingUser>" . $$jobElem{"Submitting User"} . "</submittingUser>";
        $returnXML .= "<queueID>" . "</queueID>";
        $returnXML .= "<queueIndex>" . "</queueIndex>";
        $returnXML .= "<isTopDog>" . "</isTopDog>";
        $returnXML .= "<isFavored>" . (($$jobElem{"Favored Job"} eq "No") ? "false":"true") . "</isFavored>";
        $returnXML .= "</job>";
    }
    $returnXML .= "</jobs>";
    $returnXML .= "<reservations>";
    $returnXML .= "<timeDateStamp>" . scalar (localtime) . "</timeDateStamp>";
    $returnXML .= "</reservations>";
    $returnXML .= "</JobGraphicsViewData>";
    return $returnXML;
}

1;

