#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2010,2012 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# sccsid = "@(#) src/hpcc/provision/ProvUtils.pm, hpc.cloud.prov, hpcc_rbei, rbeis002a 1.63 1/31/12 04:25:31";

package ProvUtils;

use strict;
use warnings;

use CmdUtils;
use CloudUtils;
use GeneralDefine;
use Utils::Logger;
#-------------------------------------------------------------------------------
=head1  ProvUtils
=head2    Package Description
  HPCC ProvUtils module. Some commands such as rpower have different implementations  for different hardware. This module holds the usage string for these kind
of commands so that the usage can be referenced from different modules.
=cut
#-------------------------------------------------------------------------------

sub getBilling {
	my ($class, $query, $flag, $changed_info) = @_;
	# not charged for hypervisor for now.
	if((exists$query->{'hypvflag'}) and ($query->{'hypvflag'}==1)){
		return (1,0,0);
	}

	my $userid = (getpwuid($<))[0];
	my $sysconf = CloudUtils->getConfig;
	my $billModuleName = $sysconf->{'billingMod'};
	my $billPackageName = $billModuleName . ".pm";
	my $bill;
	eval {
		require $billPackageName;
		$bill = $billModuleName->new();
	}; if ($@) {
		Logger->get()->warning('ProvUtils', 'NA', "getBilling: Failed connect $billModuleName");
		return 0;

	}
	my $prov;
	my $rtv;
	$prov->{'provisionOwner'} = $userid;
	
	
	if($flag eq 'Create'){
		if(exists($query->{'duration'})){
			$prov->{'duration'} = $query->{'duration'};
		}else{
			$prov->{'duration'} = $query->{'enddate'} - $query->{'startdate'};	
		}
		$prov->{'hypv_flag'} = $query->{'hypvflag'};
		if(exists($query->{'cpus'})){
			$prov->{'request_machine_cpu'} = $query->{'cpus'};
		}
		if (exists($query->{'machlist'})) {
			@{$prov->{'machList'}} = split(",", $query->{'machlist'});
			$prov->{'select_method'} = 0;
		}
		else{
			
				$prov->{'requestMachineType'} = $query->{'machtype'};
				$prov->{'requestMachineNumber'} = $query->{'machnum'};
				$prov->{'select_method'} = 1;
			}
	}elsif($flag eq 'Modify'){#modify but not change machine list
		$prov->{'provisionId'} = $query->{'reqid'};
		my $request = CloudUtils->getProvInfoById($query->{'reqid'}, 1); 
		if(!defined($changed_info)){
			($rtv, $changed_info) = get_changed_info($request, $query);
			if($rtv == $GeneralDefine::FAILED){
				Logger->get()->warning('ProvUtils', 'NA', "getBilling: Invalide modification request.");
				return (-1,0,0);
			}
		}
		if($request->{'request.hypv_flag'} ==1 ){
			return (1,0,0);
		}
		$prov->{'hypv_flag'} = $request->{'request.hypv_flag'};		
		$prov->{'select_method'} = 1;	
		
		if(($prov->{'hypv_flag'} == -1) and exists($request->{'request.machine_cpu'})){
			$prov->{'request_machine_cpu'} = $request->{'request.machine_cpu'};
		}
		if(exists($request->{'request.machine_type'})){
			$prov->{'requestMachineType'} = $request->{'request.machine_type'};
		}else{#currently, we may not get machine type. It need be enhance in future.
			$prov->{'requestMachineType'} = "IBM_HS21_Blade";
		}
		$prov->{'requestMachineNumber'} = $request->{'request.machine_num'};
		if(exists($changed_info->{'add_machines_num'})){
			$prov->{'requestMachineNumber'} += $changed_info->{'add_machines_num'};
		}elsif(exists($changed_info->{'add_machines'})){
			$prov->{'requestMachineNumber'} += scalar(@{$changed_info->{'add_machines'}});
		}
		if(exists($changed_info->{'del_machines'})){
			$prov->{'requestMachineNumber'} -= scalar(@{$changed_info->{'del_machines'}});
		}
		use POSIX 'strftime';
		my $now = time;
		if($now > $request->{'request.start_time'}){
			my $endtime = $request->{'request.expiry_time'};
			if(exists($changed_info->{'request'}) and exists($changed_info->{'request'}->{'expiry_time'})){
				$endtime = $changed_info->{'request'}->{'expiry_time'};
			}
			$prov->{'duration'} = int(($endtime - $now)/60);
			$prov->{'post_duration'} = int(($now - $request->{'request.start_time'})/60)+1;
			$prov->{'post_requestMachineNumber'} = $request->{'request.machine_num'};
#			print "$prov->{'duration'}, $prov->{'post_duration'}, $prov->{'post_requestMachineNumber'}, $prov->{'requestMachineNumber'}\n";
		}else{
			if(exists($changed_info->{'duration'})){
				$prov->{'duration'} = int($changed_info->{'duration'} - $GeneralDefine::JOB_IN_ADVANCE/60);
			}else{
				$prov->{'duration'} = int(($request->{'request.expiry_time'} - $request->{'request.start_time'})/60);
			}
		}
		#change cpu memory disk
 		if(exists($changed_info->{'request'}) and exists($changed_info->{'request'}->{'machine_cpu'})){
				 $prov->{'request_machine_cpu'} = $changed_info->{'request'}->{'machine_cpu'};
		}
	}else{#cancel
		my $request = CloudUtils->getProvInfoById($query->{'reqid'});
		if($request->{'request.hypv_flag'} ==1 ){
			return (1,0,0);
		} 
		$prov->{'provisionId'} = $query->{'reqid'};
		$prov->{'hypv_flag'} = $request->{'request.hypv_flag'};		
		$prov->{'select_method'} = 1;	
		if(($prov->{'hypv_flag'} == -1) and exists($request->{'request.machine_cpu'})){
			$prov->{'request_machine_cpu'} = $request->{'request.machine_cpu'};
		}
		if(exists($request->{'request.machine_type'})){
			$prov->{'requestMachineType'} = $request->{'request.machine_type'};
		}else{#currently, we may not get machine type. It need be enhance in future.
			$prov->{'requestMachineType'} = "IBM_HS21_Blade";
		}
		$prov->{'requestMachineNumber'} = $request->{'request.machine_num'};
		use POSIX 'strftime';
		my $now = time;
		if($now > $request->{'request.start_time'}){
			$prov->{'duration'} = int(($now - $request->{'request.start_time'})/60)+1;
		}else{
			$prov->{'duration'} = 0;
		}
	}
	return $bill->testProvisionRequest($prov, $flag);
}

sub applyBilling {
	my ($class, $pid,$flag, $changed_info) = @_;
	my $sysconf = CloudUtils->getConfig;
	my $billModuleName = $sysconf->{'billingMod'};
	my $billPackageName = $billModuleName . ".pm";
	my $bill;

	eval {
        require $billPackageName;
        $bill = $billModuleName->new();
	}; if ($@) {
		Logger->get()->warning('ProvUtils', "$pid", "applyBilling: Failed connect $billModuleName, request ID: $pid");
		return 0;
	}
	my $prov = CloudUtils->getProvInfoById($pid);
	unless($prov)
	{
		Logger->get()->warning('ProvUtils', "$pid", "Fail to query provison information for the billing, $pid");
		return 0;
	}
	if($prov->{'request.hypv_flag'}!=1){	
		$bill->applyProvisionRequest($prov, $flag, $changed_info);
		return 2;
	}
	return 1;
}

sub createProv {
	my ($class, $query) = @_;

	my $request_info = {
		'owner' => (getpwuid($<))[0],
		'machine_os' => $query->{'osimage'},
		'start_time' => $query->{'startdate'},
		'expiry_time' => $query->{'startdate'} + $query->{'duration'} * 60,
		'cluster' => (defined($query->{'cluster'}) && $query->{'cluster'} eq "yes") ? "Sub" : "Join",
		'hypv_flag' => $query->{'hypvflag'},
		'machine_cpu' => $query->{'cpus'},
		'machine_memory' => $query->{'memory'},
		'machine_disk' => $query->{'disk'},
		'vlan_flag' => $query->{'vlan_flag'},
		'status' => "Reserved",
	};

	my @machines = ();
	if (exists($query->{'machlist'})) {
		@machines = CloudUtils->uniq(split(",", $query->{'machlist'}));
		$request_info->{'machine_num'} = scalar(@machines);
		# indicate reserve machines by name.
		$request_info->{'select_method'} = 0;
	}
	elsif (exists($query->{'machtype'}) && exists($query->{'machnum'}) && $query->{'machnum'} > 0) {
		$request_info->{'machine_type'} = $query->{'machtype'};
		$request_info->{'machine_num'} = $query->{'machnum'};
		# indicate reserve machines by type.
		$request_info->{'select_method'} = 1;
	}
	else{
		CmdUtils->error("Select some machines.", "-1");
	}

	my @users = exists($query->{'users'}) ? CloudUtils->uniq(split(/,/, $query->{'users'})) : ();
	my @groups = exists($query->{'groups'}) ? CloudUtils->uniq(split(/,/, $query->{'groups'})) : ();

	my $now = time;
	if($request_info->{'start_time'} < ($now + 300)) {
		CmdUtils->error("The start date is earlier than or too close to the current time, stime:$request_info->{'start_time'}, ctime:$now.", "-1");
	}

	if ($request_info->{'expiry_time'} - $request_info->{'start_time'} <= 0) {
		CmdUtils->error("The duration is invalid.", "-1");
	}

	my ($enough, $cost) = ProvUtils->getBilling($query, "Create");
	if ($enough==0) {
		CmdUtils->error("Do not have enough balance to pay for this request.", "-1");
	}elsif($enough==-1) {
		CmdUtils->error("Invalid modification.", "-1");
	}

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
	$year += 1900;
	$mon += 1;
	my $timestr = sprintf("%02d.%02d.%4d_%02d:%02d:%02d", $mon, $mday, $year, $hour, $min, $sec);

	my $request_id = $request_info->{'owner'} . '_' . $timestr;

	my $rv = create_reservation($request_id, $request_info, \@machines, \@users, \@groups);
	if ($rv != 0) {
		# delete the db info.
		CmdUtils->error("Failed to reserve the requested resource.", "-1");
	}

	$rv = CloudUtils->newProvReq($request_id, $request_info, \@machines, \@users, \@groups);
	if (!$rv) {
		delete_reservation($request_info);
		CmdUtils->error("Failed to create records in database for this request.", "-1");
	}

	if ($request_info->{'hypv_flag'} != 0) {
		my $used = $request_info->{'hypv_flag'} == 1 ? 'UsedHypvNums' : 'UsedKvmVms';

		my $cfg = CloudUtils->getConfig();
		if (exists($cfg->{$used})) {
			$cfg->{$used} += $request_info->{'machine_num'};
		} else {
			$cfg->{$used} = $request_info->{'machine_num'};
		}

		if (!CloudUtils->putConfig($cfg)) {
			delete_reservation($request_info);
			CloudUtils->removeProvReq($request_id);
			CmdUtils->error("Failed to update the hypervisor/vm information into system configuration table.", "-1");
		}
	}
	$rv = submit_prov($request_id, $request_info);
	if ($rv != 0) {
		delete_reservation($request_info);
		CloudUtils->removeProvReq($request_id);
		# restore the usedxxx
		CmdUtils->error("Failed to submit a job to handle the request.", "-1");
	}

	CloudUtils->trackSubmitProvsionEvent($request_id);

	return($request_id);
}

sub updateProv {
	my ($class, $query) = @_;

	my $request_id = $query->{'reqid'};
	my $userid = getpwuid($<);
	my $errstr="";
	my $rtv=$GeneralDefine::SUCCESS;
	my $request = CloudUtils->getProvInfoById($request_id, 1);
	if (!$request) {
		$errstr = "Failed to get information of request $request_id.";
		$rtv = $GeneralDefine::FAILED;
		return ($rtv, $errstr);
	}

	if ($userid ne $request->{'request.owner'}) {
		$errstr = "You are not authorized to change the request.";
		$rtv = $GeneralDefine::FAILED;
		return ($rtv, $errstr);
	}
	if($request->{'request.status'} eq "Canceling" or 
			$request->{'request.status'} eq "Canceled" or
			$request->{'request.status'} eq "Expired" or
			$request->{'request.status'} eq "Failed"){
		$errstr = "Can not modify request $request->{'request.name'} in $request->{'request.status'} status.";
		$rtv = $GeneralDefine::FAILED;
		return ($rtv, $errstr);
	}

	# Special check, may be removed in future
	if(exists($request->{'request.hypv_flag'})){
		if($request->{'request.hypv_flag'} == 1){#hypv
			if(exists($query->{'enddate'}) or exists($query->{'duration'})){
				$errstr = "Can not modify enddate or duration of a Hypervisor request.";
				$rtv = $GeneralDefine::FAILED;
				return ($rtv, $errstr);
			}
		}elsif($request->{'request.hypv_flag'} == -1){#VM
=temp support
			if(exists($query->{'addmach'}) or exists($query->{'rmmach'}) 
				   or exists($query->{'machlist'}) or exists($query->{'addmachnum'})){
				$errstr="Can not use hcchreq to change VM request.";
				$rtv = $GeneralDefine::FAILED;
				return ($rtv, $errstr);
			}
=cut
			if($request->{'request.status'} ne 'Reserved'){
				if(exists($query->{'cpus'}) or exists($query->{'memory'}) or exists($query->{'disk'})){
					$errstr = "Can not modify cpu number, memory size or disk size while the request is in $request->{'request.status'} status.";
					$rtv = $GeneralDefine::FAILED;
					return ($rtv, $errstr);
				}
			}
			if(exists($query->{'vmcount'}) and ($query->{'vmcount'} != $request->{'request.machine_num'})){
				if($query->{'vmcount'} > $request->{'request.machine_num'}){
					$query->{'addmachnum'} = $query->{'vmcount'} - $request->{'request.machine_num'};
				}else{
					my $rmnum = $request->{'request.machine_num'} - $query->{'vmcount'};
					my @machines = sort(keys(%{$request->{'machList'}}));
					my @rmmachs=();
					my $machNum=scalar(@machines);
					for( my $idx=1; $idx<=$rmnum; $idx++){
						push(@rmmachs, $machines[$machNum - $idx]);
					}
					$query->{'rmmach'} = join(',', @rmmachs);
				}#end else
			}#end if(exists($query->{'vmcount'}) and ....
		}#end elsif($request->{'request.hypv_flag'} == -1)
	}	
#End of speical check
	my $status = $GeneralDefine::MODIFY_NO;
   	if(exists($request->{'request.modify'})){
		$status = $request->{'request.modify'};
	}
	if ($status ne $GeneralDefine::MODIFY_NO) {
		$errstr = "A modificaiton is running.";
		$rtv = $GeneralDefine::FAILED;
		return ($rtv, $errstr);
	}else{
		if($request->{'request.status'} ne 'Reserved'){
			$request->{'request.modify'} = $GeneralDefine::MODIFY_NORESERVE;
		}else{
			use POSIX 'strftime';
			my $now = time;
			if (($request->{'request.start_time'} - $now) < 60) {#less than one minutes
				$request->{'request.modify'} = $GeneralDefine::MODIFY_NORESERVE;
			}else{
				$request->{'request.modify'} = $GeneralDefine::MODIFY_RESERVE;
			}
		}
		($rtv,$errstr) = CloudUtils->modifyProvReq($request->{'request.name'},{'request'=>{'modify' => $request->{'request.modify'}}});
		if($rtv ne "1"){
			$rtv = $GeneralDefine::FAILED;
			return ($rtv, $errstr);
		}else{
			$rtv = $GeneralDefine::SUCCESS;
		}
	}

	if(($request->{'request.status'} ne 'Reserved') and (exists($query->{'startdate'}))){
		$errstr = "The startdate of request $request_id can not be changed in $request->{'request.status'} status.";
		$rtv = $GeneralDefine::FAILED;
		goto RTV;
	}
		

	($rtv, $errstr)= changed_request($request, $query);
	if ($rtv ne $GeneralDefine::SUCCESS) {
		goto RTV;
	}
	if($request->{'request.modify'} eq $GeneralDefine::MODIFY_RESERVE){
		($rtv, $errstr) = CloudUtils->modifyProvReq($request->{'request.name'},{'request'=>{'modify' => $GeneralDefine::MODIFY_NO}});
		if($rtv ne "1") {
			#do not goto RTV since useless 
			$rtv = $GeneralDefine::FAILED;
			return($rtv, $errstr);	
		}else{
			$rtv = $GeneralDefine::SUCCESS;
		}
	}	

	# track change provision events
	CloudUtils->trackChangeProvsionEvent($request_id, "An action which modifies provision executed!");
	return $GeneralDefine::SUCCESS;
RTV:
	CloudUtils->modifyProvReq($request->{'request.name'},{'request'=>{'modify' => $GeneralDefine::MODIFY_NO}});
	return($rtv, $errstr);
}

=comment==================
Input:
	$request_info: The request information
	$status_t: The target status, the possible values are Expired, Cancelled, Failed.
	$status_p: The previous status. It is not mandatory. 
=cut
sub terminateProv {
	my ($class, $request_info, $status_t, $status_p) = @_;

	my $hypv_flag = $request_info->{'request.hypv_flag'};
	my $mach_info = $request_info->{'machList'};
	my @nodes = keys(%$mach_info);
	my $modify = $request_info->{'request.modify'};

	# cancel the modified new job to avoid the mess of machine restore
	if (($modify ne $GeneralDefine::MODIFY_NO) and $request_info->{'request.ll_jobid_new'}) { 
		CmdUtils->runloadlcmd("llcancel $request_info->{'request.ll_jobid_new'}");
	}

	my $trait = CloudUtils->machine_trait($mach_info, $request_info, 1);

	# In the canceling case, request.status is set to Canceling before invoking terminateProv.
	# If the previous status is Reserved, we should not shutdown or restore the nodes.
	# There is no "release" if the request is now not using the nodes.
	if (!defined($status_p) || $status_p ne "Reserved") {
		CloudUtils->release_machine(\@nodes, $trait);
	}

	foreach my $node (@nodes) {
		my $power_state;
		my $current_os;

		if ($trait->{'machines'}->{$node}->{'mode'} eq 'share') {
			$power_state = 'on';
			$current_os = $mach_info->{$node}->{'machine.default_os'};
		}
		else {
			# full
			$power_state = 'off';
			$current_os = undef;
		}

		my $attrs = {
			'power_state' => $power_state,
			'current_os' => $current_os,
			'hypv_flag' => $hypv_flag == -1 ? -1 : 0,
			'status' => 'Avail',
		};

		CloudUtils->updateMachByName($node, $attrs);
	}

	my $request_id = $request_info->{'request.name'};

	# terminate a provision in three possible status: Expired, Cancelled, Failed.
	use POSIX 'strftime';
	my $now = time;
	CloudUtils->updateProvDynAttrById($request_id, {'status' => $status_t, 'actual_stop_time' => $now});
	# Rename ***a/d to ***.
	ProvUtils->updateReqMach($request_id, "ren", "a");
	ProvUtils->updateReqMach($request_id, "ren", "d");

	# Update UsedKvmVms or UsedHypvNums
	if ($hypv_flag != 0) {
		my $used = $hypv_flag == 1 ? 'UsedHypvNums' : 'UsedKvmVms';

		my $cfg = CloudUtils->getConfig();
		if (exists($cfg->{$used})) {
			$cfg->{$used} -= $request_info->{'request.machine_num'};
		}

		CloudUtils->putConfig($cfg);
	}

	my ($rv, $reason) = ProvUtils->createReqMachHistory($request_id, \@nodes);
	if ($rv != $GeneralDefine::SUCCESS) {
		Logger->get()->error("ProvUtils", "$request_id", "failed to add a record:[$reason].");
	}

	# Update the energy usage data into the request table.
	if ($hypv_flag != -1) {
		($rv, $reason) = ProvUtils->update_request_energy($request_id);
		if ($rv != $GeneralDefine::SUCCESS) {
			Logger->get()->error("ProvUtils", "$request_id", "failed to collect energy:[$reason].");
		}
	}

	# Clean the request_machine and requst_user tables.
	CloudUtils->cleanProvReq($request_id, \@nodes, [], [], $status_p);

	# rmres after all works done.
	delete_reservation($request_info);

	return ;
}

sub removeProv {
	my ($class, $query) = @_;

	my $request_id = $query->{'reqid'};
	my $userid = (getpwuid($<))[0];

	my $request_info = CloudUtils->getProvInfoById($request_id, 1, 1);
	if (!$request_info) {
		CmdUtils->error("Failed to get information of request $request_id.", "-1");
	}

	if ($userid ne $request_info->{'request.owner'} && !CloudUtils->isAdmin($userid)) {
		CmdUtils->error("You are not authorized to remove request $request_id.", "-1");
	}

	my $status = $request_info->{'request.status'};
	if (($status eq 'Failed') or ($status eq 'Expired') or ($status eq 'Canceled')) {
		CmdUtils->error("The request $request_id has been in $status status.", "-1");
	}

	my $modify = $request_info->{'request.modify'};
	if (($modify eq $GeneralDefine::MODIFY_RESERVE) or
	    ($modify eq $GeneralDefine::MODIFY_NORESERVE && !$request_info->{'request.ll_jobid_new'}))  {
		CmdUtils->error("The request $request_id is being modified. Please cancel it later.", "-1");
	}

	# Get the status before setting Canceling
	my $status_p = $request_info->{'request.status'};
	CloudUtils->modifyProvReq($request_id, {'request'=>{'status' => 'Canceling'}});

	ProvUtils->terminateProv($request_info, "Canceled", $status_p);

	ProvUtils->cancel_prov($request_id, $request_info->{'request.ll_jobid'});
	my ($rtv, $cost) = ProvUtils->getBilling($query, "Cancel");
	if($rtv == 1){
		my $changed->{'cost'} = $cost;
		ProvUtils->applyBilling($request_id, 'Cancel', $changed);	
	}

	CloudUtils->trackCancelProvsionEvent($request_id);

	return ;
}

# Create record(s) in thc_provision_machine_history
sub createReqMachHistory {
	my ($class, $request, $nodelist) = @_;
	my @machs = ();
	if (!$request) {
		return ($GeneralDefine::FAILED, "Request name has to be specified.");
	}

	# Get the request and machine list.
	my $req = CloudUtils->getProvInfoById($request, 1);
	unless (defined($req)) {
		return ($GeneralDefine::FAILED, "Failed to get requst $request from database.");
	}
	my $ml = $req->{'machList'};
	if ($nodelist) {
		@machs = @$nodelist;
	} else {
		@machs = keys(%$ml);
	}
	foreach my $mach (@machs) {
		if (!defined ($ml->{$mach})) {
			Logger->get()->error('ProvUtils', "$request", "Request $request does not include the specified machine $mach.");
			next; # Just log it but not exit.
		}
		my $name = $ml->{$mach}->{'machine.name'};
		my $arch = $ml->{$mach}->{'machine.arch'};
		my $os = $req->{'request.machine_os'};
		my $ip = $ml->{$mach}->{'machine.ip'};
		my $type = $ml->{$mach}->{'machine.type'};
		my $reqStart = $ml->{$mach}->{'request_machine.inuse_time'};

		# Set the stop time to be the time when the history record is created instead of the request expiry time.
		my $now = time;
		my $stop = $now;
		my $energy = 0;

		# For non-VM, get the energy usage.
		if ($type ne "IBM_KVM") {
			$energy = EnergyUtils->getEnergyOfMachProv($name, $request);
			if (!$energy) {
				Logger->get()->error('ProvUtils', "$request", "getEnergyOfMachProv for $name returns 0.");
			}
		}
		# Add a record into history table.
		my $rc = CloudUtils->addProvMachLog($name, $request, $reqStart, $stop, $energy, $arch, $os, $ip);
		if ($rc != 1) {
			Logger->get()->error("ProvUtils", "$request", "Failed to add a record into history table for $name.");
			return ($GeneralDefine::FAILED, "Failed to add a record into history table for $name."); # TODO: return or next?
		}
	}

	return ($GeneralDefine::SUCCESS);
}

sub changed_request {
	my ($request, $query) = @_;
	my $errstr;
	my $runStep = "";
	my ($rv, $info) = get_changed_info($request, $query);
	if ($rv ne $GeneralDefine::SUCCESS) {
		return ($rv, $info);
	}
#Check whether the request is a hypv and remove one hypv
	if(defined($info->{'del_machines'})){
		if(CloudUtils->isaliveVMonit($request->{'request.name'}, $info->{'del_machines'})) {
			$errstr = "There are VMs using the hypervisor in the request, remove the hypervisor will destroy these VMs.";
			$rv = $GeneralDefine::FAILED;
			return ($rv, $errstr);
		}
	}	
	my $cost;
	my $diff_cost;
	($rv, $cost, $diff_cost) = ProvUtils->getBilling($query, "Modify", $info);
	if($rv == 0){
		return($GeneralDefine::FAILED, "Do not have enough balance to pay for this request.");
	}elsif($rv == -1){
		return($GeneralDefine::FAILED, "Invalid modification.");
	}
	$info->{'cost'} = $cost;
	# change_jobnres will invoke LoadL commands to change the reservation.
	# if delete, we shall not notify loadl now, but by job step
	my $tmp_info;
	my @spec_element_array=("del_machines");
	if($request->{'request.modify'} eq $GeneralDefine::MODIFY_NORESERVE){
		foreach my $ele (@spec_element_array){
			if(exists($info->{$ele})){
				$tmp_info->{$ele}=$info->{$ele};
				delete($info->{$ele});
			}
		}
	}	
	($rv, $errstr) = ProvUtils->change_jobnres($request, $info);	
	if ($rv ne $GeneralDefine::SUCCESS) {
		return ($rv, $errstr);
	}
	$runStep .= " change_jobnres";
	
	my $spec_flag={
		'del_machines'=>'d',
		'add_machines'=>'a',
	};		
	($rv, $errstr) = updateHypvVMnum($request, $info);	
	if(defined($tmp_info)){
		foreach my $ele (keys(%$tmp_info)){
			$info->{$ele} = $tmp_info->{$ele};
		}
	}
	if ($rv ne $GeneralDefine::SUCCESS) {
		goto RESTORE;
	}
	$runStep .= " updateHypvVMnum";
	
	# Update database only when LoadL succesfully changed the reservation.
	
	
	if($request->{'request.modify'} eq $GeneralDefine::MODIFY_NORESERVE){
		($rv, $errstr) = CloudUtils->modifyProvReq($request->{'request.name'}, $info, $spec_flag);
	}else{
		($rv, $errstr) = CloudUtils->modifyProvReq($request->{'request.name'}, $info);
	}
	if($rv ne "1"){
		$rv = $GeneralDefine::FAILED;
		goto RESTORE;
	}else{
		$rv = $GeneralDefine::SUCCESS;
	}
	$runStep .= " modifyProvReq";
	if(exists($info->{'add_machines'})){
		if($request->{'request.modify'} eq $GeneralDefine::MODIFY_NORESERVE){
			($rv, $errstr) = CloudUtils->updateProvMachineById($request->{'request.name'}.$spec_flag->{'add_machines'}, {'status' => 'Reserved'});
		}else{
			($rv, $errstr) = CloudUtils->updateProvMachineById($request->{'request.name'}, {'status' => 'Reserved'});
		}
		if($rv ne "1"){
			$rv = $GeneralDefine::FAILED;
			goto RESTORE;
		}else{
			$rv = $GeneralDefine::SUCCESS;
		}
	}
#submit job 
	if(($request->{'request.modify'} eq $GeneralDefine::MODIFY_NORESERVE) and 
		(exists($info->{'add_machines'}) or exists($info->{'incUsers'})
			or exists($info->{'incGrps'}) or exists($info->{'del_machines'}))){
		ProvUtils->submit_modify($request->{'request.name'},$info);
	}else{
		if($request->{'request.status'} eq "Ready"){
			ProvUtils->applyBilling($request->{'request.name'}, 'Modify', $info);
		}
		$request->{'request.modify'} = $GeneralDefine::MODIFY_RESERVE;#need not submit anything, we can quite
	}
	return ($rv, $info);
RESTORE:
	if($request->{'request.modify'} eq $GeneralDefine::MODIFY_RESERVE){
		$spec_flag={};
	}
	restoreModify($request, $info, $spec_flag, $runStep);	
	return ($rv, $errstr);
}

sub updateHypvVMnum{
	my($request, $info) = @_;

	# Update UsedKvmVms or UsedHypvNums
	if (($request->{'request.hypv_flag'} == -1) or ($request->{'request.hypv_flag'} == 1)) {
		my $vm_num=0;
		if (exists($info->{'add_machines'})){
			$vm_num += scalar(@{$info->{'add_machines'}});
		}
		if (exists($info->{'del_machines'})){
			$vm_num -= scalar(@{$info->{'del_machines'}});
		}
		
		if($vm_num != 0){
			my $rc =0;
			my $cfg = CloudUtils->getConfig();
			if(defined($cfg)){
				if($request->{'request.hypv_flag'} == -1){
					if (defined($cfg->{'UsedKvmVms'})) {
						$cfg->{'UsedKvmVms'} += $vm_num;				
					}else{
						$cfg->{'UsedKvmVms'} = $vm_num;	
					}
					if($cfg->{'UsedKvmVms'} > $cfg->{'maxKvmVms'}){
						return ($GeneralDefine::FAILED, "UsedKvmVms exceeds maxKvmVms.");
					}
				}else{
					if (defined($cfg->{'UsedHypvNums'})) {
						$cfg->{'UsedHypvNums'} += $vm_num;				
					}else{
						$cfg->{'UsedHypvNums'} = $vm_num;	
					}
					if($cfg->{'UsedHypvNums'} > $cfg->{'maxHypvNums'}){
						return ($GeneralDefine::FAILED, "UsedHypvNums exceeds maxHypvNums.");
					}
				}
				$rc = CloudUtils->putConfig($cfg);
			}
			if($rc != 1){
				return ($GeneralDefine::FAILED, "Failed to update UsedKvmVms into database.");
			}
		}
	}
	return ($GeneralDefine::SUCCESS);
}

=desc
Input $req, $info, $spec_flag;
	$req request information;
	$info change information;
	$spec_flag whether restore delete machine or not
Return $GeneralDefine::SUCCESS or $GeneralDefine::FAILED
Description: Restore loadl configuration, restore database
=cut
sub restoreModify {
	my ($request, $info, $spec_flag, $restore_setp) = @_;
	my $restore;
	my $starttime=$request->{"request.start_time"} - $GeneralDefine::JOB_IN_ADVANCE;
	my $endtime=$request->{"request.expiry_time"};	
	my $dbRSpecial;
	if(exists($info->{'pre_expiry_time'})){
		$restore->{'request'}->{'expiry_time'} = $info->{'pre_expiry_time'}; 
		$endtime=$info->{'pre_expiry_time'};
	}
	if(exists($info->{'pre_start_time'})){
		$restore->{'request'}->{'start_time'} = $info->{'pre_start_time'};
		$starttime=$info->{'pre_start_time'} - $GeneralDefine::JOB_IN_ADVANCE;
	}
	$restore->{'duration'} = int(($endtime - $starttime)/60);
	if(exists($info->{'add_machines'})){
		$restore->{'del_machines'} = $info->{'add_machines'};
		if(defined($spec_flag) and defined($spec_flag->{'add_machines'})){
			$dbRSpecial->{'del_machines'}=$spec_flag->{'add_machines'};
		}
	}
	if(exists($info->{'del_machines'})){
		$restore->{'add_machines'} = $info->{'del_machines'};
		if(defined($spec_flag) and defined($spec_flag->{'del_machines'})){
			$dbRSpecial->{'add_machines'} = $spec_flag->{'del_machines'};
		}
	}
	$restore->{'request'}->{'machine_num'} = $request->{'request.machine_num'};

	if(defined($info->{'incUsers'})){
		my @users = @{$info->{'incUsers'}};
		if(exists($info->{'add_users'})){
			my @addusers = @{$info->{'add_users'}};
			@users = CloudUtils->diff(\@users, \@addusers);
		}
		if(exists($info->{'del_users'})){
			my @delusers = @{$info->{'del_users'}};
			push(@users, @delusers);
			@users = CloudUtils->uniq(@users);
		}
		$restore->{'incUsers'} = \@users;
		if(scalar(@users) == 0){
			$restore->{'del_users'} = $info->{'incUsers'};
		}
	}
	if(defined($info->{'incGrps'})){
		my @groups= @{$info->{'incGrps'}};
		if(exists($info->{'add_groups'})){
			my @addgroups= @{$info->{'add_groups'}};
			@groups= CloudUtils->diff(\@groups, \@addgroups);
		}
		if(exists($info->{'del_groups'})){
			my @delgroups= @{$info->{'del_groups'}};
			push(@groups, @delgroups);
			@groups= CloudUtils->uniq(@groups);
		}
		$restore->{'incGrps'} = \@groups;
		if(scalar(@groups) == 0){
			$restore->{'del_groups'} = $info->{'incGrps'};
		}
	}

	if($restore_setp =~ /modifyProvReq/){
		CloudUtils->modifyProvReq($request->{'request.name'}, $restore, $dbRSpecial);
	}
	if($restore_setp =~ /change_jobnres/){
		if(defined($spec_flag) and defined($spec_flag->{'del_machines'})
				and ($spec_flag->{'del_machines'} eq 'd')){
			delete($restore->{'add_machines'});
		}
		ProvUtils->change_jobnres($request, $restore);
	}
	if($restore_setp =~ /updateHypvVMnum/){
		if(defined($spec_flag) and defined($spec_flag->{'del_machines'})
				and ($spec_flag->{'del_machines'} eq 'd')){
			delete($restore->{'add_machines'});
		}
		updateHypvVMnum($request, $restore);
	}
}


sub get_changed_info {
	my ($request, $query) = @_;
	my @machines = keys(%{$request->{'machList'}});
	my $info;
	my $rtv = $GeneralDefine::SUCCESS;

	#we do not support add and delete at same time now, a check here
	if(exists($query->{'machlist'})){
		my @machlist= split(',', $query->{'machlist'});
		my @add_machs = CloudUtils->diff(\@machlist, \@machines);
		my @del_machs = CloudUtils->diff(\@machines, \@machlist);
		if((scalar(@add_machs) != 0) and (scalar(@del_machs) != 0)){
			$rtv = $GeneralDefine::FAILED;
			$info = "Not support add and remove machine(s) in one request.";
			goto RTV;
		}
		if(scalar(@add_machs) !=0 ){
			$query->{'addmach'} = join(',', @add_machs);
		}
		if(scalar(@del_machs) != 0){
			$query->{'rmmach'} = join(',', @del_machs);
		}
	}
	# The following section of code can be simplified by checking
	if (exists($query->{'addmach'})) {
		my @ch_mach=split(',', $query->{'addmach'});
		my @add_machs = CloudUtils->diff(\@ch_mach, \@machines);
		if (scalar(@add_machs) == scalar(@ch_mach)) {
			$info->{'add_machines'} = \@add_machs;
		}else{
			$rtv = $GeneralDefine::FAILED;
			$info = "Added machine(s) $query->{'addmach'} already exist in $request->{'request.name'}.";
			goto RTV;
		}
	}
	if (exists($query->{'rmmach'})) {
		my @ch_mach=split(',', $query->{'rmmach'});
		my @del_machs = CloudUtils->diff(\@ch_mach, \@machines);
		if (scalar(@del_machs) != 0) {
			$rtv = $GeneralDefine::FAILED;
			$info = "Removed machine(s) $query->{'rmmach'} not exist in $request->{'request.name'}.";
			goto RTV;
		}else{
			$info->{'del_machines'} = \@ch_mach;
		}
	}
	if(exists($query->{'addmachnum'})){
		$info->{'add_machines_num'} = $query->{'addmachnum'};
	}
	my $machNum=$request->{'request.machine_num'};
	if (exists($info->{'add_machines'})){
		$machNum += scalar(@{$info->{'add_machines'}});
	}elsif(exists($info->{'add_machines_num'})){
		$machNum += $info->{'add_machines_num'};
	}
	if (exists($info->{'del_machines'})){
		$machNum -= scalar(@{$info->{'del_machines'}});
	}
	if($machNum != $request->{'request.machine_num'}){		
		$info->{'request'}->{'machine_num'}=$machNum;
	}
#change cpu memory disk
	if(exists($query->{'cpus'})){
		$info->{'request'}->{'machine_cpu'} = $query->{'cpus'};
	}
	if(exists($query->{'memory'})){
		$info->{'request'}->{'machine_memory'} = $query->{'memory'};
	}
	if(exists($query->{'disk'})){
		$info->{'request'}->{'machine_disk'} = $query->{'disk'};
	}
#change users
	my @users = ();
	if(exists($request->{'incUsers'})){
		@users = @{$request->{'incUsers'}};
	}
	my $changed_flag=0;
	if (exists($query->{'adduser'})){
		my @tmp_users=split(",", $query->{'adduser'});
		@tmp_users = CloudUtils->diff(\@tmp_users, \@users);
		if (scalar(@tmp_users) != 0){
			$info->{'add_users'} = \@tmp_users;
			$changed_flag +=1;
		}
	}
	if (exists($query->{'rmuser'})){
		my @tmp_users=split(",", $query->{'rmuser'});
		my @delete_users = CloudUtils->diff(\@tmp_users, \@users);
		if (scalar(@delete_users) != 0){
			$rtv = $GeneralDefine::FAILED;
			$info = "Removed user(s) $query->{'rmuser'} not exist in $request->{'request.name'}.";
			goto RTV;
		}else{
			$info->{'del_users'} = \@tmp_users;
			$changed_flag +=1;
		}
	}
	if (exists($query->{'users'})){
		my @tmp_users=split(",", $query->{'users'});
		my @add_users = CloudUtils->diff(\@tmp_users, \@users);
		if (scalar(@add_users) != 0){
			$info->{'add_users'} = \@add_users;
			$changed_flag +=1;
		}
		my @delete_users = CloudUtils->diff(\@users, \@tmp_users);
		if (scalar(@delete_users) !=0){
			$info->{'del_users'} = \@delete_users;
			$changed_flag +=1;
		}
	}
	if($changed_flag != 0){
		if(exists($info->{'add_users'})){
			push(@users,@{$info->{'add_users'}});
		}
		@users = CloudUtils->uniq(@users);
		if(exists($info->{'del_users'})){
			@users = CloudUtils->diff(\@users, $info->{'del_users'});
		}
		$info->{'incUsers'}=\@users;
	}
#change groups 
	my @groups = ();
	if(exists($request->{'incGrps'})){
		@groups = @{$request->{'incGrps'}};
	}
	$changed_flag = 0;
	if (exists($query->{'addgroup'})){
		my @tmp_groups=split(",", $query->{'addgroup'});
		@tmp_groups= CloudUtils->diff(\@tmp_groups, \@groups);
		if (scalar(@tmp_groups) != 0){
			$info->{'add_groups'} = \@tmp_groups;
			$changed_flag +=1;
		}
	}
	if (exists($query->{'rmgroup'})){
		my @tmp_groups=split(",", $query->{'rmgroup'});
		my @delete_groups= CloudUtils->diff(\@tmp_groups, \@groups);
		if (scalar(@delete_groups) != 0){
			$rtv = $GeneralDefine::FAILED;
			$info = "Removed group(s) $query->{'rmgroup'} not exist in $request->{'request.name'}.";
			goto RTV;
		}else{
			$info->{'del_groups'} = \@tmp_groups;
			$changed_flag +=1;
		}
	}
	if (exists($query->{'groups'})){
		my @tmp_groups=split(",", $query->{'groups'});
		my @add_groups= CloudUtils->diff(\@tmp_groups, \@groups);
		if (scalar(@add_groups) != 0){
			$info->{'add_groups'} = \@add_groups;
			$changed_flag +=1;
		}
		my @delete_groups= CloudUtils->diff(\@groups, \@tmp_groups);
		if (scalar(@delete_groups) !=0){
			$info->{'del_groups'} = \@delete_groups;
			$changed_flag +=1;
		}
	}
	if($changed_flag != 0){
		if(exists($info->{'add_groups'})){
			push(@groups,@{$info->{'add_groups'}});
		}
		@groups = CloudUtils->uniq(@groups);
		if(exists($info->{'del_groups'})){
			@groups = CloudUtils->diff(\@groups, $info->{'del_groups'});
		}
		$info->{'incGrps'}=\@groups;
	}
#change date
	my $enddate = $request->{'request.expiry_time'};
	if (exists($query->{'enddate'})) {
		$info->{'request'}->{'expiry_time'} = $query->{'enddate'};
		$enddate = $info->{'request'}->{'expiry_time'};
		$info->{'pre_expiry_time'} = $request->{'request.expiry_time'};
	}
	my $startdate = $request->{'request.start_time'}-$GeneralDefine::JOB_IN_ADVANCE;
	if (exists($query->{'startdate'})){
		$info->{'request'}->{'start_time'} = $query->{'startdate'};
		$startdate = $info->{'request'}->{'start_time'} - $GeneralDefine::JOB_IN_ADVANCE;
		$info->{'pre_start_time'} = $request->{'request.start_time'};
		if(!exists($query->{'enddate'})){
			$enddate = $info->{'request'}->{'start_time'} + ($request->{'request.expiry_time'}- $request->{'request.start_time'});#keep duration not changed
			if($enddate != $request->{'request.expiry_time'}){
				$info->{'request'}->{'expiry_time'} = $enddate;
				$info->{'pre_expiry_time'} = $request->{'request.expiry_time'};
			}	
		}
	}
	if (exists($query->{'duration'})) {
		if ($query->{'duration'} > 0){
			$info->{'duration'}  = $query->{'duration'};
			$enddate = $startdate + $GeneralDefine::JOB_IN_ADVANCE + $query->{'duration'}*60;
			if($enddate != $request->{'request.expiry_time'}){
				$info->{'pre_expiry_time'} = $request->{'request.expiry_time'};
				$info->{'request'}->{'expiry_time'}=$enddate;
			}
		}
		else{
			$rtv = $GeneralDefine::FAILED;
			$info = "The duration $query->{'duration'} is invalid.";
			goto RTV;
		}
	}
	if($enddate - $startdate <= 0){
		$rtv = $GeneralDefine::FAILED;
		$info = "The startdate or enddate is invalid.";
		goto RTV;
	}
	if(($enddate - $startdate) != ($request->{'request.expiry_time'} - $request->{'request.start_time'})){
		$info->{'duration'} = int(($enddate - $startdate)/60);
	}
RTV:
	return ($rtv,$info);
}

sub change_jobnres {
	my ($class, $request, $info) = @_;
	my $changetime = "";
	my $postjobid = "";
	my $rv = 0;
	my @outs;
	my $llchres = "llchres -R $request->{'request.ll_resid'}";
	my $change_time_ref;
	use POSIX 'strftime';

	# A reservation owner who is not a LoadLeveler administrator cannot change the
	# start time, duration or reserved resources if the start time is not at least later
	# than the current time by the number of minutes specified by the
	# RESERVATION_MIN_ADVANCE_TIME keyword.

	# The start time change
	if (exists($info->{'request'}) and exists($info->{'request'}->{'start_time'})) {
		# Do not allow the change if the new expiry_time - current_time < 10 minutes.
		my $now = time;
		if (($info->{'request'}->{'start_time'} - $now) < ($GeneralDefine::JOB_IN_ADVANCE + 60)) {
			return ($GeneralDefine::FAILED, "The new starttime of the request is too close to the current time.");
		}
#step 0
		$changetime= strftime("%m/%d/%Y %H:%M", localtime($info->{'request'}->{'start_time'}-$GeneralDefine::JOB_IN_ADVANCE));
		# TODO: hard code step id
		my $llmodify;
		foreach my $stepid (0..3){
			$postjobid = $request->{'request.ll_jobid'} . ".$stepid";
			$llmodify = "llmodify -k startdate=\"$changetime\" $postjobid";
			($rv, @outs) = CmdUtils->runloadlcmd("$llmodify", "cloudadm");

			# llmodify may fail if the step has started or unexpected error occured in LoadL.
			if ($rv != 0) {
				# return (0, @outs); ## write the outputs to log DB?
				return ($GeneralDefine::FAILED, "Failed to modify the startdate of provisioning steps.");
			}
		}
		$change_time_ref->{"setup_time"} = $changetime;
		$request->{'request.setup_time'} = $request->{'request.start_time'} - $GeneralDefine::JOB_IN_ADVANCE;

		$changetime= strftime("%m/%d/%Y %H:%M", localtime($info->{'request'}->{'start_time'}));
		# TODO: hard code step id .4
		$postjobid = $request->{'request.ll_jobid'} . ".4";
		$llmodify = "llmodify -k startdate=\"$changetime\" $postjobid";
		($rv, @outs) = CmdUtils->runloadlcmd("$llmodify", "cloudadm");

		# llmodify may fail if the step has started or unexpected error occured in LoadL.
		if ($rv != 0) {
			# return (0, @outs); ## write the outputs to log DB?
			return ($GeneralDefine::FAILED, "Failed to modify the startdate of the step that should be run when the request is ready.");
		}
		$change_time_ref->{"start_time"} = $changetime;
	}
	# Change the startdate of the PostProvisionDataCenter step first because
	# it's easier to undo llmodify than to undo llchres.
	# The PostProvisionDataCenter step will be run when the request expires.
	if (exists($info->{'request'}) and exists($info->{'request'}->{'expiry_time'})) {
		# Do not allow the change if the new expiry_time - current_time < 10 minutes.
		my $now = time;
		if (($info->{'request'}->{'expiry_time'} - $now) < $GeneralDefine::JOB_IN_ADVANCE) { # TODO: need to check in WEB
			return ($GeneralDefine::FAILED, "The new enddate of the request is too close to the current time.");
		}

		$changetime= strftime("%m/%d/%Y %H:%M", localtime($info->{'request'}->{'expiry_time'}));
		# TODO: hard code step id .5
		$postjobid = $request->{'request.ll_jobid'} . ".5";
		my $llmodify = "llmodify -k startdate=\"$changetime\" $postjobid";
		($rv, @outs) = CmdUtils->runloadlcmd("$llmodify", "cloudadm");
		# llmodify may fail if the step has started or unexpected error occured in LoadL.
		if ($rv != 0) {
			# return (0, @outs); ## write the outputs to log DB?
			return ($GeneralDefine::FAILED, "Failed to modify the startdate of the step that should be run when the request expires.");
		}
		$change_time_ref->{"expiry_time"} = $changetime;
	}

	if (exists($info->{'add_machines'})) {
		my $machlist = join(" ", @{$info->{'add_machines'}});
		$llchres .= " -h + $machlist";
	}

	elsif (exists($info->{'del_machines'})) {
		my $machlist = join(" ", @{$info->{'del_machines'}});
		$llchres .= " -h - $machlist";
	}

	elsif (exists($info->{'add_machines_num'}) && $info->{'add_machines_num'} > 0) {
		$llchres .= " -n + $info->{'add_machines_num'}";
	}
	#llchres -h - host1 host2 -d duration has a bug, the host2 will not be removed, so I seperate the command
	($rv, @outs) = CmdUtils->runloadlcmd("$llchres", "cloudadm");
	if ($rv != 0) {
		goto FAILED;
	}
	# If add machines by llchres -n + n, need to figure out the new added machine names.
	if(exists($info->{'add_machines_num'})){
		my $queryres = "llqres -lsR $request->{'request.ll_resid'}";
		($rv, @outs) = CmdUtils->runloadlcmd("$queryres");
		if ($rv != 0) {
			# TODO: Undo llchres and llmodify?
			# Should never be here.
			return ($GeneralDefine::FAILED, "Failed to query reservation $request->{'request.ll_resid'}.");
		}
		my @cur_machs = parse_nodes(\@outs);
		my @old_machs = keys(%{$request->{'machList'}});
		my @add_machs = CloudUtils->diff(\@cur_machs, \@old_machs);
		$info->{'add_machines'} = \@add_machs;
	}

	$llchres = "llchres -R $request->{'request.ll_resid'}";
	if(exists($info->{'incUsers'})) {
		if(scalar(@{$info->{'incUsers'}} != 0)){
			my $userlist = join(" ", @{$info->{'incUsers'}});
			$llchres .= " -U $userlist";
		}elsif(exists($info->{'del_users'}) and scalar(@{$info->{'del_users'}}) != 0){
			my $userlist = join(" ", @{$info->{'del_users'}});
			$llchres .= " -U - $userlist";
		}
	}
	if (exists($info->{'incGrps'})) {
		if(scalar(@{$info->{'incGrps'}} != 0)){
			my $grouplist = join(" ", @{$info->{'incGrps'}});
			$llchres .= " -G $grouplist";
		}elsif(exists($info->{'del_groups'}) and scalar(@{$info->{'del_groups'}}) != 0){
			my $grouplist = join(" ", @{$info->{'del_groups'}});
			$llchres .= " -G - $grouplist";
		}
	}

	# The duration/enddate of the request is changed.
	if (exists($info->{'duration'})){
		my $duration = $info->{'duration'};
		$llchres .= " -d $duration";
	}
	# The duration/enddate of the request is changed.
	if (exists($info->{'request'}) and exists($info->{'request'}->{'start_time'})){
		my $starttime= $change_time_ref->{"setup_time"};
		$llchres .= " -t  \"$starttime\"";
	}
	($rv, @outs) = CmdUtils->runloadlcmd("$llchres", "cloudadm");
	if ($rv != 0) {
		if (exists($info->{'add_machines'}) or (exists($info->{'del_machines'}))) {
			$llchres = "llchres -R $request->{'request.ll_resid'}";
			if (exists($info->{'add_machines'})) {
				my $machlist = join(" ", @{$info->{'add_machines'}});
				$llchres .= " -h - $machlist";
			}
			elsif (exists($info->{'del_machines'})) {
				my $machlist = join(" ", @{$info->{'del_machines'}});
				$llchres .= " -h + $machlist";
			}
			($rv, @outs) = CmdUtils->runloadlcmd("$llchres", "cloudadm");
		}
		goto FAILED;
	}

	# Update the JCF to make it be consistent with the updated job.
	if (defined($change_time_ref)){
		change_jcf($request, $change_time_ref);
	}

	return ($GeneralDefine::SUCCESS, $info);
FAILED:
	# undo llmodify before return.
	if (exists($info->{'request'}) and exists($info->{'request'}->{'start_time'})){
		my $old_time= strftime("%m/%d/%Y %H:%M", localtime($request->{'request.start_time'}));
		$postjobid = $request->{'request.ll_jobid'} . ".4";
		my $llmodify = "llmodify -k startdate=\"$old_time\" $postjobid";
		CmdUtils->runloadlcmd("$llmodify", "cloudadm"); # Need not check if it is successful or not
		$old_time= strftime("%m/%d/%Y %H:%M", localtime($request->{'request.setup_time'}));
		foreach my $stepid (0..3){
			$postjobid = $request->{'request.ll_jobid'} . ".0";
			$llmodify = "llmodify -k startdate=\"$old_time\" $postjobid";
			CmdUtils->runloadlcmd("$llmodify", "cloudadm"); # Need not check if it is successful or not
		}
	}
	if (exists($info->{'request'}) and exists($info->{'request'}->{'expiry_time'})) {
		my $old_time= strftime("%m/%d/%Y %H:%M", localtime($request->{'request.expiry_time'}));
		$postjobid = $request->{'request.ll_jobid'} . ".5";
		my $llmodify = "llmodify -k startdate=\"$old_time\" $postjobid";
		CmdUtils->runloadlcmd("$llmodify", "cloudadm"); # Need not check if it is successful or not
	}
	return ($GeneralDefine::FAILED, "Failed to reserve the modified resource for the request.");
}

sub change_jcf {
	my ($request, $changeTime) = @_;
	my $template = "metaprov";
	my $jcf = "/hpcc/home/" . $request->{'request.owner'} . "/" . $request->{'request.name'} . "/";
	$jcf .= $template . ".jcf";

	use POSIX 'strftime';
	foreach my $step(keys(%$changeTime)){
		my $origtime = strftime("%m/%d/%Y %H:%M", localtime($request->{"request.$step"}));
		CmdUtils->runcmd("sed -i 's|$origtime|$changeTime->{$step}|g' $jcf");
	}

	return ($GeneralDefine::SUCCESS);
}

=comment
Input: request_id, action(ren|del), flag(a|d)
=cut
sub updateReqMach {
	my ($class, $request_id, $action, $flag) = @_;

	# Verify the inputs.
	if (!$request_id) {
		return ($GeneralDefine::FAILED, "No request id is specified.");
	}

	if (($action ne "ren") && ($action ne "del")) {
		return ($GeneralDefine::FAILED, "The specified action $action is not supported.");
	}

	if (($flag ne "a") && ($flag ne "d")) {
		return ($GeneralDefine::FAILED, "The specified flag $flag is not supported.");
	}

	my $req_id_new = $request_id . $flag;
	my $pd = ProvisionDAO->new();

	if ($action eq 'ren') {
		# Rename $req_id_new to $request_id in thcr_request_machine
		my ($rc, $param) = $pd->modifyRequestMachine({'request_name' => $req_id_new}, {'request_name' => $request_id});
		if ($rc != 0) {
			return ($GeneralDefine::FAILED, "Failed to rename $req_id_new to $request_id.");
		}
	}
	else {
		# Delete the $req_id_new records from thcr_request_machine
		my ($rc, $param) = $pd->deleteRequestMachine({"request_name" => $req_id_new});
		if ($rc != 0) {
			return ($GeneralDefine::FAILED, "Failed to delete $req_id_new records.");
		}
	}

	$pd->commit();

	return ($GeneralDefine::SUCCESS);
}

sub setup_vlan {
	my ($class, $request_id, $nodes) = @_;

	my $args->{'noderange'} = join(",", sort(@$nodes));

	my ($rv, $info) = mkvlan($args);
	if ($rv != 0) {
		return(1, $info);
	}

	my $vlan_info = $info;

	($rv, $info) = update_vlan_info($request_id, $vlan_info);
	if ($rv != 0) {
		return(1, $info);
	}

	return(0, $vlan_info);
}

sub mkvlan {
	my ($args) = @_;
	my $mkvlan = "mkvlan";

	if (exists($args->{'vlanid'})) {
		$mkvlan .= " $args->{'vlanid'}";
	}

	if (exists($args->{'noderange'})) {
		$mkvlan .= " --nodes $args->{'noderange'}";
	}

	if (exists($args->{'subnet'})) {
		$mkvlan .= " --net $args->{'subnet'}";
	}

	if (exists($args->{'netmask'})) {
		$mkvlan .= " --mask $args->{'netmask'}";
	}

	if (exists($args->{'prefix'})) {
		$mkvlan .= " --prefix $args->{'prefix'}";
	}

	my ($rv, @outs) = CmdUtils->runxcatcmd($mkvlan, "root");
	# cause $rv can not indicate failed.
	while (my $out = shift(@outs)) {
		if ($out =~ 'successful') {
			return(0, parse_vlan(\@outs));
		}
	}

	# mkvlan failed.
	return(1, 'failed');
}

sub join_vlan {
	my ($class, $request_id, $vlan_id, $nodes) = @_;
	my $issub = 0;
	my $args->{'noderange'} = join(",", sort(@$nodes));
	if ($vlan_id < 1) {
		return (1);
	}
	$args->{'vlanid'} = $vlan_id;
	my ($ret, $info) = chvlan($args);
	if ($ret != 0) {
		return(1, $info);
	}

	my $vlan_info = $info;
	#update vlan info in thcr_request_machine for added machine
	($ret, $info) = update_vlan_info($request_id, $vlan_info, "join");
	if ($ret != 0) {
		return(1, $info);
	}

	return(0, $vlan_info);
}

sub withdraw_vlan {
	my ($class, $vlan_id, $nodes) = @_;
	my $args->{'noderange'} = join(",", sort(@$nodes));

	$args->{'remove'} = 1;
	$args->{'vlanid'} = $vlan_id;

	my ($ret, $info) = chvlan($args);
	if ($ret != 0) {
		return(1, $info);
	}

	return(0);
}

sub chvlan {
	my ($args) = @_;
	my $mkvlan = "chvlan";

	$mkvlan .= " $args->{'vlanid'}";

	$mkvlan .= " --nodes $args->{'noderange'}";

	if (exists($args->{'remove'})) {
		$mkvlan .= " -d";
	}

	my ($rv, @outs) = CmdUtils->runxcatcmd($mkvlan, "root");
#	if ($rv == 0) {
#do not check rv since if the machine not provisioned, xcat will return 1 but in fact, everything is ok
	return(0, list_vlan($args->{'vlanid'}));
#	}

	# chvlan failed.
#	return(1, 'failed');
}

sub update_vlan_info {
	my ($request_id, $vlan_info, $modify) = @_;

	# if not modify request,update the vlan_id into reqeust table.
	my $req_info = {
		'vlan_id' => $vlan_info->{'id'},
	};
	my $rv = 0;
	my $reason;

	if(!defined($modify)) {
		($rv, $reason) = CloudUtils->updateProvDynAttrById($request_id, $req_info);
		if ($rv != 1) {
			print("$reason\n");
			return 1;
		}
	}

	$rv = 0;

	if (defined($modify)) {
		$request_id .= "a";	
	}
	foreach my $mach (keys(%{$vlan_info->{'hypervisors'}})) {
		my $reqmach_info = {
			'using_hostname' => $vlan_info->{'hypervisors'}->{$mach}->{'hostname'},
			'using_ip' => $vlan_info->{'hypervisors'}->{$mach}->{'ip address'},
		};
		my ($rv, $reason) = update_request_machine($request_id, $mach, $reqmach_info);
		if ($rv != 0) {
			print("$reason\n");
			return 1;
		}
	}

	foreach my $mach (keys(%{$vlan_info->{'machines'}})) {
		my $reqmach_info = {
			'hypervisor' => $vlan_info->{'machines'}->{$mach}->{'vm host'},
			'using_hostname' => $vlan_info->{'machines'}->{$mach}->{'hostname'},
			'using_ip' => $vlan_info->{'machines'}->{$mach}->{'ip address'},
		};

		($rv, $reason) = update_request_machine($request_id, $mach, $reqmach_info);
		if ($rv != 0) {
			print("$reason\n");
			return 1;
		}
	}

	return $rv;
}

sub parse_vlan {
	my ($outs) = @_;
	my $vlan_info;

	foreach my $out (@$outs) {
		if ($out =~ /vlan id\: (\d+)/) {
			$vlan_info->{'id'} = $1;
		}
		elsif ($out =~ /vlan subnet\: (\d+\.\d+\.\d+\.\d+)/) {
			$vlan_info->{'subnet'} = $1;
		}
		elsif ($out =~ /vlan netmask\: (\d+\.\d+\.\d+\.\d+)/) {
			$vlan_info->{'netmask'} = $1;
		}
	}

	my ($rv, @lsouts) = CmdUtils->runxcatcmd("lsvlan $vlan_info->{'id'}", "root");
	if ($rv != 0) {
		return $vlan_info;
	}

	splice(@lsouts, 0, 5);

	my $machines;
	my $hypervisors;
	foreach my $out (@lsouts) {
		$out =~ /(\S+)\s+(\S+)\s+(\S+)\s*(.*)/;
		if ($4) {
			$machines->{$3}->{'hostname'} = $1;
			$machines->{$3}->{'ip address'} = $2;
			$machines->{$3}->{'vm host'} = $4;
		}
		else {
			$hypervisors->{$3}->{'hostname'} = $1;
			$hypervisors->{$3}->{'ip address'} = $2;
		}
	}

	$vlan_info->{'machines'} = $machines;
	$vlan_info->{'hypervisors'} = $hypervisors;

	return $vlan_info;
}
sub list_vlan {
	my ($vlanid) = @_;
	my $vlan_info;
	$vlan_info->{'id'} = $vlanid;

	my ($rv, @lsouts) = CmdUtils->runxcatcmd("lsvlan $vlanid", "root");
	if ($rv != 0) {
		return undef;
	}

	shift(@lsouts);
	my $out1 = shift (@lsouts);
	$out1 =~ /subnet (\d+\.\d+\.\d+\.\d+)/;
	$vlan_info->{'subnet'} = $1;

	$out1 = shift (@lsouts);
	$out1 =~ /netmask (\d+\.\d+\.\d+\.\d+)/; 
	$vlan_info->{'netmask'} = $1;
	shift(@lsouts);
	shift(@lsouts);

	my $machines;
	my $hypervisors;
	foreach my $out (@lsouts) {
		$out =~ /(\S+)\s+(\S+)\s+(\S+)\s*(.*)/;
		if ($4) {
			$machines->{$3}->{'hostname'} = $1;
			$machines->{$3}->{'ip address'} = $2;
			$machines->{$3}->{'vm host'} = $4;
		}
		else {
			$hypervisors->{$3}->{'hostname'} = $1;
			$hypervisors->{$3}->{'ip address'} = $2;
		}
	}

	$vlan_info->{'machines'} = $machines;
	$vlan_info->{'hypervisors'} = $hypervisors;

	return $vlan_info;
}
sub update_request_machine {
	my ($reqname, $mach, $info) = @_;

	my $pd = ProvisionDAO->new();

	my $query = {
		'request_name' => $reqname,
		'machine_name' => $mach,
	};

	my ($rv, $reason) = $pd->modifyRequestMachine($query, $info);
	if ($rv != 0) {
		return (1, $reason);
	}

	$pd->commit();

	return $rv;
}

sub remove_vlan {
	my ($class, $vlan_id) = @_;

	if (!defined($vlan_id) || ($vlan_id < 1)) {
		return ;
	}

	CmdUtils->runxcatcmd("rmvlan $vlan_id", "root");

	return ;
}

sub create_reservation {
	my ($request_id, $request_info, $machines, $users, $groups) = @_;

	my $llmkres = "llmkres -P";

	use POSIX 'strftime';
	my $starttime = $request_info->{'start_time'}-$GeneralDefine::JOB_IN_ADVANCE;
	my $posttime = $request_info->{'expiry_time'};
	my $duration = ($posttime - $starttime ) / 60;

	my $startdate = strftime("%m/%d/%Y %H:%M\n", localtime($starttime));
	chomp($startdate);

	$llmkres .= " -t \"$startdate\" -d $duration";

	# Query the user by this provison request id from HPC Cloud Database.
	if (scalar(@$users)) {
		my $usrlist = join(" ", @$users);
		$llmkres .= " -U $usrlist";
	}

	# Query the groups by this provision request id from HPC Cloud Database.
	if (scalar(@$groups)) {
		my $grplist = join(" ", @$groups);
		$llmkres .= " -G $grplist";
	}

	my $nodetype = $request_info->{'machine_type'};
	# reserve by type
	if (defined($nodetype)) {
		$llmkres .= " -T $nodetype -n $request_info->{'machine_num'}";
	}
	else {
		my $nodelist = join(" ", @$machines);

		$llmkres .= " -h $nodelist";
	}

	my ($rv, @out) = CmdUtils->runloadlcmd($llmkres);
	if ($rv != 0) {
		goto failed;
	}

	my @results = split(" ", $out[0]);
	$request_info->{'ll_resid'} = $results[3];

	# get the machlist for reserve by type.
	if (defined($nodetype)) {
		($rv, @out) = CmdUtils->runloadlcmd("llqres -l -s -R $request_info->{'ll_resid'}");
		if ($rv != 0) {
			goto failed;
		}

		@$machines = parse_nodes(\@out);
	}

failed:
	return $rv;
}

sub delete_reservation {
	my ($request_info) = @_;

	# Remove the reservation for the specified provision request.
	if (exists($request_info->{'request.ll_resid'})) {
		CmdUtils->runloadlcmd("llrmres -R $request_info->{'request.ll_resid'}");
	}
	elsif (exists($request_info->{'ll_resid'})) {
		CmdUtils->runloadlcmd("llrmres -R $request_info->{'ll_resid'}");
	}

	return ;
}

sub parse_nodes {
	my ($out) = shift;

	my $nodenum = 0;
	my $i;

	for ($i = 0; $i < scalar(@$out); ++$i) {
		if ($out->[$i] =~ /Nodes:\s*(\d*)\s*/) {
			$nodenum = $1;
			last;
		}
	}

	++$i;
	my $last = ($nodenum + $i <= scalar(@$out)) ? $nodenum + $i : scalar(@$out);

	my @nodes;
	for (; $i < $last; ++$i) {
		chomp($out->[$i]);
		$out->[$i] =~ s/\s*//i;
		if ($out->[$i] ne "") {
			push(@nodes, $out->[$i]);
		}
	}

	return @nodes;
}

sub submit_prov {
	my ($request_id, $request_info) = @_;

	#change the work dir.
	my $mypath = $ENV{'HOME'} . "/" . "$request_id";
	my ($rv, @out) = CmdUtils->runcmd("mkdir -p $mypath");
	if ($rv != 0) {
		return $rv;
	}

	my $template = "/opt/ibm/hpccloud/cloudbin/prov/metaprov.cmd";
	my $myjcf = "~/" . $request_id . "/metaprov.jcf";

	my $start = $request_info->{'start_time'};
	my $end = $request_info->{'expiry_time'};
	my $setup = $start - $GeneralDefine::JOB_IN_ADVANCE; 

	use POSIX 'strftime';
	$start = strftime("%m/%d/%Y %H:%M\n", localtime($start));
	chomp($start);
	$end = strftime("%m/%d/%Y %H:%M\n", localtime($end));
	chomp($end);
	$setup = strftime("%m/%d/%Y %H:%M\n", localtime($setup));
	chomp($setup);

	my $formatjcf = "cat $template | sed \"s|SETUP_DATE|$setup|g\" | sed \"s|START_DATE|$start|g\" | sed \"s|END_DATE|$end|g\" | sed \"s|PROVISION_ID|$request_id|g\" > $myjcf";

	# produce prov jcf file and copy it to user home dir.
	($rv, @out) = CmdUtils->runcmd("$formatjcf");
	if ($rv != 0) {
		CmdUtils->runcmd("rm -rf ~/$request_id/");
		return $rv;
	}

	# change the working dir.
	chdir($mypath);

	# Submit provision job.
	($rv, @out) = CmdUtils->runloadlcmd("llsubmit $myjcf");
	if ($rv != 0) {
		CmdUtils->runcmd("rm -rf ~/$request_id/");
	}

	my $jobid = parse_jobid(\@out);

	# Insert the job id to HPC Cloud Database.
	$rv = CloudUtils->updateProvDynAttrById($request_id, {'ll_jobid' => $jobid});
	if (!$rv) {
		# need xxx cleanup.
		CmdUtils->runloadlcmd("llcancel $jobid");
		CmdUtils->runcmd("rm -rf ~/$request_id/");
		print("CloudUtils->updateProvDynAttrById failed.\n");
	}

	return 0;
}

sub cancel_prov {
	my ($class, $request_id, $jobid) = @_;

	CmdUtils->runloadlcmd("llcancel $jobid");

	return ;
}

sub submit_modify {
	my ($class, $request_id, $changed_info) = @_;

	my $parameters = $request_id;
	my $template = "/opt/ibm/hpccloud/cloudbin/prov/modifyprov.cmd";

	$parameters .= ",";
	if (exists($changed_info->{'add_machines'})) {
		$parameters .= join(' ', @{$changed_info->{'add_machines'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'del_machines'})) {
		$parameters .= join(' ', @{$changed_info->{'del_machines'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'add_users'})) {
		$parameters .= join(' ', @{$changed_info->{'add_users'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'del_users'})) {
		$parameters .= join(' ', @{$changed_info->{'del_users'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'add_groups'})) {
		$parameters .= join(' ', @{$changed_info->{'add_groups'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'del_groups'})) {
		$parameters .= join(' ', @{$changed_info->{'del_groups'}});
	}

	$parameters .= ",";
	if (exists($changed_info->{'pre_expiry_time'})) {
		$parameters .= $changed_info->{'pre_expiry_time'};
	}

	$parameters .= ",";
	if (exists($changed_info->{'cost'})) {
		$parameters .= $changed_info->{'cost'};
	}

	my $myjcf = "~/" . $request_id . "/modifyprov." . time() . ".jcf";

	my $formatjcf = "cat $template | sed \"s|PARAMETERS|$parameters|g\" > $myjcf";

	# produce prov jcf file and copy it to user home dir.
	my ($rv, @out) = CmdUtils->runcmd("$formatjcf");
	if ($rv != 0) {
		return $rv;
	}

	#change the work dir.
	my $mypath = $ENV{'HOME'} . "/" . "$request_id";
	chdir($mypath);

	# Submit provision job.
	($rv, @out) = CmdUtils->runloadlcmd("llsubmit $myjcf");
	if ($rv != 0) {
		return $rv;
	}

	my $jobid = parse_jobid(\@out);

	# Insert the job id to HPC Cloud Database.
	$rv = CloudUtils->updateProvDynAttrById($request_id, {'ll_jobid_new' => $jobid});
	if (!$rv) {
		CmdUtils->runloadlcmd("llcancel $jobid");
		CmdUtils->runcmd("rm -f $myjcf");
		print("CloudUtils->updateProvDynAttrById failed.\n");
	}

	return 0;
}

sub modify_addmachs {
	my ($class, $request_info, $machs) = @_;

	my $request_id = $request_info->{'request.name'};
	my $hypv_flag = $request_info->{'request.hypv_flag'};
	my $rv = 0;
	my @outs;

	if ($hypv_flag == -1) {
		($rv, @outs) = CmdUtils->runcmd("/opt/ibm/hpccloud/cloudbin/prov/findvhypv $request_id Modify @$machs", "root");
		if ($rv != 0) {
			print("findvhypv failed.\n");
			goto failed;
		}
		#pass the new machs to makevm
		$rv = ProvUtils->makevm($request_info, $machs);
		if ($rv != 0) {
			print("ProvUtils->makevm failed.\n");
			goto failed;
		}
	}

	my $machs_info;
	my $minfo;
	foreach my $mach (@$machs) {
		($rv, $minfo) = CloudUtils->getMachine($mach);
		if ($rv == 1) {
			$machs_info->{$mach} = $minfo;
		}
	}
	my $trait = CloudUtils->machine_trait($machs_info, $request_info);
	foreach my $mach (@$machs) {
		my $current_os = $trait->{'machines'}->{$mach}->{'os'} . '-' . $trait->{'machines'}->{$mach}->{'profile'};
		my $attrs = {
			'status' => 'Install',
		};

		CloudUtils->updateMachByName($mach, $attrs);
	}
	my $newReqname="$request_id"."a";
	$rv = CloudUtils->updateProvMachineById($newReqname, {'status' => 'Install'});
	if ($rv != 1) {
	    print("CloudUtils->updateProvMachineById to Install status failed.\n");
	    goto failed;
	}

	# Find out the central manager
	my $vlaninfo;
	if(($request_info->{'request.vlan_flag'} == 1) and defined($request_info->{'request.vlan_id'})){
		$vlaninfo = list_vlan($request_info->{'request.vlan_id'});
	}
	my $cm = "";
	my $realCM="";
	if ($request_info->{'request.cluster'} eq 'Sub') {
		my @oldmachs = keys(%{$request_info->{'machList'}});
		foreach my $mach (@oldmachs) {
			my $is_cm;
			my $newmach = $mach;
			if(defined($vlaninfo)){
				if(defined($vlaninfo->{'machines'}) and defined($vlaninfo->{'machines'}->{$mach})){
					$newmach = $vlaninfo->{'machines'}->{$mach}->{'hostname'};
				}elsif(defined($vlaninfo->{'hypervisors'}) and defined($vlaninfo->{'hypervisors'}->{$mach})){
					$newmach = $vlaninfo->{'hypervisors'}->{$mach}->{'hostname'};
				}
			}
			($rv, $is_cm) = ProvUtils->checkNodeCM($mach, $newmach);
			if (($rv == 0) && $is_cm) {
				$cm = $newmach;
				$realCM = $mach;
				last;
			}
		}
	}
	$trait->{'cm'} = $cm;

	$rv = CloudUtils->setup_machine($machs, $trait);
	if ($rv != 0) {
		print("CloudUtils->setup_machines failed.\n");
		goto failed;
	}

	# Check the status of the blades as they install.
	$rv = CloudUtils->wait_machine($machs, $GeneralDefine::WAIT_MACHINE_READY, $trait);
	if ($rv != 0) {
		print("CloudUtils->wait_machine failed.\n");
		CloudUtils->release_machine($machs, $trait);
		goto failed;
	}

	my @newMachs=();
	if(defined($vlaninfo)){
#refresh after join new machines
		$vlaninfo = list_vlan($request_info->{'request.vlan_id'});
		foreach my $mach (@$machs) {
			my $newmach;
			if(defined($vlaninfo->{'machines'}) and defined($vlaninfo->{'machines'}->{$mach})){
				$newmach = $vlaninfo->{'machines'}->{$mach}->{'hostname'};
			}elsif(defined($vlaninfo->{'hypervisors'}) and defined($vlaninfo->{'hypervisors'}->{$mach})){
				$newmach = $vlaninfo->{'hypervisors'}->{$mach}->{'hostname'};
			}
			push(@newMachs, $newmach);
		}
	}
	# reconfig the loadleveler on old machs if it is a sub cluster.
	if ($request_info->{'request.cluster'} eq 'Sub' && $cm ne "") {
		print "sub and cm is $cm\n";
		if(scalar(@newMachs) > 0){
			ProvUtils->modify_reconfigSubLoadl($realCM, 1, \@newMachs);
		}else{
			ProvUtils->modify_reconfigSubLoadl($cm, 1, $machs);
		}
	}

	foreach my $mach (@$machs) {
		my $current_os = $trait->{'machines'}->{$mach}->{'os'} . '-' . $trait->{'machines'}->{$mach}->{'profile'};
		my $attrs = {
			'hypv_flag' => $hypv_flag,
			'power_state' => 'on',
			'current_os' => $current_os,
			'status' => 'InUse',
		};

		CloudUtils->updateMachByName($mach, $attrs);
	}
	use POSIX 'strftime';
	my $now = time;
	$rv = CloudUtils->updateProvMachineById($newReqname, {'status' => 'InUse','inuse_time'=>$now});
	if ($rv != 1) {
	    print("CloudUtils->updateProvMachineById to InUse status failed.\n");
	    goto failed;
	}
	my $machlist = join(',', @$machs);
	($rv, @outs) = CmdUtils->runcmd("/opt/ibm/hpccloud/cloudbin/energy/xcat/applyepolicy $machlist", "root");
	if ($rv != 0) {
		print("applyepolicy failed.\n");
		$rv = 0;#energy failed, we do not quit
	}
failed:
	return $rv;
}

sub modify_delmachs {
	my ($class, $request_info, $machs) = @_;

	my $request_id = $request_info->{'request.name'};
	my $hypv_flag = $request_info->{'request.hypv_flag'};
	my $rv = 0;
	my $is_cm = 0;

	# check if if any of the specified machine is the Central Manager of the subcluster
	my $vlaninfo;
	if(($request_info->{'request.vlan_flag'} == 1) and defined($request_info->{'request.vlan_id'})){
		$vlaninfo = list_vlan($request_info->{'request.vlan_id'});
	}
	my @newMachs=();
	if ($request_info->{'request.cluster'} eq 'Sub') {
		foreach my $mach (@$machs) {
			my $newmach;
			if(defined($vlaninfo)){
				if(defined($vlaninfo->{'machines'}) and defined($vlaninfo->{'machines'}->{$mach})){
					$newmach = $vlaninfo->{'machines'}->{$mach}->{'hostname'};
				}elsif(defined($vlaninfo->{'hypervisors'}) and defined($vlaninfo->{'hypervisors'}->{$mach})){
					$newmach = $vlaninfo->{'hypervisors'}->{$mach}->{'hostname'};
				}
				push(@newMachs, $newmach);
			}
			($rv, $is_cm) = ProvUtils->checkNodeCM($mach, $newmach);
			if (($rv == 0) && $is_cm) {
				# can not delete the central manager.
				Logger->get()->warning('ProvUtils', '$request_id', "modify_delmachs: Can not delete machine $mach from the reuest since $mach is the central manager.");
				return 1;
			}
		}
	}

	# reconfig the loadleveler on old machs.
	if ($request_info->{'request.cluster'} eq 'Sub') {
		my @oldmachs = keys(%{$request_info->{'machList'}});

		foreach my $mach (@oldmachs) {
			my $newmach;
			if(defined($vlaninfo)){
				if(defined($vlaninfo->{'machines'}) and defined($vlaninfo->{'machines'}->{$mach})){
					$newmach = $vlaninfo->{'machines'}->{$mach}->{'hostname'};
				}elsif(defined($vlaninfo->{'hypervisors'}) and defined($vlaninfo->{'hypervisors'}->{$mach})){
					$newmach = $vlaninfo->{'hypervisors'}->{$mach}->{'hostname'};
				}
			}
			($rv, $is_cm) = ProvUtils->checkNodeCM($mach, $newmach);
			if (($rv == 0) && $is_cm) {
				if(scalar(@newMachs) > 0){
					ProvUtils->modify_reconfigSubLoadl($mach, 0, \@newMachs);
				}else{
					ProvUtils->modify_reconfigSubLoadl($mach, 0, $machs);
				}
				last;
			}
		}
	}

	$rv = 0;

	my $machs_info;
	foreach my $mach (@$machs) {
		my $minfo = CloudUtils->getMachine($mach);
		$machs_info->{$mach} = $minfo;
	}

	my $trait = CloudUtils->machine_trait($machs_info, $request_info, 1);

	# release the undesirable machines.
	CloudUtils->release_machine($machs, $trait);

	# update the res & used vm num.
	my $machlist = join(" ", @$machs);
	CmdUtils->runloadlcmd("llchres -R $request_info->{'request.ll_resid'} -h - $machlist");

	# Update UsedHypvNums & UsedKvmVms
	if ($hypv_flag != 0) {
		my $used = $hypv_flag == 1 ? 'UsedHypvNums' : 'UsedKvmVms';

		my $cfg = CloudUtils->getConfig();
		if (exists($cfg->{$used})) {
			$cfg->{$used} -= scalar(@$machs);
			CloudUtils->putConfig($cfg);
		}
		else {
			print"$used is empty in sysconfig table.\n";
		}
	}

	foreach my $mach (@$machs) {
		my $power_state;
		my $current_os;

		if ($trait->{'machines'}->{$mach}->{'mode'} eq 'share') {
			$power_state = 'on';
			$current_os = $machs_info->{$mach}->{'machine.default_os'};
		}
		else {
			$power_state = 'off';
			$current_os = undef;
		}

		my $attrs = {
			'hypv_flag' => $hypv_flag == -1 ? -1 : 0,
			'power_state' => $power_state,
			'current_os' => $current_os,
			'status' => 'Avail',
		};

		CloudUtils->updateMachByName($mach, $attrs);
	}

	return $rv;
}

sub parse_jobid {
	my ($outs) = @_;

	my $jobid;

	foreach my $out (@$outs) {
		if ($out =~ /\"(.+)\"/) {
			$jobid = $1;
			last;
		}
	}

	return $jobid;
}

sub makevm {
	my ($class, $req_info, $machs) = @_;

	my $reqid = $req_info->{'request.name'};
	my @nodes = keys(%{$req_info->{'machList'}});
	my $nodelist = join(",", @nodes);

	my $vmcpus = $req_info->{'request.machine_cpu'};
	my $vmmem  = $req_info->{'request.machine_memory'};
	my $vmdisk = $req_info->{'request.machine_disk'};
	if(defined($machs)){
		@nodes = @$machs;#modify add vm case
	}
	my ($rv, @outs) = CmdUtils->runxcatcmd("gettab key=master site.value");
	if ($rv != 0) {
		print("gettab failed\n");
		goto failed;
	}

	my $master = $outs[0];
	chomp($master);

	# Get vnode->hypv info.
	my $vhref = CloudUtils->getvNodeHypv($reqid, \@nodes);
	if (!$vhref) {
		print "Fail to get vnode->hyperv map.\n";
		goto failed;
	}

	# Define KVM vnode.
	foreach my $vnode (@nodes) {
		my $vmhost = $vhref->{$vnode};
		if (!$vmhost) {
			print "Fail to get vnode->hyperv map, $vnode.\n";
			goto failed;
		}

		($rv, @outs) = CmdUtils->runxcatcmd("mkdef $vnode arch=x86_64 groups=kvm,all installnic=mac primarynic=mac mgt=kvm netboot=xnba nfsserver=$master serialport=0 serialspeed=115200 vmcpus=$vmcpus vmhost=$vmhost vmmemory=$vmmem vmnics=br0 vmstorage=");
		if ($rv != 0) {
			print("mkdef failed\n");
			goto failed;
		}

		$rv = xCatUtils->updatexCathostTable($vnode);
		if($rv eq $GeneralDefine::FAILED){
			print "Fail to add $vnode IP into xCat hosts table.\n";
			goto failed;
		}
		print "Add $vnode IP into xCat hosts table Done.\n";
	}

	CmdUtils->runcmd("mkdir -p /install/vms", "root");

	return 0;
failed:
	return 1;
}

sub wait_request_ready {
	my ($class, $request_id, $timeout) = @_;
	my $request_info;
	my $status;

	for(my $sleeptime = 0; $sleeptime < $timeout; $sleeptime += 10) {
		$request_info = CloudUtils->getProvInfoById($request_id, 1);
		if (!$request_info) {
			print("CloudUtils->getProvInfoById failed\n");
			return 1;
		}

		$status = $request_info->{'request.status'};
		if ($status eq 'Ready') {
			last;
		}
		elsif (($status eq 'Failed') || ($status eq 'Expired') || ($status eq 'Canceled')) {
			print("Request $request_id has been $status\n");
			return 1;
		}
		else {
			sleep(10);
		}
	}

	return (0, $request_info);
}

sub modify_addusers {
	my ($class,$nodlist, $user) = @_;
	my $node_str = "";
	my $dst="/hpcc/bin";
	foreach my $node (@$nodlist) {
		$node_str.=$node.","
	}
	chop $node_str;
	print $node_str;

	my $cmd = "/opt/ibm/hpccloud/cloudbin/prov/modusr -add @$user";
	print "$cmd\n";
	my $xcatcmd = "xdsh $node_str -t 180 -e \"$cmd\" 2>&1";
	print "$xcatcmd\n";
	# Change the governor and it's parameters
	my ($rv, @outs) = CmdUtils->runxcatcmd($xcatcmd, "root");
	chomp (@outs);
	if ($rv != 0) {
	# handle error: log the error
		foreach (@outs) {
			print "$_\n";
		}
		return (1);
	}

	return (0);
}
		
sub modify_delusers{
	my ($class,$nodlist, $user) = @_;
	my $node_str = "";
	my $dst="/hpcc/bin";
	foreach my $node (@$nodlist) {
		$node_str.=$node.","
	}
	chop $node_str;
	print $node_str;

	my $cmd = "/opt/ibm/hpccloud/cloudbin/prov/modusr -sub @$user";
	print $cmd;
	my $xcatcmd = "xdsh $node_str -t 180 -e \"$cmd\" 2>&1";
	print "$xcatcmd\n";
	# Change the governor and it's parameters
	my ($rv, @outs) = CmdUtils->runxcatcmd($xcatcmd, "root");
	chomp (@outs);
	if ($rv != 0) {
	# handle error: log the error
		foreach (@outs) {
			print "$_\n";
		}
		return (1);
	} 
}
sub modify_addgroups{
	my ($class,$reqinfo, $groups) = @_;
	return (0);
}
sub modify_delgroups{
	my ($class,$reqinfo, $groups) = @_;
	return (0);
}
sub modify_reconfigSubLoadl{
	my ($class, $mnnod, $flag, $nodelist) = @_;
	my $node_str = "";
	my $dst="/hpcc/bin";
	print $node_str;

	my $cmd = "/opt/ibm/hpccloud/cloudbin/prov/modsubcluster ".($flag? "-add" : "-sub")." @$nodelist";
	my $xcatcmd = "xdsh $mnnod -t 180 -e \"$cmd\" 2>&1";
	print "$xcatcmd\n";
	# Change the governor and it's parameters
	my ($rv, @outs) = CmdUtils->runxcatcmd($xcatcmd, "root");
	chomp (@outs);
	if ($rv != 0) {
	# handle error: log the error
		foreach (@outs) {
			print "$_\n";
		}
		return (1);
	}

	return (0);
}
sub checkNodeBusy{
	my ($class, $dstnode) = @_;
	my $flag = 0;
	my $cmd ="llq -h $dstnode";
	my $indexstr = "llq: There is currently no job status to report.";
	print $cmd;
	my $xcatcmd = "xdsh $dstnode -t 180 -e \"$cmd\" 2>&1";
	print "$xcatcmd\n";
	my ($rv, @outs) = CmdUtils->runxcatcmd($xcatcmd, "root");
	chomp (@outs);
	if ($rv != 0) {
	# handle error: log the error
		foreach (@outs) {
			print "$_\n";
		}
		return (1, $flag);	
	}

	foreach (@outs) {
		my $ret = index($_,$indexstr);
		if ($ret != -1) {
			$flag = 1;
			last;
		} 
	}
	return (0, $flag);
}
		
sub checkNodeCM {
	my ($class, $dstnode, $ckname) = @_;
	my $flag = 0;
	my $cmd = "/opt/ibm/hpccloud/cloudbin/prov/getcmnode";
	my $xcatcmd = "xdsh $dstnode -t 180 -e \"$cmd\" 2>&1";
	print "$xcatcmd\n";
	# Change the governor and it's parameters
	my ($rv, @outs) = CmdUtils->runxcatcmd($xcatcmd, "root");

	chomp (@outs);
	if ($rv != 0) {
	# handle error: log the error
		foreach (@outs) {
			print "$_\n";
		}
		return (1, $flag);	
	}

	my $hostname = "";
	foreach (@outs) {
		if ($_ =~ /MNnode=(.+)/) {
			$hostname = $1;
			last;
		}
	}	   

	if(defined($ckname)){
		if ($hostname eq $ckname) {
			$flag = 1;
		}
	}else{
		if ($hostname eq $dstnode) {
			$flag = 1;
		}
	} 

	return (0, $flag);
}

sub update_request_energy {
	my ($class, $request_id) = @_;

	my $hash_mach_energy_ref;
	my $sum_energy = 0;

	my %hash_energy_value;

	# Check the thc_provision_machine_history table to accumulate the total energy used by the provition.
	$hash_mach_energy_ref = EnergyUtils->getEnergyFromProvMachLogByPM($request_id);
	if (!$hash_mach_energy_ref) {
		return ($GeneralDefine::FAILED, "getEnergyFromProvMachLogByPM does not return any matched records");
	}

	my $energyhash;
	foreach my $tmp_id (keys %$hash_mach_energy_ref) {
		$energyhash = $hash_mach_energy_ref->{$tmp_id};
		$sum_energy += $energyhash->{'provision_machine_history.energy'};
	}

	# Update thc_request.
	%hash_energy_value = ("energy" => $sum_energy);
	my $rc = CloudUtils->updateProvDynAttrById($request_id, \%hash_energy_value);
	if ($rc != 1) {
		return ($GeneralDefine::FAILED, "updateProvStatusById failed");
	}

	return $GeneralDefine::SUCCESS;
}

sub get_loadl_configuration {
	my ($class, $file_name, $key) = @_;
	my $sysconf = CloudUtils->getConfig();
	my $loadlconf = $sysconf->{'llAdminFile'};
	if(!defined($loadlconf) or ($loadlconf eq "")){
		$loadlconf = $GeneralDefine::LOADLCONFDIR;
	}
	if ($file_name!~ /^\//) {
		$file_name = "$loadlconf/$file_name";
	}
	if(!(-e $file_name)){
		return ($GeneralDefine::FAILED, "$file_name does not exist");
	}
	my $cmd="grep -i $key $file_name";
	my($rtv, @out) = CmdUtils->runcmd($cmd, "root");
	if($rtv != 0){
		return ($GeneralDefine::FAILED, "grep $key failed.");
	}
	my @returnValue=();
	foreach my $eachline (@out){
		chomp($eachline);
		$eachline =~ /=\s*(\d+)/;
		if(defined($1)){
			push (@returnValue, $1);
		}
	}
	return ($GeneralDefine::SUCCESS, @returnValue);
}
1;
