/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2010,2012          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* sccsid = "@(#) src/hpcc/webui/hpccloud/scripts/datePicker.js, hpc.cloud.ui, hpcc_rbei, rbeis002a 1.10 1/10/12 04:05:01"; */
	
/*
JavaScript date picker from Julian Robichaux: http://www.nsftools.com
*/

var datePickerDivID = "datepicker";
var iFrameDivID = "datepickeriframe";

var dayArrayShort = new Array('Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa');
var dayArrayMed = new Array('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
var dayArrayLong = new Array('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
var monthArrayShort = new Array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
var monthArrayMed = new Array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'June', 'July', 'Aug', 'Sept', 'Oct', 'Nov', 'Dec');
var monthArrayLong = new Array('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
 
// these variables define the date formatting we're expecting and outputting.
// If you want to use a different format by default, change the defaultDateSeparator
// and defaultDateFormat variables either here or on your HTML page.
var defaultDateSeparator = "/";		// common values would be "/" or "."
var defaultDateFormat = "mdy"	// valid values are "mdy", "dmy", and "ymd"
var dateSeparator = defaultDateSeparator;
var dateFormat = defaultDateFormat;

/**
This is the main function you'll call from the onClick event of a button.
Normally, you'll have something like this on your HTML page:

Start Date: <input name="StartDate">
<input type=button value="select" onclick="displayDatePicker('StartDate');">

That will cause the datepicker to be displayed beneath the StartDate field and
any date that is chosen will update the value of that field. If you'd rather have the
datepicker display beneath the button that was clicked, you can code the button
like this:

<input type=button value="select" onclick="displayDatePicker('StartDate', this);">

So, pretty much, the first argument (dateFieldName) is a string representing the
name of the field that will be modified if the user picks a date, and the second
argument (displayBelowThisObject) is optional and represents an actual node
on the HTML document that the datepicker should be displayed below.

In version 1.1 of this code, the dtFormat and dtSep variables were added, allowing
you to use a specific date format or date separator for a given call to this function.
Normally, you'll just want to set these defaults globally with the defaultDateSeparator
and defaultDateFormat variables, but it doesn't hurt anything to add them as optional
parameters here. An example of use is:

<input type=button value="select" onclick="displayDatePicker('StartDate', false, 'dmy', '.');">

This would display the datepicker beneath the StartDate field (because the
displayBelowThisObject parameter was false), and update the StartDate field with
the chosen value of the datepicker using a date format of dd.mm.yyyy
*/
function displayDatePicker(dateFieldName, displayBelowThisObject, dtFormat, dtSep)
{
  var targetDateField = document.getElementsByName (dateFieldName).item(0);
 
  // if we weren't told what node to display the datepicker beneath, just display it
  // beneath the date field we're updating
  if (!displayBelowThisObject)
	displayBelowThisObject = targetDateField;
 
  // if a date separator character was given, update the dateSeparator variable
  if (dtSep)
	dateSeparator = dtSep;
  else
	dateSeparator = defaultDateSeparator;
 
  // if a date format was given, update the dateFormat variable
  if (dtFormat)
	dateFormat = dtFormat;
  else
	dateFormat = defaultDateFormat;
 
  var x = displayBelowThisObject.offsetLeft;
  var y = displayBelowThisObject.offsetTop + displayBelowThisObject.offsetHeight ;
 
  // deal with elements inside tables and such
  var parent = displayBelowThisObject;
  while (parent.offsetParent) {
	parent = parent.offsetParent;
	x += parent.offsetLeft;
	y += parent.offsetTop ;
  }
 
  drawDatePicker(targetDateField, x, y);
}


/**
Draw the datepicker object (which is just a table with calendar elements) at the
specified x and y coordinates, using the targetDateField object as the input tag
that will ultimately be populated with a date.

This function will normally be called by the displayDatePicker function.
*/
function drawDatePicker(targetDateField, x, y) {
  var dt = getFieldDate(targetDateField.value );
 
  // the datepicker table will be drawn inside of a <div> with an ID defined by the
  // global datePickerDivID variable. If such a div doesn't yet exist on the HTML
  // document we're working with, add one.
  if (!document.getElementById(datePickerDivID)) {
	// don't use innerHTML to update the body, because it can cause global variables
	// that are currently pointing to objects on the page to have bad references
	//document.body.innerHTML += "<div id='" + datePickerDivID + "' class='dpDiv'></div>";
	var newNode = document.createElement("div");
	newNode.setAttribute("id", datePickerDivID);
	newNode.setAttribute("class", "dpDiv");
	newNode.setAttribute("style", "visibility: hidden;");
	document.body.appendChild(newNode);
  }
 
  // move the datepicker div to the proper x,y coordinate and toggle the visiblity
  var pickerDiv = document.getElementById(datePickerDivID);
  pickerDiv.style.position = "absolute";
  pickerDiv.style.left = x + "px";
  pickerDiv.style.top = y + "px";
  pickerDiv.style.visibility = (pickerDiv.style.visibility == "visible" ? "hidden" : "visible");
  pickerDiv.style.display = (pickerDiv.style.display == "block" ? "none" : "block");
  pickerDiv.style.zIndex = 10000;
 
  // draw the datepicker table
  refreshDatePicker(targetDateField.name, dt.getFullYear(), dt.getMonth(), dt.getDate());
}


/**
This is the function that actually draws the datepicker calendar.
*/
function refreshDatePicker(dateFieldName, year, month, day) {
  // if no arguments are passed, use today's date; otherwise, month and year
  // are required (if a day is passed, it will be highlighted later)
  var thisDay = new Date();
 
  if ((month >= 0) && (year > 0)) {
	thisDay = new Date(year, month, 1);
  } else {
	day = thisDay.getDate();
	thisDay.setDate(1);
  }
 
  // the calendar will be drawn as a table
  // you can customize the table elements with a global CSS style sheet,
  // or by hardcoding style and formatting elements below
  var crlf = "\r\n";
  var TABLE = "<table cols=7 class='dpTable'>" + crlf;
  var xTABLE = "</table>" + crlf;
  var TR = "<tr class='dpTR'>";
  var TR_title = "<tr class='dpTitleTR'>";
  var TR_days = "<tr class='dpDayTR'>";
  var TR_todaybutton = "<tr class='dpTodayButtonTR'>";
  var xTR = "</tr>" + crlf;
  var TD = "<td class='dpTD' onMouseOut='this.className=\"dpTD\";' onMouseOver=' this.className=\"dpTDHover\";' ";	// leave this tag open, because we'll be adding an onClick event
  var TD_title = "<td colspan=5 class='dpTitleTD'>";
  var TD_buttons = "<td class='dpButtonTD'>";
  var TD_todaybutton = "<td colspan=7 class='dpTodayButtonTD'>";
  var TD_days = "<td class='dpDayTD'>";
  var TD_selected = "<td class='dpDayHighlightTD' onMouseOut='this.className=\"dpDayHighlightTD\";' onMouseOver='this.className=\"dpTDHover\";' ";	// leave this tag open, because we'll be adding an onClick event
  var xTD = "</td>" + crlf;
  var DIV_title = "<div class='dpTitleText'>";
  var DIV_selected = "<div class='dpDayHighlight'>";
  var xDIV = "</div>";
 
  // start generating the code for the calendar table
  var html = TABLE;
 
  // this is the title bar, which displays the month and the buttons to
  // go back to a previous month or forward to the next month
  html += TR_title;
  html += TD_buttons + getButtonCode(dateFieldName, thisDay, -1, "&lt;") + xTD;
  html += TD_title + DIV_title + monthArrayLong[ thisDay.getMonth()] + " " + thisDay.getFullYear() + xDIV + xTD;
  html += TD_buttons + getButtonCode(dateFieldName, thisDay, 1, "&gt;") + xTD;
  html += xTR;
 
  // this is the row that indicates which day of the week we're on
  html += TR_days;
  for(i = 0; i < dayArrayShort.length; i++)
	html += TD_days + dayArrayShort[i] + xTD;
  html += xTR;
 
  // now we'll start populating the table with days of the month
  html += TR;
 
  // first, the leading blanks
  for (i = 0; i < thisDay.getDay(); i++)
	html += TD + "&nbsp;" + xTD;
 
  // now, the days of the month
  do {
	dayNum = thisDay.getDate();
	TD_onclick = " onclick=\"updateDateField('" + dateFieldName + "', '" + getDateString(thisDay) + "');\">";
	
	if (dayNum == day)
	  html += TD_selected + TD_onclick + DIV_selected + dayNum + xDIV + xTD;
	else
	  html += TD + TD_onclick + dayNum + xTD;
	
	// if this is a Saturday, start a new row
	if (thisDay.getDay() == 6)
	  html += xTR + TR;
	
	// increment the day
	thisDay.setDate(thisDay.getDate() + 1);
  } while (thisDay.getDate() > 1)
 
  // fill in any trailing blanks
  if (thisDay.getDay() > 0) {
	for (i = 6; i > thisDay.getDay(); i--)
	  html += TD + "&nbsp;" + xTD;
  }
  html += xTR;
 
  // add a button to allow the user to easily return to today, or close the calendar
  var today = new Date();
  var todayString = "Today is " + dayArrayMed[today.getDay()] + ", " + monthArrayMed[ today.getMonth()] + " " + today.getDate();
  html += TR_todaybutton + TD_todaybutton;
  html += "<button class='dpTodayButton' onClick='refreshDatePicker(\"" + dateFieldName + "\");'>This month</button> ";
  html += "<button class='dpTodayButton' onClick='updateDateField(\"" + dateFieldName + "\");'>Close</button>";
  html += xTD + xTR;
 
  // and finally, close the table
  html += xTABLE;
 
  document.getElementById(datePickerDivID).innerHTML = html;
  // add an "iFrame shim" to allow the datepicker to display above selection lists
  adjustiFrame();
}


/**
Convenience function for writing the code for the buttons that bring us back or forward
a month.
*/
function getButtonCode(dateFieldName, dateVal, adjust, label) {
  var newMonth = (dateVal.getMonth () + adjust) % 12;
  var newYear = dateVal.getFullYear() + parseInt((dateVal.getMonth() + adjust) / 12);
  if (newMonth < 0) {
	newMonth += 12;
	newYear += -1;
  }
 
  return "<button class='dpButton' onClick='refreshDatePicker(\"" + dateFieldName + "\", " + newYear + ", " + newMonth + ");'>" + label + "</button>";
}


/**
Convert a JavaScript Date object to a string, based on the dateFormat and dateSeparator
variables at the beginning of this script library.
*/
function getDateString(dateVal) {
  var dayString = "00" + dateVal.getDate();
  var monthString = "00" + (dateVal.getMonth()+1);
  dayString = dayString.substring(dayString.length - 2);
  monthString = monthString.substring(monthString.length - 2);
 
  switch (dateFormat) {
	case "dmy" :
	  return dayString + dateSeparator + monthString + dateSeparator + dateVal.getFullYear();
	case "ymd" :
	  return dateVal.getFullYear() + dateSeparator + monthString + dateSeparator + dayString;
	case "Mdy" :
	  return monthArrayShort[monthString-1] + " " + dayString + ", " + dateVal.getFullYear();
	case "mdy" :
	default :
	  return monthString + dateSeparator + dayString + dateSeparator + dateVal.getFullYear();
  }
}


/**
Convert a string to a JavaScript Date object.
*/
function getFieldDate(dateString) {
  var dateVal;
  var dArray;
  var d, m, y;
 
  try {
	dArray = splitDateString(dateString);
	if (dArray) {
	  switch (dateFormat) {
		case "dmy" :
		  d = parseInt(dArray[0], 10);
		  m = parseInt(dArray[1], 10) - 1;
		  y = parseInt(dArray[2], 10);
		  break;
		case "ymd" :
		  d = parseInt(dArray[2], 10);
		  m = parseInt(dArray[1], 10) - 1;
		  y = parseInt(dArray[0], 10);
		  break;
		case "mdy" :
		default :
		  d = parseInt(dArray[1], 10);
		  m = parseInt(dArray[0], 10) - 1;
		  y = parseInt(dArray[2], 10);
		  break;
	  }
	  dateVal = new Date(y, m, d);
	} else if (dateString) {
	  dateVal = new Date(dateString);
	} else {
	  dateVal = new Date();
	}
  } catch(e) {
	dateVal = new Date();
  }
 
  return dateVal;
}


/**
Try to split a date string into an array of elements, using common date separators.
If the date is split, an array is returned; otherwise, we just return false.
*/
function splitDateString(dateString) {
  var dArray;
  if (dateString.indexOf("/") >= 0)
	dArray = dateString.split("/");
  else if (dateString.indexOf(".") >= 0)
	dArray = dateString.split(".");
  else if (dateString.indexOf("-") >= 0)
	dArray = dateString.split("-");
  else if (dateString.indexOf("\\") >= 0)
	dArray = dateString.split("\\");
  else
	dArray = false;
 
  return dArray;
}

/**
Update the field with the given dateFieldName with the dateString that has been passed,
and hide the datepicker. If no dateString is passed, just close the datepicker without
changing the field value.

Also, if the page developer has defined a function called datePickerClosed anywhere on
the page or in an imported library, we will attempt to run that function with the updated
field as a parameter. This can be used for such things as date validation, setting default
values for related fields, etc. For example, you might have a function like this to validate
a start date field:

function datePickerClosed(dateField) {
  var dateObj = getFieldDate(dateField.value);
  var today = new Date();
  today = new Date(today.getFullYear(), today.getMonth(), today.getDate());
 
  if (dateField.name == "StartDate") {
	if (dateObj < today) {
	  // if the date is before today, alert the user and display the datepicker again
	  alert("Please enter a date that is today or later");
	  dateField.value = "";
	  document.getElementById(datePickerDivID).style.visibility = "visible";
	  adjustiFrame();
	} else {
	  // if the date is okay, set the EndDate field to 7 days after the StartDate
	  dateObj.setTime(dateObj.getTime() + (7 * 24 * 60 * 60 * 1000));
	  var endDateField = document.getElementsByName ("EndDate").item(0);
	  endDateField.value = getDateString(dateObj);
	}
  }
}

*/
function updateDateField(dateFieldName, dateString) {
  var targetDateField = document.getElementsByName (dateFieldName).item(0);
  if (dateString) {
	targetDateField.value = dateString;
	targetDateField.onchange();
  }
 
  var pickerDiv = document.getElementById(datePickerDivID);
  pickerDiv.style.visibility = "hidden";
  pickerDiv.style.display = "none";
 
  adjustiFrame();
  targetDateField.focus();
 
  // after the datepicker has closed, optionally run a user-defined function called
  // datePickerClosed, passing the field that was just updated as a parameter
  // (note that this will only run if the user actually selected a date from the datepicker)
  if ((dateString) && (typeof(datePickerClosed) == "function"))
	datePickerClosed(targetDateField);
}


/**
Use an "iFrame shim" to deal with problems where the datepicker shows up behind
selection list elements, if they're below the datepicker. The problem and solution are
described at:

http://dotnetjunkies.com/WebLog/jking/archive/2003/07/21/488.aspx
http://dotnetjunkies.com/WebLog/jking/archive/2003/10/30/2975.aspx
*/
function adjustiFrame(pickerDiv, iFrameDiv) {
  // we know that Opera doesn't like something about this, so if we
  // think we're using Opera, don't even try
  var is_opera = (navigator.userAgent.toLowerCase().indexOf("opera") != -1);
  if (is_opera)
	return;
  
  // put a try/catch block around the whole thing, just in case
  try {
	if (!document.getElementById(iFrameDivID)) {
	  // don't use innerHTML to update the body, because it can cause global variables
	  // that are currently pointing to objects on the page to have bad references
	  //document.body.innerHTML += "<iframe id='" + iFrameDivID + "' src='javascript:false;' scrolling='no' frameborder='0'>";
	  var newNode = document.createElement("iFrame");
	  newNode.setAttribute("id", iFrameDivID);
	  newNode.setAttribute("src", "javascript:false;");
	  newNode.setAttribute("scrolling", "no");
	  newNode.setAttribute ("frameborder", "0");
	  document.body.appendChild(newNode);
	}
	
	if (!pickerDiv)
	  pickerDiv = document.getElementById(datePickerDivID);
	if (!iFrameDiv)
	  iFrameDiv = document.getElementById(iFrameDivID);
	
	try {
	  iFrameDiv.style.position = "absolute";
	  iFrameDiv.style.width = pickerDiv.offsetWidth;
	  iFrameDiv.style.height = pickerDiv.offsetHeight ;
	  iFrameDiv.style.top = pickerDiv.style.top;
	  iFrameDiv.style.left = pickerDiv.style.left;
	  iFrameDiv.style.zIndex = pickerDiv.style.zIndex - 1;
	  iFrameDiv.style.visibility = pickerDiv.style.visibility ;
	  iFrameDiv.style.display = pickerDiv.style.display;
	} catch(e) {
	}
 
  } catch (ee) {
  }
 
}

function updateEndFromDuration() {
	var duration = document.getElementById("duration");

	// Make sure there is a value to use for the update
	if (! duration.value) return true;
	// get the start date and time
	var startDate = document.getElementById("start_mon|start_day|start_year");
	var startHour = document.getElementById("start_hour");
	var startMin = document.getElementById("start_min");
	if ((! startDate.value)) return true;

	// parsing Start values and verify
	var dateString = new String(startDate.value);
	var splitDate=dateString.split("/");

	if (parseInt(splitDate[2]) >= 2038) {
		alert("The start date must be earlier than 2038.");
		return false;
	}

	// creating date from them -- Date(year, month -1, day)
	var myDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],startHour.value,startMin.value);
	var nowTime = document.getElementById("now_time");
	if (nowTime) {
		dateString = new String(nowTime.value);
		splitDate = dateString.split("/");
		var myNowDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],splitDate[3],splitDate[4]);
		var earlier = myNowDate.getTime() - myDate.getTime();
		if (earlier > 0) {
			alert("The start date must be later than now.");
			return false;
		}
	}

	var endDate = document.getElementById("end_mon|end_day|end_year");
	var endHour = document.getElementById("end_hour");
	var endMin = document.getElementById("end_min");

	// turning the duration minutes duration into millis
	durationMillis= duration.value * 60 * 1000;
	// add start date and time to duration
	myDate.setTime(myDate.getTime() + durationMillis);
	endHour.value = myDate.getHours();
	endMin.value = myDate.getMinutes();
	endDate.value = (myDate.getMonth()+1) + "/" + myDate.getDate() + "/" + myDate.getFullYear() ;
 
	return true;
}

function updateDurationFromEnd() {
	var duration = document.getElementById("duration");
	duration.value = "";

	var endDate = document.getElementById("end_mon|end_day|end_year");
	var endHour = document.getElementById("end_hour");
	var endMin = document.getElementById("end_min");
	if ( endDate.value) { 
		var startDate = document.getElementById("start_mon|start_day|start_year");
		var startHour = document.getElementById("start_hour");
		var startMin = document.getElementById("start_min");
		// Don't continue with update to duration if any of these are true
		if (! startDate.value) return true;
		if (startDate.value > endDate.value) {
			alert("The end date must be later than its start date.");
			return false;
		}
	
		// Parse date mm/dd/yyyy and time HH:mm strings
		var dateString = new String(startDate.value);
		var splitDate=dateString.split("/");
		// myDate is start Date and Time
		var myDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],startHour.value,startMin.value);

		// do same to get endDate
		dateString = new String(endDate.value);
		splitDate=dateString.split("/");
		if (parseInt(splitDate[2]) >= 2038) {
			alert("The end date must be earlier than 2038.");
			return false;
		}

		var myEndDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],endHour.value,endMin.value);
		// the difference is the duration in millis
		var durationMillis = myEndDate.getTime() - myDate.getTime();
		var durationMins = durationMillis / (60 * 1000);
		
		// update duration if we have a positive integer
		if (durationMins > 0)
			duration.value = durationMins ;
		else {
			alert("The end time must be later than its start time.");
			return false;
		}
	}
	return true;
}
function checkReqFormNext() {
	var temp = document.getElementsByName("osimage");
	if(temp.length!=0) {
		return true;
	}
	alert("no image is selected");
	return false;
}
function checkReqForm() {
	var machSelect = document.getElementById("machList");
	if (machSelect) {
		var empty = true;
		for(var i=0; i < machSelect.length; i++) {
			if (machSelect.options[i].selected) {
				empty = false;
				break;
			}
		}
		if (empty) {
			alert("Select some machines.");
			return false;
		}
	}

	var startDate = document.getElementById("start_mon|start_day|start_year");
	var startHour = document.getElementById("start_hour");
	var startMin = document.getElementById("start_min");

	// Parse date mm/dd/yyyy and time HH:mm strings
	var dateString = new String(startDate.value);
	var splitDate=dateString.split("/");
	if (parseInt(splitDate[2]) >= 2038) {
		alert("The start date must be earlier than 2038.");
		return false;
	}
	// myDate is start Date and Time
	var myDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],startHour.value,startMin.value);

	var endDate = document.getElementById("end_mon|end_day|end_year");
	dateString = new String(endDate.value);
	splitDate = dateString.split("/");
	if (parseInt(splitDate[2]) >= 2038) {
		alert("The end date must be earlier than 2038.");
		return false;
	}

	var nowTime = document.getElementById("now_time");
	if (nowTime) {
		dateString = new String(nowTime.value);
		splitDate = dateString.split("/");
		var myNowDate = new Date(splitDate[2],splitDate[0]-1,splitDate[1],splitDate[3],splitDate[4]);
		var earlier = myNowDate.getTime() - myDate.getTime();
		if (earlier > 0) {
			alert("The start date must be later than now.");
			return false;
		}
	}

	var duration = document.getElementById("duration");
	if (! duration.value) {
		alert("The duration or end date is invalid.");
		return false;
	}

	var test_duration = duration.value;
	if (! /^[0-9]+$/.test(test_duration)) {
		alert("The duration " + test_duration + " is invalid");
		return false;
	}
	showImages();
	return true;
}
function showImages() {
	var cmdstr = " --status=AVAILABLE"; 
	if( document.getElementById('subcluster').checked ){cmdstr = cmdstr + "  --scheduler="+document.getElementById('scheduler_select').value; }
	if( document.getElementById('setuppe').checked ){cmdstr = cmdstr + " --parallelenv="+document.getElementById('environment_select').value; }
	cmdstr = cmdstr + " --interconnect="+document.getElementById('network_select').value ;
	//cmdstr = cmdstr + " --sharedfs="+document.getElementById('filesystem_select').value ;
	if (!document.getElementById('filesystem_select').value &&
		document.getElementById('filesystem_select').value != "NA"){
			cmdstr = cmdstr + " --sharedfs="+document.getElementById('filesystem_select').value ;
	}

	var xhrArgs = {

	url: "GetInfoXML.cgi?function=listImageXML&cmdstr="+cmdstr,
	handleAs: "xml",
		preventCache: true,
		load: function(data) {
			onImageUpdate(data);
		},
		error: function(error) {
			showInfo("An unexpected error occurred: " + error);
		}
	}

	//Call the asynchronous xhrGet
	var deferred = dojo.xhrGet(xhrArgs);
}

function onImageUpdate(xmlDoc) {
	var imageData = parseImageXML(xmlDoc);
	clearImageTab();
	var Is_empty = true;
	var theTable = document.getElementById("reqmimaglist_tab");
	for (var i=0; i < imageData.images.length; ++i) {
		var row = theTable.insertRow(theTable.rows.length);
		row.className = "tab_content";
		var cell1 = row.insertCell(0);
		if(imageData.images[i].status=="AVAILABLE") {
			Is_empty = false;
			if (i == 0){
				cell1.innerHTML = "<td><input type=radio name=osimage value=" + imageData.images[i].name + " checked ></td>";
			} else {
				cell1.innerHTML = "<td><input type=radio name=osimage value=" + imageData.images[i].name + "></td>";
			}
		}
		var cell2 = row.insertCell(1);
		cell2.innerHTML = "<td>" + imageData.images[i].name + " </td>";
		var cell3 = row.insertCell(2);
		var logo = "";
		var base = imageData.images[i].base.toLowerCase();
		if (base.indexOf("rhel") != -1) {
			logo = "<img src=\"/hpccloud/images/redhat_logo.png\" height=30 width=30>";
			base = "RedHat Enterprise Linux";
		} else if (base.indexOf("redhat") != -1) {
			logo = "<img src=\"/hpccloud/images/redhat_logo.png\" height=30 width=30>";
			base = "RedHat Enterprise Linux";
		} else if (base.indexOf("sles") != -1) {
			logo = "<img src=\"/hpccloud/images/suse_logo.png\" height=30 width=30>";
			base = "SuSE Enterprise Linux";
		} else if (base.indexOf("suse") != -1) {
			logo = "<img src=\"/hpccloud/images/suse_logo.png\" height=30 width=30>";
			base = "SuSE Enterprise Linux";
		} else if (base.indexOf("imagex") != -1) {
			logo = "<img src=\"/hpccloud/images/windows_logo.png\" height=30 width=30>";
			base = "Microsoft Windows";
		} else if (base.indexOf("microsoft") != -1) {
			logo = "<img src=\"/hpccloud/images/windows_logo.png\" height=30 width=30>";
			base = "Microsoft Windows";
		} else if (base.indexOf("ms") != -1) {
			logo = "<img src=\"/hpccloud/images/windows_logo.png\" height=30 width=30>";
			base = "Microsoft Windows";
		} else if (base.indexOf("windows") != -1) {
			logo = "<img src=\"/hpccloud/images/windows_logo.png\" height=30 width=30>";
			base = "Microsoft Windows";
		} else if (base.indexOf("centos") != -1) {
			logo = "<img src=\"/hpccloud/images/centos_logo.png\" height=30 width=30>";
			base = "CentOS";
		} else {
			base = imageData.images[i].base;
			logo = "";
		}

		cell3.innerHTML = "<td><table><tr><td>" + logo +"</td><td>" + base + "</td></tr></table></td>";

		var cell4 = row.insertCell(3);
		cell4.innerHTML = imageData.images[i].arch;
		var cell5 = row.insertCell(4);
		cell5.innerHTML = imageData.images[i].infrastructure;;
		var cell6 = row.insertCell(5);
		cell6.innerHTML = imageData.images[i].description;
	   // var cell7 = row.insertCell(6);
	   //var cell7 = row.insertCell(6);
	}
	if(Is_empty) {
		var row = theTable.insertRow(theTable.rows.length);
		row.innerHTML = "<th colspan=\"6\" align=\"left\">There is no matched image.</th>";
	}
}

function parseImageXML(xmlDoc) {
	var result = new Object();
	result.images = [];
	var infra_item = ["parallel_env","scheduler","shared_fs","interconnect"];
	var items = xmlDoc.documentElement.getElementsByTagName('image');
	for(var n=0; n < items.length; n++) {
		var image = new Object();
		image.name = getElementDataFromNode(items[n], "name");
		image.arch = getElementDataFromNode(items[n], "arch");
		image.base = getElementDataFromNode(items[n], "base");
		image.status = getElementDataFromNode(items[n], "status");
		image.description = getElementDataFromNode(items[n], "description");
		//image.infrastructure = getElementDataFromNode(items[n], "parallel_env")+","+getElementDataFromNode(items[n], "scheduler")+","
		//	+getElementDataFromNode(items[n], "shared_fs")+","+getElementDataFromNode(items[n], "interconnect");
		image.infrastructure = "";
		for(var i=0; i<infra_item.length; i++){
			if(getElementDataFromNode(items[n], infra_item[i]) != ""){
				if(image.infrastructure == ""){
					image.infrastructure = getElementDataFromNode(items[n], infra_item[i]);
				}else{
					image.infrastructure += ','+getElementDataFromNode(items[n], infra_item[i]);
				}
			}
		}
		result.images.push(image);
	}

	return result;
}

function clearImageTab() {
	var theTable = document.getElementById("reqmimaglist_tab");
	for(var i = (theTable.rows.length-1); theTable.rows.length > 2; i--) {
		theTable.deleteRow(i);
	}
}
